/*
 * CarbX, except for the proprietary licensed third party source code in human
 * and/or machine-readable form, is free software. The third-party source code is
 * not shipped in human and/or machine-readable form along with these source
 * files. If you build CarbX from the provided CarbX source code files while
 * obtaining the required libraries, files and third party source code as well as
 * the required licences on your own, you can redistribute and/or modify the
 * remaining parts of CarbX (i. e. the parts developed by Torben Pfaff) under the
 * terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version. This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details. You should have received a copy of the GNU Lesser General
 * Public License along with this program. If not, see http://www.gnu.org/licenses/.
 */

#ifndef AUTOFITSETTINGS_H
#define AUTOFITSETTINGS_H

#include <QBrush>
#include <QCloseEvent>
#include <QColor>
#include <QDir>
#include <QFile>
#include <QFileInfoList>
#include <QKeyEvent>
#include <QListWidgetItem>
#include <QMessageBox>
#include <QRegExp>
#include <QStringList>

#include "autofitmathematica.h" //leave out if "wstp.h" is not available
#include "autofitsequence.h"
#include "autofitstep.h"
#include "auxiliarymethods.h"
#include "datastorage.h"
#include "dialog_name.h"
#include "enumerations.h"
#include "parameterfileaccess.h"
#include "settingsstorage.h"

#include <QMainWindow>


class MainWindow;


namespace Ui {
    class AutoFitSettings;
}


class AutoFitSettings : public QMainWindow{


    Q_OBJECT


public:

    explicit AutoFitSettings(DataStorage *dst = 0, SettingsStorage *sst = 0, ParameterFileAccess *pfa = 0, MainWindow *mw = 0, QWidget *parent = 0);
    ~AutoFitSettings();


private:

    Ui::AutoFitSettings *ui;

    DataStorage *dst;
    SettingsStorage *sst;
    ParameterFileAccess *pfa;
    MainWindow *mw;

    QBrush *brush_activeStep;    //brush for active step
    QBrush *brush_inactiveStep;  //brush for inactive step

    AutoFitSequence *afseq; //AutoFitSequence loaded and modifiable by user

    short activeParameters;     //number of active parameters in step, must be >= 1
    short activeSteps;          //number of active steps             , must be >= 1
    bool  must_takeOverResults;
    bool  must_saveSequence;

    void setDefaultAutoFitSequence();  //set afseq to default AutoFitSequence (1 step, all parameters); backup if no file is found on hard disk
    void setParameterWidgetValues(); //sets the values of the lineEdits for the parameter values according to dst->parsAutoFit
    bool existsSavePath(QString *name); //check if savePath for name of certain AutoFitSequence already exists, needed for adding/renamaning AutoFitSequences
    void saveAutoFitSequenceMaybe(); //ask if user wants to save sequence on chaning sequence / adding a new sequence

    //Events
    void keyPressEvent(QKeyEvent *event);
    void closeEvent(QCloseEvent *event);

private slots:

    void on_scrollBarControls_valueChanged(int value);

    void on_cno_clicked(bool checked);
    void on_button_cnoValueTakeover_clicked();
    void on_cnoMin_editingFinished();
    void on_cnoMax_editingFinished();

    void on_mu_clicked(bool checked);
    void on_button_muValueTakeover_clicked();
    void on_muMin_editingFinished();
    void on_muMax_editingFinished();

    void on_beta_clicked(bool checked);
    void on_button_betaValueTakeover_clicked();
    void on_betaMin_editingFinished();
    void on_betaMax_editingFinished();

    void on_a3_clicked(bool checked);
    void on_button_a3ValueTakeover_clicked();
    void on_a3Min_editingFinished();
    void on_a3Max_editingFinished();

    void on_da3_clicked(bool checked);
    void on_button_da3ValueTakeover_clicked();
    void on_da3Min_editingFinished();
    void on_da3Max_editingFinished();

    void on_sig3_clicked(bool checked);
    void on_button_sig3ValueTakeover_clicked();
    void on_sig3Min_editingFinished();
    void on_sig3Max_editingFinished();

    void on_u3_clicked(bool checked);
    void on_button_u3ValueTakeover_clicked();
    void on_u3Min_editingFinished();
    void on_u3Max_editingFinished();

    void on_eta_clicked(bool checked);
    void on_button_etaValueTakeover_clicked();
    void on_etaMin_editingFinished();
    void on_etaMax_editingFinished();

    void on_nu_clicked(bool checked);
    void on_button_nuValueTakeover_clicked();
    void on_nuMin_editingFinished();
    void on_nuMax_editingFinished();

    void on_alpha_clicked(bool checked);
    void on_button_alphaValueTakeover_clicked();
    void on_alphaMin_editingFinished();
    void on_alphaMax_editingFinished();

    void on_sig1_clicked(bool checked);
    void on_button_sig1ValueTakeover_clicked();
    void on_sig1Min_editingFinished();
    void on_sig1Max_editingFinished();

    void on_lcc_clicked(bool checked);
    void on_button_lccValueTakeover_clicked();
    void on_lccMin_editingFinished();
    void on_lccMax_editingFinished();

    void on_q_clicked(bool checked);
    void on_button_qValueTakeover_clicked();
    void on_qMin_editingFinished();
    void on_qMax_editingFinished();

    void on_dan_clicked(bool checked);
    void on_button_danValueTakeover_clicked();
    void on_danMin_editingFinished();
    void on_danMax_editingFinished();

    void on_k_clicked(bool checked);
    void on_button_kValueTakeover_clicked();
    void on_kMin_editingFinished();
    void on_kMax_editingFinished();

    void on_const1_clicked(bool checked);
    void on_button_const1ValueTakeover_clicked();
    void on_const1Min_editingFinished();
    void on_const1Max_editingFinished();

    void on_const2_clicked(bool checked);
    void on_button_const2ValueTakeover_clicked();
    void on_const2Min_editingFinished();
    void on_const2Max_editingFinished();

    void on_g_clicked(bool checked);
    void on_button_gValueTakeover_clicked();
    void on_gMin_editingFinished();
    void on_gMax_editingFinished();

    void on_buttonTakeOverResults_clicked();
    void on_buttonDisplayResults_clicked(bool checked);
    void on_buttonTakeOverValues_clicked();
    void on_buttonDoAutoFit_clicked();

    //AutoFit sequences
    void on_comboBox_AutoFitSequences_currentIndexChanged(int index);
    void on_buttonAddAutoFitSequence_clicked();
    void on_buttonSaveAutoFitSequence_clicked();
    void on_buttonRenameAutoFitSequence_clicked();
    void on_buttonDeleteAutoFitSequence_clicked();

    //AutoFit steps
    void on_listWidget_AutoFitSteps_currentRowChanged(int currentRow);
    void on_buttonMoveAutoFitStepUp_clicked();
    void on_buttonMoveAutoFitStepDown_clicked();
    void on_buttonAddAutoFitStep_clicked();
    void on_buttonRenameAutoFitStep_clicked();
    void on_buttonDeleteAutoFitStep_clicked();
    void on_buttonDisableAutoFitStep_clicked();
    void on_buttonExecuteAutoFitStep_clicked();

    //additional Conditions
    void on_buttonAddAdditonalCondition_clicked();
    void on_buttonChangeAdditonalCondition_clicked();
    void on_buttonDeleteAdditonalCondition_clicked();

};

#endif // AUTOFITSETTINGS_H
