/*
 * CarbX, except for the proprietary licensed third party source code in human
 * and/or machine-readable form, is free software. The third-party source code is
 * not shipped in human and/or machine-readable form along with these source
 * files. If you build CarbX from the provided CarbX source code files while
 * obtaining the required libraries, files and third party source code as well as
 * the required licences on your own, you can redistribute and/or modify the
 * remaining parts of CarbX (i. e. the parts developed by Torben Pfaff) under the
 * terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version. This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details. You should have received a copy of the GNU Lesser General
 * Public License along with this program. If not, see http://www.gnu.org/licenses/.
 */

#ifndef CALCULATIONS_H
#define CALCULATIONS_H

#include <cmath>
#include <complex>

#include <QFile>
#include <QPointF>
#include <QString>
#include <QStringList>
#include <QTextStream>
#include <QVector>

extern "C"{
    #include "z_numrecip/nr.h"
    #include "z_numrecip/nrcomplex.h"
}

using namespace std;

class Calculations{

private:

    float *IcomC_x; //array for interpolation
    float *IcomC_y; //array for interpolation
    int sizeOf_IcomC; //size of array

    float *fN_x; //array for interpolation
    float *fN_y; //array for interpolation
    int sizeOf_fN; //size of array

    float *fO_x; //array for interpolation
    float *fO_y; //array for interpolation
    int sizeOf_fO; //size of array

public:

    //constructor including initialization of interpolation functions
    Calculations(QString *dataPath = 0);

    //destructor including release of memory
    ~Calculations();


    //hypergeometrical function 2F1
    complex<double> hypgeo2F1(double a, double b, double c, complex<double> z);


    //interpolation of the form factor of carbon
    double fCpara(double s);

    //interpolation of the form factor of carbon with respect to anisotropy
    double fCperp(double dan, double s);

    //interpolation of the form factor of nitrogen
    double fN(double s);

    //interpolation of the form factor of oxygen
    double fO(double s);


    //incoherent scattering

    double IcomC(double s);
    double Recoil(double s, double wavelength);
    double Qabs(double s, double wavelength);
    double Dq(double s);
    double Dlam(double s, double wavelength);
    double Dlamc(double s, double wavelength);

    //attenuation function Q for compton scattering; due to secondary monochromation, pass-band b [angstr.]
    double Q(double b, double s, double wavelength);

    double IincC(bool useQ, double b, double s, double wavelength);


    double P_(double s, double wavelength); //polarization factor P (P_ to avoid problems with a C macro in wstp.h)


    //I_intra - scattering from the layers

    double shk(int h, int k, double lcc);
    double g(double q, double lcc, int h, int k, double s);
    double g0(double q);
    complex<double> F(double m, double nu, double lm, double lcc, double sig1, int h, int k, double s);

    //analytical expression for J_hk without epsilon_ 1
    double Jhk(double nu, double lm, double lcc, double sig1, int h, int k, double q, double s);
    int Jhk_prefactor(int h, int k); //prefactor for J_hk
    double JhkXprefactor(double nu, double lm, double lcc, double sig1, int h, int k, double q, double s); // J_hk times prefactor

    double n0S0(double lcc);
    double Iintra(double nu, double lm, double lcc, double sig1, double q, double s, double wavelength);


    //I_inter - scattering from the stacking of layers

    complex<double> H(double a3min, double da3, double sig3, double s);
    complex<double> HN(double mu, double Nm, double a3min, double da3, double sig3, double s);
    double I1DI0(double mu, double Nm, double a3min, double da3, double sig3, double s);
    double I0(double mu, double Nm, double a3min, double da3, double s);
    double I1D(double mu, double Nm, double a3min, double da3, double sig3, double s);
    double DD(double u3, double s);
    double Iinter(double mu, double Nm, double a3min, double da3, double sig3, double u3, double eta, double lcc, double q, double s);


    //I_coh - coherent scattering
    double Icoh(double cno, double mu, double Nm, double a3min, double da3, double sig3, double u3, double eta, double nu, double lm , double lcc, double sig1, double q, double cN, double cO, double dan, double s, double wavelength);


    //I_obs - theoretical observed intensity

    //I_fit - fit function
    double Ifit(bool useQ, double b, double k, double cno, double mu, double Nm, double a3min, double da3, double sig3, double u3, double eta, double nu, double lm, double lcc, double sig1, double q, double cN, double cO, double dan, double s, double wavelength);

    //calculation of absorption factor A as function of z = 2 theta [degrees]
    double A(double z, double d, double mue_ab);

    //exponential term for tuning of gradient/slope as function of s using parameter g
    double T(double g, double s);

    //conversion factor corrAutoColl for correction of automatic collimator
    //use to convert intensity for fixed colliminator to intensity for automatic collimator
    //needs radius of diffractometer r, divergence angle in degrees, irradiated length l, wavelength and scattering vector s
    double corrAutoColl(double r, double delta, double l, double wavelength, double s);


    //data correction

    //calculation of mut from thickness d of sample and absorption factor mue_ab
    double mut(double d, double mue_ab);

    //calculation of muR from radius of diffractometer r and absorption factor mue_ab
    double muR(double r, double mue_ab);

    //calculation of Dz as function of z = 2 theta [degrees]
    double Dz(double muR , double mut , double z);

    //calculation of intensity positions via D2thtx as function of i = position/index of intenstiy in QVector<QPointF> data
    double PositionByD2thtx(double R, double mue_ab, double t, int i);

    //calculation of opacity for intensity correction as function of z = 2 theta [degrees]
    double opacity(double d, double mue_ab, double z);

    //calculation of the pragmatic approach for intensity correction as function of z = 2 theta [degrees]
    double pragmatic(double p, double z);


    //auxiliary functions

    //hyperbolic cotangent
    double coth(double x);

    //cosecant
    double csc(double x);

};

#endif // CALCULATIONS_H
