/*
 * CarbX, except for the proprietary licensed third party source code in human
 * and/or machine-readable form, is free software. The third-party source code is
 * not shipped in human and/or machine-readable form along with these source
 * files. If you build CarbX from the provided CarbX source code files while
 * obtaining the required libraries, files and third party source code as well as
 * the required licences on your own, you can redistribute and/or modify the
 * remaining parts of CarbX (i. e. the parts developed by Torben Pfaff) under the
 * terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version. This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details. You should have received a copy of the GNU Lesser General
 * Public License along with this program. If not, see http://www.gnu.org/licenses/.
 */

#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QBrush>
#include <QCloseEvent>
#include <QColor>
#include <QDesktopWidget>
#include <QMessageBox>
#include <QPixmap>
#include <QPen>
#include <QPoint>
#include <QResizeEvent>
#include <QSize>
#include <QString>
#include <QStringList>
#include <QTextStream>

#include <windows.h>

#include "autofitsettings.h"
#include "auxiliarymethods.h"
#include "datacalculations.h"
#include "datastorage.h"
#include "dialog_choosetemplate.h"
#include "dialog_fileconversion.h"
#include "dialog_preferences.h"
#include "deviation.h"
#include "enumerations.h"
#include "parameterfileaccess.h"
#include "parameterset.h"
#include "settingsstorage.h"
#include "structs.h"

#include <qwt_curve_fitter.h>
#include <qwt_plot.h>
#include <qwt_plot_curve.h>
#include <qwt_plot_grid.h>
#include <qwt_series_data.h>
#include <qwt_symbol.h>

#include <QMainWindow>

namespace Ui {
    class MainWindow;
}

class MainWindow : public QMainWindow{

    Q_OBJECT

public:

    explicit MainWindow(QWidget *parent = 0);
    ~MainWindow();

    //method needed for interaction with deviation window
    void uncheck_buttonShowDeviation();

    //method needed for interaction with AutoFitSettings window
    void display_afsResults(bool displayAutoFitResults);
    void takeOver_afsResults();
    void on_close_afs();

private:

    Ui::MainWindow *ui; //generated by Qt

    //own private variables
    DataStorage      *dst;
    SettingsStorage  *sst;
    DataCalculations *datacal;
    ParameterFileAccess *pfa;
    Deviation *dev; //pointer to deviation window
    AutoFitSettings *afs;


    //variables needed for plot
    QwtPlotGrid *grid;

    QColor *colorData;
    QBrush *brushData;
    QPen *penData;
    QSize *sizeData;
    QwtSymbol *symbolData;
    QwtPlotCurve *curveData;

    QColor *colorFit;
    QPen *penFit;
    QwtPlotCurve *curveFit;


    //own private methods

    void setSize(); //adjustment of size
    void applyProgramSettings(bool startup);
    void update__comboBox_Wavelengths(int index); //updates entries/items and choses entry/item with "index"
    void set_slider_values(); //sets the values of sliders and corresponding editFields
    void set_widget_values(bool startup); //sets the values of other widgets (checkBoxes and editFields)
    void check_uncheck_buttons(); //checks/unchecks the button draw... and showDeviation and performs corresponding action
    void setEnabledDisabled(); //only enable relevant parameters for chosen fit-options and data-correction
    void setWindowCaption(); //set WindowTitle according to parsPath
    void chooseTemplate(); //let the user choose a template and load it
    void loadTemplate(short templateNumber); //loads templates with given index and sets slider/widget values accordingly
    bool saveMaybe();

    //draws/updates data and fit curve
    void draw(bool mustUpdateDeviation);

    //attaches/detaches curves in correct order and corresponding to "drawData" and "drawFit"
    //applies settings of thickness
    void updateCurves();

    //Events
    void resizeEvent(QResizeEvent *event);
    void closeEvent(QCloseEvent *event);

    //Slider-Functionality (private methods)
    void set_cno_min_max();
    void set_mu_min_max();
    void set_beta_min_max();
    void set_a3_min_max();
    void set_da3_min_max();
    void set_sig3_min_max();
    void set_u3_min_max();
    void set_eta_min_max();
    void set_nu_min_max();
    void set_alpha_min_max();
    void set_sig1_min_max();
    void set_lcc_min_max();
    void set_q_min_max();
    void set_cN_min_max();
    void set_cO_min_max();
    void set_dan_min_max();
    void set_k_min_max();
    void set_const1_min_max();
    void set_const2_min_max();
    void set_g_min_max();
    void set_mue_ab_min_max();
    void set_p_min_max();
    void set_R_min_max();
    void set_t_min_max();


private slots:

    //Slider-Functionality (slots)

    //cno
    void on_cno_max_editingFinished();
    void on_cno_min_editingFinished();
    void on_cno_sub_editingFinished();
    void on_cno_add_editingFinished();
    void on_cno_value_editingFinished();
    void on_cno_doSub_clicked();
    void on_cno_doAdd_clicked();
    void on_cno_valueChanged(int value);

    //mu
    void on_mu_max_editingFinished();
    void on_mu_min_editingFinished();
    void on_mu_sub_editingFinished();
    void on_mu_add_editingFinished();
    void on_mu_value_editingFinished();
    void on_mu_doSub_clicked();
    void on_mu_doAdd_clicked();
    void on_mu_valueChanged(int value);

    //beta
    void on_beta_max_editingFinished();
    void on_beta_min_editingFinished();
    void on_beta_sub_editingFinished();
    void on_beta_add_editingFinished();
    void on_beta_value_editingFinished();
    void on_beta_doSub_clicked();
    void on_beta_doAdd_clicked();
    void on_beta_valueChanged(int value);

    //a3
    void on_a3_max_editingFinished();
    void on_a3_min_editingFinished();
    void on_a3_sub_editingFinished();
    void on_a3_add_editingFinished();
    void on_a3_value_editingFinished();
    void on_a3_doSub_clicked();
    void on_a3_doAdd_clicked();
    void on_a3_valueChanged(int value);

    //da3
    void on_da3_max_editingFinished();
    void on_da3_min_editingFinished();
    void on_da3_sub_editingFinished();
    void on_da3_add_editingFinished();
    void on_da3_value_editingFinished();
    void on_da3_doSub_clicked();
    void on_da3_doAdd_clicked();
    void on_da3_valueChanged(int value);

    //sig3
    void on_sig3_max_editingFinished();
    void on_sig3_min_editingFinished();
    void on_sig3_sub_editingFinished();
    void on_sig3_add_editingFinished();
    void on_sig3_value_editingFinished();
    void on_sig3_doSub_clicked();
    void on_sig3_doAdd_clicked();
    void on_sig3_valueChanged(int value);

    //u3
    void on_u3_max_editingFinished();
    void on_u3_min_editingFinished();
    void on_u3_sub_editingFinished();
    void on_u3_add_editingFinished();
    void on_u3_value_editingFinished();
    void on_u3_doSub_clicked();
    void on_u3_doAdd_clicked();
    void on_u3_valueChanged(int value);

    //eta
    void on_eta_max_editingFinished();
    void on_eta_min_editingFinished();
    void on_eta_sub_editingFinished();
    void on_eta_add_editingFinished();
    void on_eta_value_editingFinished();
    void on_eta_doSub_clicked();
    void on_eta_doAdd_clicked();
    void on_eta_valueChanged(int value);

    //nu
    void on_nu_max_editingFinished();
    void on_nu_min_editingFinished();
    void on_nu_sub_editingFinished();
    void on_nu_add_editingFinished();
    void on_nu_value_editingFinished();
    void on_nu_doSub_clicked();
    void on_nu_doAdd_clicked();
    void on_nu_valueChanged(int value);

    //alpha
    void on_alpha_max_editingFinished();
    void on_alpha_min_editingFinished();
    void on_alpha_sub_editingFinished();
    void on_alpha_add_editingFinished();
    void on_alpha_value_editingFinished();
    void on_alpha_doSub_clicked();
    void on_alpha_doAdd_clicked();
    void on_alpha_valueChanged(int value);

    //sig1
    void on_sig1_max_editingFinished();
    void on_sig1_min_editingFinished();
    void on_sig1_sub_editingFinished();
    void on_sig1_add_editingFinished();
    void on_sig1_value_editingFinished();
    void on_sig1_doSub_clicked();
    void on_sig1_doAdd_clicked();
    void on_sig1_valueChanged(int value);

    //lcc
    void on_lcc_max_editingFinished();
    void on_lcc_min_editingFinished();
    void on_lcc_sub_editingFinished();
    void on_lcc_add_editingFinished();
    void on_lcc_value_editingFinished();
    void on_lcc_doSub_clicked();
    void on_lcc_doAdd_clicked();
    void on_lcc_valueChanged(int value);

    //q
    void on_q_max_editingFinished();
    void on_q_min_editingFinished();
    void on_q_sub_editingFinished();
    void on_q_add_editingFinished();
    void on_q_value_editingFinished();
    void on_q_doSub_clicked();
    void on_q_doAdd_clicked();
    void on_q_valueChanged(int value);

    //cN
    void on_cN_max_editingFinished();
    void on_cN_min_editingFinished();
    void on_cN_sub_editingFinished();
    void on_cN_add_editingFinished();
    void on_cN_value_editingFinished();
    void on_cN_doSub_clicked();
    void on_cN_doAdd_clicked();
    void on_cN_valueChanged(int value);

    //cO
    void on_cO_max_editingFinished();
    void on_cO_min_editingFinished();
    void on_cO_sub_editingFinished();
    void on_cO_add_editingFinished();
    void on_cO_value_editingFinished();
    void on_cO_doSub_clicked();
    void on_cO_doAdd_clicked();
    void on_cO_valueChanged(int value);

    //dan
    void on_dan_max_editingFinished();
    void on_dan_min_editingFinished();
    void on_dan_sub_editingFinished();
    void on_dan_add_editingFinished();
    void on_dan_value_editingFinished();
    void on_dan_doSub_clicked();
    void on_dan_doAdd_clicked();
    void on_dan_valueChanged(int value);

    //k
    void on_k_max_editingFinished();
    void on_k_min_editingFinished();
    void on_k_sub_editingFinished();
    void on_k_add_editingFinished();
    void on_k_value_editingFinished();
    void on_k_doSub_clicked();
    void on_k_doAdd_clicked();
    void on_k_valueChanged(int value);

    //const1
    void on_const1_max_editingFinished();
    void on_const1_min_editingFinished();
    void on_const1_sub_editingFinished();
    void on_const1_add_editingFinished();
    void on_const1_value_editingFinished();
    void on_const1_doSub_clicked();
    void on_const1_doAdd_clicked();
    void on_const1_valueChanged(int value);

    //const2
    void on_const2_max_editingFinished();
    void on_const2_min_editingFinished();
    void on_const2_sub_editingFinished();
    void on_const2_add_editingFinished();
    void on_const2_value_editingFinished();
    void on_const2_doSub_clicked();
    void on_const2_doAdd_clicked();
    void on_const2_valueChanged(int value);

    //g
    void on_g_max_editingFinished();
    void on_g_min_editingFinished();
    void on_g_sub_editingFinished();
    void on_g_add_editingFinished();
    void on_g_value_editingFinished();
    void on_g_doSub_clicked();
    void on_g_doAdd_clicked();
    void on_g_valueChanged(int value);

    //mue_ab
    void on_mue_ab_max_editingFinished();
    void on_mue_ab_min_editingFinished();
    void on_mue_ab_sub_editingFinished();
    void on_mue_ab_add_editingFinished();
    void on_mue_ab_value_editingFinished();
    void on_mue_ab_doSub_clicked();
    void on_mue_ab_doAdd_clicked();
    void on_mue_ab_valueChanged(int value);

    //p
    void on_p_max_editingFinished();
    void on_p_min_editingFinished();
    void on_p_sub_editingFinished();
    void on_p_add_editingFinished();
    void on_p_value_editingFinished();
    void on_p_doSub_clicked();
    void on_p_doAdd_clicked();
    void on_p_valueChanged(int value);

    //R
    void on_R_max_editingFinished();
    void on_R_min_editingFinished();
    void on_R_sub_editingFinished();
    void on_R_add_editingFinished();
    void on_R_value_editingFinished();
    void on_R_doSub_clicked();
    void on_R_doAdd_clicked();
    void on_R_valueChanged(int value);

    //t
    void on_t_max_editingFinished();
    void on_t_min_editingFinished();
    void on_t_sub_editingFinished();
    void on_t_add_editingFinished();
    void on_t_value_editingFinished();
    void on_t_doSub_clicked();
    void on_t_doAdd_clicked();
    void on_t_valueChanged(int value);


    void on_action_New_triggered();
    void on_action_Open_triggered();
    void on_action_Save_triggered();
    void on_action_SaveAs_triggered();
    void on_action_SaveAsTemplate_triggered();
    void on_action_Close_triggered();
    void on_action_simpleMode_triggered(bool checked);
    void on_action_autoUpdateFit_triggered(bool checked);
    void on_action_Settings_triggered();
    void on_action_ChooseTemplate_triggered();
    void on_action_ValuesBackToTemplate_triggered();


    void on_scrollBarH_valueChanged(int value);
    void on_scrollBarV_valueChanged(int value);
    void on_scrollBarControls_valueChanged(int value);

    void on_buttonLoad_clicked();
    void on_buttonDrawData_clicked(bool checked);
    void on_buttonDrawFit_clicked(bool checked);
    void on_buttonShowDeviation_clicked(bool checked);
    void on_buttonDrawAll_clicked(bool checked);
    void on_buttonAutoFit_clicked();
    void on_buttonOutputPars_clicked();
    void on_buttonSavePlotImage_clicked();
    void on_buttonSavePlotDataXY_clicked();
    void on_buttonSavePlotFitXY_clicked();

    void on_plot_minX_editingFinished();
    void on_plot_maxX_editingFinished();
    void on_plot_minY_editingFinished();
    void on_plot_maxY_editingFinished();
    void on_checkBox_Autoscale_clicked(bool checked);

    void on_spinBoxThinningFactor_valueChanged(int value);
    void on_buttonUpdateFit_clicked();

    void on_r_editingFinished();
    void on_d_editingFinished();
    void on_b_editingFinished();
    void on_divergenceAngle_editingFinished();
    void on_irradiatedLength_editingFinished();

    void on_checkBox_optQ_clicked(bool checked);
    void on_checkBox_optA_clicked(bool checked);
    void on_checkBox_optGradient_clicked(bool checked);
    void on_checkBox_optCorrAutoColl_clicked(bool checked);
    void on_checkBox_optPosition_clicked(bool checked);
    void on_comboBox_Position_currentIndexChanged(int index);
    void on_checkBox_optIntensity_clicked(bool checked);
    void on_comboBox_Intensity_currentIndexChanged(int index);

    void on_comboBox_Wavelengths_editTextChanged(const QString &arg1);
    void on_buttonInsertWavelength_clicked();
    void on_buttonSaveWavelength_clicked();
    void on_buttonDeleteWavelength_clicked();
};

#endif // MAINWINDOW_H
