/*
 * CarbX, except for the proprietary licensed third party source code in human
 * and/or machine-readable form, is free software. The third-party source code is
 * not shipped in human and/or machine-readable form along with these source
 * files. If you build CarbX from the provided CarbX’ source code files while
 * obtaining the required libraries, files and third party source code as well as
 * the required licences on your own, you can redistribute and/or modify the
 * remaining parts of CarbX (i. e. the parts developed by Torben Pfaff) under the
 * terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version. This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details. You should have received a copy of the GNU Lesser General
 * Public License along with this program. If not, see http://www.gnu.org/licenses/.
 */

#include "mainwindow.h"
#include "ui_mainwindow.h"

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow){

    ui->setupUi(this);

    //own code

    sst = new SettingsStorage(new QString (QApplication::applicationDirPath()) ,this);

    //size adaption
    sst->minSize_MainWindow = new QPoint(ui->mainWidget->width(), ui->mainWidget->height());
    setSize();

    sst->checkParameters_MainWindow = false; //no need at program startup

    if(!(sst->readProgramSettings()))
       AuxiliaryMethods::errorMessage(this, new QString(tr("Laden der Programmeinstellungen (teilweise) fehlgeschlagen!\nEs werden (teilweise) Standardeinstellungen verwendet.")));

    ui->plot->setAxisTitle(QwtPlot::yLeft,tr("Log (Intensitaet [arb. unit])"));
    ui->plot->setAxisTitle(QwtPlot::xBottom,"s [1/Å]");
    grid = new QwtPlotGrid();
    grid->enableX(true);
    grid->enableXMin(true);
    grid->enableY(true);
    grid->enableYMin(true);
    grid->attach(ui->plot);

    curveData = new QwtPlotCurve("data");
    curveData->setStyle(QwtPlotCurve::Dots);
    colorData = new QColor(Qt::blue);
    brushData = new QBrush (*colorData);
    penData = new QPen(*colorData);
    sizeData = new QSize(sst->prgset->thicknessOfCurveDots, sst->prgset->thicknessOfCurveDots);
    symbolData = new QwtSymbol(QwtSymbol::Ellipse, *brushData, *penData, *sizeData);
    curveData->setSymbol(symbolData);

    curveFit = new QwtPlotCurve("fit");
    curveFit->setStyle(QwtPlotCurve::Lines);
    colorFit = new QColor(Qt::red);
    penFit = new QPen(*colorFit);
    penFit->setWidth(sst->prgset->thicknessOfCurveLines);
    curveFit->setPen(*penFit);
    curveFit->setCurveFitter(new QwtSplineCurveFitter());

    dst = new DataStorage(new QString (QApplication::applicationDirPath()), this);

    bool prevState = ui->comboBox_Wavelengths->blockSignals(true); //save previous state
    update__comboBox_Wavelengths(dst->readWavelengths(false)); //read wavelength-structures and chose on former exit chosen wavelength-structure
    if (ui->comboBox_Wavelengths->currentIndex() < 0){ //no wavelengths are listed in user-defined file
        update__comboBox_Wavelengths(dst->readWavelengths(true));
    }
    dst->prgpar->currentWavelength = AuxiliaryMethods::validateAndParse_Wavelength(new QString(ui->comboBox_Wavelengths->currentText()));
    ui->comboBox_Wavelengths->blockSignals(prevState); //restore previous state

    dst->prgpar->setDefaultValues(true); //axis intervals have to be set at startup

    if( (dst->readParsMeaning()) )
        if (dst->readParsDefaultValues(dst->pars)){ // read default values
            set_slider_values();           // also initialises sliders
            dst->readParsDefaultValues(dst->pars); //  read default values again as initalisation of sliders may changed some values
            set_slider_values();          //  set slider values again
        }
        else
            AuxiliaryMethods::errorMessage(this, new QString(tr("Die Liste der Parameter-Standardwerte \"pars-defaultValues_de.txt\"\nkonnte nicht geladen werden.\nAus diesem Grund werden die Parameter auf programmintere\nStandardwerte gesetzt.\nDiese Werten entsprechen nicht den in der Textdatei\nhinterlegten, sinnvollen Werten!")));
    else{
        ui->buttonOutputPars->setDisabled(true);
        AuxiliaryMethods::errorMessage(this, new QString(tr("Die Liste der Parameter-Bedeutungen \"pars_de.txt\"\nkonnte nicht geladen werden.\nEs ist daher keine Ausgabe einer Parameter-Liste moeglich.\nAuch das Laden und Speichern von NGC/TPL-Dateien\nist nicht moeglich.\nZudem koennen die Standardwerte nicht gelesen werden.")));
        ui->menuDatei->removeAction(ui->action_Open);
        ui->menuDatei->removeAction(ui->action_Save);
        ui->menuDatei->removeAction(ui->action_SaveAs);
        ui->menuDatei->removeAction(ui->action_SaveAsTemplate);
    }

    if (!(dst->readParsNamesAutoFit())){
        AuxiliaryMethods::errorMessage(this, new QString("Die Liste der Parameter-Namen\nfuer die automatische Anpassung\n\"parsAutoFit_de.txt\" konnte nicht geladen werden.\nEs ist daher keine automatische Anpassung moeglich."));
        ui->buttonAutoFit->setDisabled(true);
    }

    datacal = new DataCalculations(new QString(*(dst->applicationDirPath) + "/data"), this);

    dev = new Deviation(this, datacal, dst, sst);

    applyProgramSettings(true);

    set_widget_values(true);

    sst->mustSave = false;

    sst->reCheck_buttonShowDeviation_MainWindow = false;
    sst->reCheck_optA_MainWindow = false;
    sst->reCheck_optIntensity_MainWindow = false;

    pfa = new ParameterFileAccess(datacal, this);

    afs = 0; //to be initialised later in onClick-method of autoFit-Button

    //widgets are only needed for video
    ui->widget_La->hide();
    ui->widget_Lc->hide();

    //disable action as no template is loaded at startup
    ui->action_ValuesBackToTemplate->setDisabled(true);

    setFocus();
    ui->plot->replot();

}


MainWindow::~MainWindow(){

    delete ui; //generated by Qt

    delete afs;

    delete dev;

    delete datacal;
    delete dst;

    //delete variables needed for plot

    curveFit->detach();
    delete curveFit;
    delete penFit;
    delete colorFit;

    curveData->detach();
    delete curveData;
    delete sizeData;
    delete penData;
    delete brushData;
    delete colorData;

    grid->detach();
    delete grid;

    //delete own settings stored in sst
    delete sst->minSize_MainWindow;

    delete sst;

}


void MainWindow::uncheck_buttonShowDeviation(){
    ui->buttonShowDeviation->setChecked(false);
    sst->drawDev = false;
    dev->hide();
    sst->reCheck_buttonShowDeviation_MainWindow = false;
    sst->mustSave = true;
    check_uncheck_buttons();
}


void MainWindow::display_afsResults(bool displayAutoFitResults){
    ui->groupBoxControls->setDisabled(displayAutoFitResults);
    ui->buttonUpdateFit->setDisabled(displayAutoFitResults || sst->prgset->autoUpdateFit);
    dev->setEnabledDisabledForDisplayOfAutoFitResults(displayAutoFitResults);
    bool prevState = sst->prgset->autoUpdateFit; //save previous state
    sst->prgset->autoUpdateFit = true; //temporary so that calling draw() updates the plot
    draw(true);
    sst->prgset->autoUpdateFit = prevState; //restore previous state
    if (sst->drawDev && !(dst->prgpar->followChanges))
        dev->updatePlot();
}


void MainWindow::takeOver_afsResults(){
    afs->close();
    set_slider_values();
    bool prevState = sst->prgset->autoUpdateFit; //save previous state
    sst->prgset->autoUpdateFit = true; //temporary so that calling draw() updates the plot
    draw(true);
    sst->prgset->autoUpdateFit = prevState; //restore previous state
    if (sst->drawDev && !(dst->prgpar->followChanges))
        dev->updatePlot();
    sst->mustSave = true;
}


void MainWindow::on_close_afs(){
    delete afs;
    afs = 0;
    ui->menuBar->setEnabled(true);
    ui->buttonLoad->setEnabled(true);
    ui->buttonDrawData->setEnabled(true);
    ui->buttonDrawFit->setEnabled(true);
    ui->buttonDrawAll->setEnabled(true);
    ui->buttonAutoFit->setChecked(false);
    ui->buttonAutoFit->setEnabled(true);
    ui->groupBoxParSettings->setEnabled(true);
    ui->widget_cN->setEnabled(true);
    ui->widget_cO->setEnabled(true);
    setEnabledDisabled(); //for mue_ab, p, R, t widgets
}



//own private methods


//adjustment of size
void MainWindow::setSize(){

    RECT clientRect, windowRect;
    GetClientRect((HWND)(winId()), &clientRect);
    GetWindowRect((HWND)(winId()), &windowRect);

    int borderSize = ( ( (windowRect.right - windowRect.left) - (clientRect.right - clientRect.left) ) / 2);

    QDesktopWidget *desktop = new QDesktopWidget();
    int targetWidth = desktop->screenGeometry().width() - 2 * borderSize;    // maximum Width of mainWidget
    int targetHeight = desktop->screenGeometry().height() - ((windowRect.bottom - windowRect.top) - (clientRect.bottom - clientRect.top)) - 50 - 26; // maximum Height of mainWidget
    delete desktop;

    int targetWidthforCentering = targetWidth;
    int targetHeightforCentering = targetHeight;

    if (targetWidth > sst->minSize_MainWindow->x() && targetHeight > sst->minSize_MainWindow->y()){
        int max = ( ((targetWidth - 820 - 150) < (targetHeight - 409 - 25)) ? (targetWidth - 820 - 150) : (targetHeight - 409 - 25) ) ;
        ui->plot->resize(max, max);
        ui->widgetPlot->resize(max + 150, max + 25);

        ui->label_plot_minX->move(ui->label_plot_minX->x(), max + 5);
        ui->plot_minX->move(ui->plot_minX->x(), max + 5);
        ui->label_plot_maxX->move(max - 100, max + 5);
        ui->plot_maxX->move(max - 40, max + 5);

        ui->checkBox_Autoscale->move(max + 10, max / 2 - 20);

        ui->label_plot_minY->move(max + 10, max - 75);
        ui->plot_minY->move(max + 50, max - 75);
        ui->label_plot_maxY->move(max + 10, ui->label_plot_maxY->y());
        ui->plot_maxY->move(max + 50, max + ui->plot_maxY->y());

        ui->widget_FitPoints_UpdateFit->move(810, ui->widgetPlot->height() + 30);
        ui->groupBoxParSettings->move(810, ui->widgetPlot->height() + 80);
        targetWidth = ( (ui->widgetPlot->width() + 820 + 150) > (sst->minSize_MainWindow->x()) ) ? ((ui->widgetPlot->width() + 820 + 150)) : (sst->minSize_MainWindow->x());
        targetHeight = ( (ui->widgetPlot->height() + 409) > (sst->minSize_MainWindow->y()) ) ? ((ui->widgetPlot->height() + 409)) : (sst->minSize_MainWindow->y());
    }
    else{
        targetWidth = ( (targetWidth < sst->minSize_MainWindow->x()) ? targetWidth : sst->minSize_MainWindow->x() + (targetHeight < sst->minSize_MainWindow->y() ? 30 : 0));
        targetHeight = ( (targetHeight < sst->minSize_MainWindow->y()) ? targetHeight : sst->minSize_MainWindow->y() + (targetWidth < sst->minSize_MainWindow->x() ? 30 : 0));
    }

    ui->centralWidget->resize(targetWidth,targetHeight);
    ui->mainWidget->resize(targetWidth - (targetHeight < sst->minSize_MainWindow->y() ? 30 : 0), targetHeight - (targetWidth < sst->minSize_MainWindow->x() ? 30 : 0));

    if (targetWidth >= sst->minSize_MainWindow->x()){
        ui->scrollBarH->setDisabled(true);
        ui->scrollBarH->hide();
    }
    else{
        ui->scrollBarH->setGeometry(0, ui->centralWidget->height() - 30, centralWidget()->width() - (targetHeight < sst->minSize_MainWindow->y() ? 30 : 10), ui->scrollBarH->height());
        ui->scrollBarH->setMinimum(0);
        ui->scrollBarH->setMaximum(sst->minSize_MainWindow->x() - ui->mainWidget->width());
        ui->scrollBarH->setSingleStep(10);
        ui->scrollBarH->setPageStep(10);
    }

    if (targetHeight >= sst->minSize_MainWindow->y()){
        ui->scrollBarV->setDisabled(true);
        ui->scrollBarV->hide();
    }
    else{
        ui->scrollBarV->setGeometry(ui->centralWidget->width() - 30, 0, ui->scrollBarV->width(), ui->centralWidget->height() - (targetWidth < sst->minSize_MainWindow->x() ? 30 : 10));
        ui->scrollBarV->setMinimum(0);
        ui->scrollBarV->setMaximum(sst->minSize_MainWindow->y() - ui->mainWidget->height());
        ui->scrollBarV->setSingleStep(10);
        ui->scrollBarV->setPageStep(10);
    }

    resize(targetWidth, targetHeight + 26);
    move((targetWidthforCentering/2 - width()/2), ( (targetHeightforCentering + 26)/2 - height()/2));
    setFixedSize(targetWidth, targetHeight + 26);
    setWindowFlags( (windowFlags() | Qt::CustomizeWindowHint) & ~Qt::WindowMaximizeButtonHint);

    sst->visibleSlider_MainWindow = (ui->mainWidget->height() - 240) / 90;
    ui->groupBoxControls->resize(ui->groupBoxControls->width(), 90 * sst->visibleSlider_MainWindow);
    ui->scrollBarControls->resize(ui->scrollBarControls->width(), 90 * sst->visibleSlider_MainWindow - 20);
    ui->scrollBarControls->setMaximum( (Enumerations::countOf_parName + 1) / 2 - sst->visibleSlider_MainWindow);
}


void MainWindow::applyProgramSettings(bool startup){

    if (startup)
        ui->spinBoxThinningFactor->setValue(dst->prgpar->thinningFactor);

    ui->action_simpleMode->setChecked(sst->prgset->simpleMode);

    if (sst->prgset->simpleMode){
        ui->spinBoxThinningFactor->setDisabled(true);
        dst->prgpar->thinningFactor = ui->spinBoxThinningFactor->value();
        bool prevState = ui->spinBoxThinningFactor->blockSignals(true); //save previous state
        ui->spinBoxThinningFactor->setValue(10); //default value for simple mode
        draw(false);
        ui->spinBoxThinningFactor->blockSignals(prevState); //restore previous state
        ui->cno_min->setDisabled(true);
        ui->cno_max->setDisabled(true);
        ui->cno_sub->setDisabled(true);
        ui->cno_add->setDisabled(true);

        ui->mu_min->setDisabled(true);
        ui->mu_max->setDisabled(true);
        ui->mu_sub->setDisabled(true);
        ui->mu_add->setDisabled(true);

        ui->beta_min->setDisabled(true);
        ui->beta_max->setDisabled(true);
        ui->beta_sub->setDisabled(true);
        ui->beta_add->setDisabled(true);

        ui->a3_min->setDisabled(true);
        ui->a3_max->setDisabled(true);
        ui->a3_sub->setDisabled(true);
        ui->a3_add->setDisabled(true);

        ui->da3_min->setDisabled(true);
        ui->da3_max->setDisabled(true);
        ui->da3_sub->setDisabled(true);
        ui->da3_add->setDisabled(true);

        ui->sig3_min->setDisabled(true);
        ui->sig3_max->setDisabled(true);
        ui->sig3_sub->setDisabled(true);
        ui->sig3_add->setDisabled(true);

        ui->u3_min->setDisabled(true);
        ui->u3_max->setDisabled(true);
        ui->u3_sub->setDisabled(true);
        ui->u3_add->setDisabled(true);

        ui->eta_min->setDisabled(true);
        ui->eta_max->setDisabled(true);
        ui->eta_sub->setDisabled(true);
        ui->eta_add->setDisabled(true);

        ui->nu_min->setDisabled(true);
        ui->nu_max->setDisabled(true);
        ui->nu_sub->setDisabled(true);
        ui->nu_add->setDisabled(true);

        ui->alpha_min->setDisabled(true);
        ui->alpha_max->setDisabled(true);
        ui->alpha_sub->setDisabled(true);
        ui->alpha_add->setDisabled(true);

        ui->sig1_min->setDisabled(true);
        ui->sig1_max->setDisabled(true);
        ui->sig1_sub->setDisabled(true);
        ui->sig1_add->setDisabled(true);

        ui->lcc_min->setDisabled(true);
        ui->lcc_max->setDisabled(true);
        ui->lcc_sub->setDisabled(true);
        ui->lcc_add->setDisabled(true);

        ui->q_min->setDisabled(true);
        ui->q_max->setDisabled(true);
        ui->q_sub->setDisabled(true);
        ui->q_add->setDisabled(true);

        ui->cN_min->setDisabled(true);
        ui->cN_max->setDisabled(true);
        ui->cN_sub->setDisabled(true);
        ui->cN_add->setDisabled(true);

        ui->cO_min->setDisabled(true);
        ui->cO_max->setDisabled(true);
        ui->cO_sub->setDisabled(true);
        ui->cO_add->setDisabled(true);

        ui->dan_min->setDisabled(true);
        ui->dan_max->setDisabled(true);
        ui->dan_sub->setDisabled(true);
        ui->dan_add->setDisabled(true);

        ui->k_min->setDisabled(true);
        ui->k_max->setDisabled(true);
        ui->k_sub->setDisabled(true);
        ui->k_add->setDisabled(true);

        ui->const1_min->setDisabled(true);
        ui->const1_max->setDisabled(true);
        ui->const1_sub->setDisabled(true);
        ui->const1_add->setDisabled(true);

        ui->const2_min->setDisabled(true);
        ui->const2_max->setDisabled(true);
        ui->const2_sub->setDisabled(true);
        ui->const2_add->setDisabled(true);

        ui->g_min->setDisabled(true);
        ui->g_max->setDisabled(true);
        ui->g_sub->setDisabled(true);
        ui->g_add->setDisabled(true);

        ui->mue_ab_min->setDisabled(true);
        ui->mue_ab_max->setDisabled(true);
        ui->mue_ab_sub->setDisabled(true);
        ui->mue_ab_add->setDisabled(true);

        ui->R_min->setDisabled(true);
        ui->R_max->setDisabled(true);
        ui->R_sub->setDisabled(true);
        ui->R_add->setDisabled(true);

        ui->p_min->setDisabled(true);
        ui->p_max->setDisabled(true);
        ui->p_sub->setDisabled(true);
        ui->p_add->setDisabled(true);

        ui->t_min->setDisabled(true);
        ui->t_max->setDisabled(true);
        ui->t_sub->setDisabled(true);
        ui->t_add->setDisabled(true);

    }
    else {
        ui->spinBoxThinningFactor->setValue(dst->prgpar->thinningFactor);
        ui->spinBoxThinningFactor->setEnabled(true);

        ui->cno_min->setEnabled(true);
        ui->cno_max->setEnabled(true);
        ui->cno_sub->setEnabled(true);
        ui->cno_add->setEnabled(true);

        ui->mu_min->setEnabled(true);
        ui->mu_max->setEnabled(true);
        ui->mu_sub->setEnabled(true);
        ui->mu_add->setEnabled(true);

        ui->beta_min->setEnabled(true);
        ui->beta_max->setEnabled(true);
        ui->beta_sub->setEnabled(true);
        ui->beta_add->setEnabled(true);

        ui->a3_min->setEnabled(true);
        ui->a3_max->setEnabled(true);
        ui->a3_sub->setEnabled(true);
        ui->a3_add->setEnabled(true);

        ui->da3_min->setEnabled(true);
        ui->da3_max->setEnabled(true);
        ui->da3_sub->setEnabled(true);
        ui->da3_add->setEnabled(true);

        ui->sig3_min->setEnabled(true);
        ui->sig3_max->setEnabled(true);
        ui->sig3_sub->setEnabled(true);
        ui->sig3_add->setEnabled(true);

        ui->u3_min->setEnabled(true);
        ui->u3_max->setEnabled(true);
        ui->u3_sub->setEnabled(true);
        ui->u3_add->setEnabled(true);

        ui->eta_min->setEnabled(true);
        ui->eta_max->setEnabled(true);
        ui->eta_sub->setEnabled(true);
        ui->eta_add->setEnabled(true);

        ui->nu_min->setEnabled(true);
        ui->nu_max->setEnabled(true);
        ui->nu_sub->setEnabled(true);
        ui->nu_add->setEnabled(true);

        ui->alpha_min->setEnabled(true);
        ui->alpha_max->setEnabled(true);
        ui->alpha_sub->setEnabled(true);
        ui->alpha_add->setEnabled(true);

        ui->sig1_min->setEnabled(true);
        ui->sig1_max->setEnabled(true);
        ui->sig1_sub->setEnabled(true);
        ui->sig1_add->setEnabled(true);

        ui->lcc_min->setEnabled(true);
        ui->lcc_max->setEnabled(true);
        ui->lcc_sub->setEnabled(true);
        ui->lcc_add->setEnabled(true);

        ui->q_min->setEnabled(true);
        ui->q_max->setEnabled(true);
        ui->q_sub->setEnabled(true);
        ui->q_add->setEnabled(true);

        ui->cN_min->setEnabled(true);
        ui->cN_max->setEnabled(true);
        ui->cN_sub->setEnabled(true);
        ui->cN_add->setEnabled(true);

        ui->cO_min->setEnabled(true);
        ui->cO_max->setEnabled(true);
        ui->cO_sub->setEnabled(true);
        ui->cO_add->setEnabled(true);

        ui->dan_min->setEnabled(true);
        ui->dan_max->setEnabled(true);
        ui->dan_sub->setEnabled(true);
        ui->dan_add->setEnabled(true);

        ui->k_min->setEnabled(true);
        ui->k_max->setEnabled(true);
        ui->k_sub->setEnabled(true);
        ui->k_add->setEnabled(true);

        ui->const1_min->setEnabled(true);
        ui->const1_max->setEnabled(true);
        ui->const1_sub->setEnabled(true);
        ui->const1_add->setEnabled(true);

        ui->const2_min->setEnabled(true);
        ui->const2_max->setEnabled(true);
        ui->const2_sub->setEnabled(true);
        ui->const2_add->setEnabled(true);

        ui->g_min->setEnabled(true);
        ui->g_max->setEnabled(true);
        ui->g_sub->setEnabled(true);
        ui->g_add->setEnabled(true);

        ui->mue_ab_min->setEnabled(true);
        ui->mue_ab_max->setEnabled(true);
        ui->mue_ab_sub->setEnabled(true);
        ui->mue_ab_add->setEnabled(true);

        ui->p_min->setEnabled(true);
        ui->p_max->setEnabled(true);
        ui->p_sub->setEnabled(true);
        ui->p_add->setEnabled(true);

        ui->R_min->setEnabled(true);
        ui->R_max->setEnabled(true);
        ui->R_sub->setEnabled(true);
        ui->R_add->setEnabled(true);

        ui->t_min->setEnabled(true);
        ui->t_max->setEnabled(true);
        ui->t_sub->setEnabled(true);
        ui->t_add->setEnabled(true);

    }

    ui->action_autoUpdateFit->setChecked(sst->prgset->autoUpdateFit);
    ui->buttonUpdateFit->setDisabled(sst->prgset->autoUpdateFit); //set enabled state of button correspondig to checked state of menu item

}


void MainWindow::update__comboBox_Wavelengths(int index){
    bool prevState = ui->comboBox_Wavelengths->blockSignals(true); //save previous state
    ui->comboBox_Wavelengths->clear();
    for (int i = 0; i < dst->wavelengths->size(); ++i)
        ui->comboBox_Wavelengths->addItem( ( (dst->wavelengths->at(i).name.length() > 0) ? (dst->wavelengths->at(i).name + " - ") : "" ) + QString::number(dst->wavelengths->at(i).value,'g',10));
    ui->comboBox_Wavelengths->blockSignals(prevState); //restore previous state
    ui->comboBox_Wavelengths->setCurrentIndex(index);
}

void MainWindow::set_slider_values(){

    bool prevState = true; //to save previous state of blockSignals

    sst->checkParameters_MainWindow = false; //parameters are read from corresponding variables

    ui->cno_sub->setText(QString::number(dst->pars->getParameterValue(Enumerations::cno, Enumerations::dec),'f',5));
    ui->cno_add->setText(QString::number(dst->pars->getParameterValue(Enumerations::cno, Enumerations::inc),'f',5));
    ui->cno_min->setText(QString::number(dst->pars->getParameterValue(Enumerations::cno, Enumerations::min),'f',5));
    ui->cno_max->setText(QString::number(dst->pars->getParameterValue(Enumerations::cno, Enumerations::max),'f',5));
    prevState = ui->cno->blockSignals(true);
    ui->cno->setMaximum((int)(floor(dst->pars->getParameterValue(Enumerations::cno, Enumerations::max)*1.E5)));
    ui->cno->setMinimum((int)(floor(dst->pars->getParameterValue(Enumerations::cno, Enumerations::min)*1.E5)));
    set_cno_min_max();
    ui->cno->setValue((int)(floor(dst->pars->getParameterValue(Enumerations::cno, Enumerations::val) * 1.E5)));
    ui->cno->blockSignals(prevState);
    ui->cno_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::cno, Enumerations::val),'f',5));

    ui->mu_sub->setText(QString::number(dst->pars->getParameterValue(Enumerations::mu, Enumerations::dec),'f',5));
    ui->mu_add->setText(QString::number(dst->pars->getParameterValue(Enumerations::mu, Enumerations::inc),'f',5));
    ui->mu_min->setText(QString::number(dst->pars->getParameterValue(Enumerations::mu, Enumerations::min),'f',5));
    ui->mu_max->setText(QString::number(dst->pars->getParameterValue(Enumerations::mu, Enumerations::max),'f',5));
    prevState = ui->mu->blockSignals(true);
    ui->mu->setMaximum((int)(floor(dst->pars->getParameterValue(Enumerations::mu, Enumerations::max)*1.E5)));
    ui->mu->setMinimum((int)(floor(dst->pars->getParameterValue(Enumerations::mu, Enumerations::min)*1.E5)));
    set_mu_min_max();
    ui->mu->setValue((int)(floor(dst->pars->getParameterValue(Enumerations::mu, Enumerations::val) * 1.E5)));
    ui->mu->blockSignals(prevState);
    ui->mu_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::mu, Enumerations::val),'f',5));

    ui->beta_sub->setText(QString::number(dst->pars->getParameterValue(Enumerations::beta, Enumerations::dec),'f',5));
    ui->beta_add->setText(QString::number(dst->pars->getParameterValue(Enumerations::beta, Enumerations::inc),'f',5));
    ui->beta_min->setText(QString::number(dst->pars->getParameterValue(Enumerations::beta, Enumerations::min),'f',5));
    ui->beta_max->setText(QString::number(dst->pars->getParameterValue(Enumerations::beta, Enumerations::max),'f',5));
    prevState = ui->beta->blockSignals(true);
    ui->beta->setMaximum((int)(floor(dst->pars->getParameterValue(Enumerations::beta, Enumerations::max)*1.E5)));
    ui->beta->setMinimum((int)(floor(dst->pars->getParameterValue(Enumerations::beta, Enumerations::min)*1.E5)));
    set_beta_min_max();
    ui->beta->setValue((int)(floor(dst->pars->getParameterValue(Enumerations::beta, Enumerations::val) * 1.E5)));
    ui->beta->blockSignals(prevState);
    ui->beta_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::beta, Enumerations::val),'f',5));

    ui->a3_sub->setText(QString::number(dst->pars->getParameterValue(Enumerations::a3, Enumerations::dec),'f',5));
    ui->a3_add->setText(QString::number(dst->pars->getParameterValue(Enumerations::a3, Enumerations::inc),'f',5));
    ui->a3_min->setText(QString::number(dst->pars->getParameterValue(Enumerations::a3, Enumerations::min),'f',5));
    ui->a3_max->setText(QString::number(dst->pars->getParameterValue(Enumerations::a3, Enumerations::max),'f',5));
    prevState = ui->a3->blockSignals(true);
    ui->a3->setMaximum((int)(floor(dst->pars->getParameterValue(Enumerations::a3, Enumerations::max)*1.E5)));
    ui->a3->setMinimum((int)(floor(dst->pars->getParameterValue(Enumerations::a3, Enumerations::min)*1.E5)));
    set_a3_min_max();
    ui->a3->setValue((int)(floor(dst->pars->getParameterValue(Enumerations::a3, Enumerations::val) * 1.E5)));
    ui->a3->blockSignals(prevState);
    ui->a3_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::a3, Enumerations::val),'f',5));

    ui->da3_sub->setText(QString::number(dst->pars->getParameterValue(Enumerations::da3, Enumerations::dec),'f',5));
    ui->da3_add->setText(QString::number(dst->pars->getParameterValue(Enumerations::da3, Enumerations::inc),'f',5));
    ui->da3_min->setText(QString::number(dst->pars->getParameterValue(Enumerations::da3, Enumerations::min),'f',5));
    ui->da3_max->setText(QString::number(dst->pars->getParameterValue(Enumerations::da3, Enumerations::max),'f',5));
    prevState = ui->da3->blockSignals(true);
    ui->da3->setMaximum((int)(floor(dst->pars->getParameterValue(Enumerations::da3, Enumerations::max)*1.E5)));
    ui->da3->setMinimum((int)(floor(dst->pars->getParameterValue(Enumerations::da3, Enumerations::min)*1.E5)));
    set_da3_min_max();
    ui->da3->setValue((int)(floor(dst->pars->getParameterValue(Enumerations::da3, Enumerations::val) * 1.E5)));
    ui->da3->blockSignals(prevState);
    ui->da3_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::da3, Enumerations::val),'f',5));

    ui->sig3_sub->setText(QString::number(dst->pars->getParameterValue(Enumerations::sig3, Enumerations::dec),'f',5));
    ui->sig3_add->setText(QString::number(dst->pars->getParameterValue(Enumerations::sig3, Enumerations::inc),'f',5));
    ui->sig3_min->setText(QString::number(dst->pars->getParameterValue(Enumerations::sig3, Enumerations::min),'f',5));
    ui->sig3_max->setText(QString::number(dst->pars->getParameterValue(Enumerations::sig3, Enumerations::max),'f',5));
    prevState = ui->sig3->blockSignals(true);
    ui->sig3->setMaximum((int)(floor(dst->pars->getParameterValue(Enumerations::sig3, Enumerations::max)*1.E5)));
    ui->sig3->setMinimum((int)(floor(dst->pars->getParameterValue(Enumerations::sig3, Enumerations::min)*1.E5)));
    set_sig3_min_max();
    ui->sig3->setValue((int)(floor(dst->pars->getParameterValue(Enumerations::sig3, Enumerations::val) * 1.E5)));
    ui->sig3->blockSignals(prevState);
    ui->sig3_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::sig3, Enumerations::val),'f',5));

    ui->u3_sub->setText(QString::number(dst->pars->getParameterValue(Enumerations::u3, Enumerations::dec),'f',5));
    ui->u3_add->setText(QString::number(dst->pars->getParameterValue(Enumerations::u3, Enumerations::inc),'f',5));
    ui->u3_min->setText(QString::number(dst->pars->getParameterValue(Enumerations::u3, Enumerations::min),'f',5));
    ui->u3_max->setText(QString::number(dst->pars->getParameterValue(Enumerations::u3, Enumerations::max),'f',5));
    prevState = ui->u3->blockSignals(true);
    ui->u3->setMaximum((int)(floor(dst->pars->getParameterValue(Enumerations::u3, Enumerations::max)*1.E5)));
    ui->u3->setMinimum((int)(floor(dst->pars->getParameterValue(Enumerations::u3, Enumerations::min)*1.E5)));
    set_u3_min_max();
    ui->u3->setValue((int)(floor(dst->pars->getParameterValue(Enumerations::u3, Enumerations::val) * 1.E5)));
    ui->u3->blockSignals(prevState);
    ui->u3_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::u3, Enumerations::val),'f',5));

    ui->eta_sub->setText(QString::number(dst->pars->getParameterValue(Enumerations::eta, Enumerations::dec),'f',5));
    ui->eta_add->setText(QString::number(dst->pars->getParameterValue(Enumerations::eta, Enumerations::inc),'f',5));
    ui->eta_min->setText(QString::number(dst->pars->getParameterValue(Enumerations::eta, Enumerations::min),'f',5));
    ui->eta_max->setText(QString::number(dst->pars->getParameterValue(Enumerations::eta, Enumerations::max),'f',5));
    prevState = ui->eta->blockSignals(true);
    ui->eta->setMaximum((int)(floor(dst->pars->getParameterValue(Enumerations::eta, Enumerations::max)*1.E5)));
    ui->eta->setMinimum((int)(floor(dst->pars->getParameterValue(Enumerations::eta, Enumerations::min)*1.E5)));
    set_eta_min_max();
    ui->eta->setValue((int)(floor(dst->pars->getParameterValue(Enumerations::eta, Enumerations::val) * 1.E5)));
    ui->eta->blockSignals(prevState);
    ui->eta_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::eta, Enumerations::val),'f',5));

    ui->nu_sub->setText(QString::number(dst->pars->getParameterValue(Enumerations::nu, Enumerations::dec),'f',0));
    ui->nu_add->setText(QString::number(dst->pars->getParameterValue(Enumerations::nu, Enumerations::inc),'f',0));
    ui->nu_min->setText(QString::number(dst->pars->getParameterValue(Enumerations::nu, Enumerations::min),'f',0));
    ui->nu_max->setText(QString::number(dst->pars->getParameterValue(Enumerations::nu, Enumerations::max),'f',0));
    prevState = ui->nu->blockSignals(true);
    ui->nu->setMaximum((int)(floor(dst->pars->getParameterValue(Enumerations::nu, Enumerations::max))));
    ui->nu->setMinimum((int)(floor(dst->pars->getParameterValue(Enumerations::nu, Enumerations::min))));
    set_nu_min_max();
    ui->nu->setValue((int)(floor(dst->pars->getParameterValue(Enumerations::nu, Enumerations::val))));
    ui->nu->blockSignals(prevState);
    ui->nu_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::nu, Enumerations::val),'f',0));

    ui->alpha_sub->setText(QString::number(dst->pars->getParameterValue(Enumerations::alpha, Enumerations::dec),'f',5));
    ui->alpha_add->setText(QString::number(dst->pars->getParameterValue(Enumerations::alpha, Enumerations::inc),'f',5));
    ui->alpha_min->setText(QString::number(dst->pars->getParameterValue(Enumerations::alpha, Enumerations::min),'f',5));
    ui->alpha_max->setText(QString::number(dst->pars->getParameterValue(Enumerations::alpha, Enumerations::max),'f',5));
    prevState = ui->alpha->blockSignals(true);
    ui->alpha->setMaximum((int)(floor(dst->pars->getParameterValue(Enumerations::alpha, Enumerations::max)*1.E5)));
    ui->alpha->setMinimum((int)(floor(dst->pars->getParameterValue(Enumerations::alpha, Enumerations::min)*1.E5)));
    set_alpha_min_max();
    ui->alpha->setValue((int)(floor(dst->pars->getParameterValue(Enumerations::alpha, Enumerations::val) * 1.E5)));
    ui->alpha->blockSignals(prevState);
    ui->alpha_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::alpha, Enumerations::val),'f',5));

    ui->sig1_sub->setText(QString::number(dst->pars->getParameterValue(Enumerations::sig1, Enumerations::dec),'f',5));
    ui->sig1_add->setText(QString::number(dst->pars->getParameterValue(Enumerations::sig1, Enumerations::inc),'f',5));
    ui->sig1_min->setText(QString::number(dst->pars->getParameterValue(Enumerations::sig1, Enumerations::min),'f',5));
    ui->sig1_max->setText(QString::number(dst->pars->getParameterValue(Enumerations::sig1, Enumerations::max),'f',5));
    prevState = ui->sig1->blockSignals(true);
    ui->sig1->setMaximum((int)(floor(dst->pars->getParameterValue(Enumerations::sig1, Enumerations::max)*1.E5)));
    ui->sig1->setMinimum((int)(floor(dst->pars->getParameterValue(Enumerations::sig1, Enumerations::min)*1.E5)));
    set_sig1_min_max();
    ui->sig1->setValue((int)(floor(dst->pars->getParameterValue(Enumerations::sig1, Enumerations::val) * 1.E5)));
    ui->sig1->blockSignals(prevState);
    ui->sig1_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::sig1, Enumerations::val),'f',5));

    ui->lcc_sub->setText(QString::number(dst->pars->getParameterValue(Enumerations::lcc, Enumerations::dec),'f',5));
    ui->lcc_add->setText(QString::number(dst->pars->getParameterValue(Enumerations::lcc, Enumerations::inc),'f',5));
    ui->lcc_min->setText(QString::number(dst->pars->getParameterValue(Enumerations::lcc, Enumerations::min),'f',5));
    ui->lcc_max->setText(QString::number(dst->pars->getParameterValue(Enumerations::lcc, Enumerations::max),'f',5));
    prevState = ui->lcc->blockSignals(true);
    ui->lcc->setMaximum((int)(floor(dst->pars->getParameterValue(Enumerations::lcc, Enumerations::max)*1.E5)));
    ui->lcc->setMinimum((int)(floor(dst->pars->getParameterValue(Enumerations::lcc, Enumerations::min)*1.E5)));
    set_lcc_min_max();
    ui->lcc->setValue((int)(floor(dst->pars->getParameterValue(Enumerations::lcc, Enumerations::val) * 1.E5)));
    ui->lcc->blockSignals(prevState);
    ui->lcc_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::lcc, Enumerations::val),'f',5));

    ui->q_sub->setText(QString::number(dst->pars->getParameterValue(Enumerations::q, Enumerations::dec),'f',5));
    ui->q_add->setText(QString::number(dst->pars->getParameterValue(Enumerations::q, Enumerations::inc),'f',5));
    ui->q_min->setText(QString::number(dst->pars->getParameterValue(Enumerations::q, Enumerations::min),'f',5));
    ui->q_max->setText(QString::number(dst->pars->getParameterValue(Enumerations::q, Enumerations::max),'f',5));
    prevState = ui->q->blockSignals(true);
    ui->q->setMaximum((int)(floor(dst->pars->getParameterValue(Enumerations::q, Enumerations::max)*1.E5)));
    ui->q->setMinimum((int)(floor(dst->pars->getParameterValue(Enumerations::q, Enumerations::min)*1.E5)));
    set_q_min_max();
    ui->q->setValue((int)(floor(dst->pars->getParameterValue(Enumerations::q, Enumerations::val) * 1.E5)));
    ui->q->blockSignals(prevState);
    ui->q_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::q, Enumerations::val),'f',5));

    ui->cN_sub->setText(QString::number(dst->pars->getParameterValue(Enumerations::cN, Enumerations::dec),'f',5));
    ui->cN_add->setText(QString::number(dst->pars->getParameterValue(Enumerations::cN, Enumerations::inc),'f',5));
    ui->cN_min->setText(QString::number(dst->pars->getParameterValue(Enumerations::cN, Enumerations::min),'f',5));
    ui->cN_max->setText(QString::number(dst->pars->getParameterValue(Enumerations::cN, Enumerations::max),'f',5));
    prevState = ui->cN->blockSignals(true);
    ui->cN->setMaximum((int)(floor(dst->pars->getParameterValue(Enumerations::cN, Enumerations::max)*1.E5)));
    ui->cN->setMinimum((int)(floor(dst->pars->getParameterValue(Enumerations::cN, Enumerations::min)*1.E5)));
    set_cN_min_max();
    ui->cN->setValue((int)(floor(dst->pars->getParameterValue(Enumerations::cN, Enumerations::val) * 1.E5)));
    ui->cN->blockSignals(prevState);
    ui->cN_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::cN, Enumerations::val),'f',5));

    ui->cO_sub->setText(QString::number(dst->pars->getParameterValue(Enumerations::cO, Enumerations::dec),'f',5));
    ui->cO_add->setText(QString::number(dst->pars->getParameterValue(Enumerations::cO, Enumerations::inc),'f',5));
    ui->cO_min->setText(QString::number(dst->pars->getParameterValue(Enumerations::cO, Enumerations::min),'f',5));
    ui->cO_max->setText(QString::number(dst->pars->getParameterValue(Enumerations::cO, Enumerations::max),'f',5));
    prevState = ui->cO->blockSignals(true);
    ui->cO->setMaximum((int)(floor(dst->pars->getParameterValue(Enumerations::cO, Enumerations::max)*1.E5)));
    ui->cO->setMinimum((int)(floor(dst->pars->getParameterValue(Enumerations::cO, Enumerations::min)*1.E5)));
    set_cO_min_max();
    ui->cO->setValue((int)(floor(dst->pars->getParameterValue(Enumerations::cO, Enumerations::val) * 1.E5)));
    ui->cO->blockSignals(prevState);
    ui->cO_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::cO, Enumerations::val),'f',5));

    ui->dan_sub->setText(QString::number(dst->pars->getParameterValue(Enumerations::dan, Enumerations::dec),'f',5));
    ui->dan_add->setText(QString::number(dst->pars->getParameterValue(Enumerations::dan, Enumerations::inc),'f',5));
    ui->dan_min->setText(QString::number(dst->pars->getParameterValue(Enumerations::dan, Enumerations::min),'f',5));
    ui->dan_max->setText(QString::number(dst->pars->getParameterValue(Enumerations::dan, Enumerations::max),'f',5));
    prevState = ui->dan->blockSignals(true);
    ui->dan->setMaximum((int)(floor(dst->pars->getParameterValue(Enumerations::dan, Enumerations::max)*1.E5)));
    ui->dan->setMinimum((int)(floor(dst->pars->getParameterValue(Enumerations::dan, Enumerations::min)*1.E5)));
    set_dan_min_max();
    ui->dan->setValue((int)(floor(dst->pars->getParameterValue(Enumerations::dan, Enumerations::val) * 1.E5)));
    ui->dan->blockSignals(prevState);
    ui->dan_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::dan, Enumerations::val),'f',5));

    ui->k_sub->setText(QString::number(dst->pars->getParameterValue(Enumerations::k, Enumerations::dec),'f',3));
    ui->k_add->setText(QString::number(dst->pars->getParameterValue(Enumerations::k, Enumerations::inc),'f',3));
    ui->k_min->setText(QString::number(dst->pars->getParameterValue(Enumerations::k, Enumerations::min),'f',3));
    ui->k_max->setText(QString::number(dst->pars->getParameterValue(Enumerations::k, Enumerations::max),'f',3));
    prevState = ui->k->blockSignals(true);
    ui->k->setMaximum((int)(floor(dst->pars->getParameterValue(Enumerations::k, Enumerations::max)*1.E3)));
    ui->k->setMinimum((int)(floor(dst->pars->getParameterValue(Enumerations::k, Enumerations::min)*1.E3)));
    set_k_min_max();
    ui->k->setValue((int)(floor(dst->pars->getParameterValue(Enumerations::k, Enumerations::val) * 1.E3)));
    ui->k->blockSignals(prevState);
    ui->k_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::k, Enumerations::val),'f',3));

    ui->const1_sub->setText(QString::number(dst->pars->getParameterValue(Enumerations::const1, Enumerations::dec),'f',3));
    ui->const1_add->setText(QString::number(dst->pars->getParameterValue(Enumerations::const1, Enumerations::inc),'f',3));
    ui->const1_min->setText(QString::number(dst->pars->getParameterValue(Enumerations::const1, Enumerations::min),'f',3));
    ui->const1_max->setText(QString::number(dst->pars->getParameterValue(Enumerations::const1, Enumerations::max),'f',3));
    prevState = ui->const1->blockSignals(true);
    ui->const1->setMaximum((int)(floor(dst->pars->getParameterValue(Enumerations::const1, Enumerations::max)*1.E3)));
    ui->const1->setMinimum((int)(floor(dst->pars->getParameterValue(Enumerations::const1, Enumerations::min)*1.E3)));
    set_const1_min_max();
    ui->const1->setValue((int)(floor(dst->pars->getParameterValue(Enumerations::const1, Enumerations::val) * 1.E3)));
    ui->const1->blockSignals(prevState);
    ui->const1_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::const1, Enumerations::val),'f',3));

    ui->const2_sub->setText(QString::number(dst->pars->getParameterValue(Enumerations::const2, Enumerations::dec),'f',5));
    ui->const2_add->setText(QString::number(dst->pars->getParameterValue(Enumerations::const2, Enumerations::inc),'f',5));
    ui->const2_min->setText(QString::number(dst->pars->getParameterValue(Enumerations::const2, Enumerations::min),'f',5));
    ui->const2_max->setText(QString::number(dst->pars->getParameterValue(Enumerations::const2, Enumerations::max),'f',5));
    prevState = ui->const2->blockSignals(true);
    ui->const2->setMaximum((int)(floor(dst->pars->getParameterValue(Enumerations::const2, Enumerations::max)*1.E5)));
    ui->const2->setMinimum((int)(floor(dst->pars->getParameterValue(Enumerations::const2, Enumerations::min)*1.E5)));
    set_const2_min_max();
    ui->const2->setValue((int)(floor(dst->pars->getParameterValue(Enumerations::const2, Enumerations::val) * 1.E5)));
    ui->const2->blockSignals(prevState);
    ui->const2_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::const2, Enumerations::val),'f',5));

    ui->g_sub->setText(QString::number(dst->pars->getParameterValue(Enumerations::g, Enumerations::dec),'f',5));
    ui->g_add->setText(QString::number(dst->pars->getParameterValue(Enumerations::g, Enumerations::inc),'f',5));
    ui->g_min->setText(QString::number(dst->pars->getParameterValue(Enumerations::g, Enumerations::min),'f',5));
    ui->g_max->setText(QString::number(dst->pars->getParameterValue(Enumerations::g, Enumerations::max),'f',5));
    prevState = ui->g->blockSignals(true);
    ui->g->setMaximum((int)(floor(dst->pars->getParameterValue(Enumerations::g, Enumerations::max)*1.E5)));
    ui->g->setMinimum((int)(floor(dst->pars->getParameterValue(Enumerations::g, Enumerations::min)*1.E5)));
    set_g_min_max();
    ui->g->setValue((int)(floor(dst->pars->getParameterValue(Enumerations::g, Enumerations::val) * 1.E5)));
    ui->g->blockSignals(prevState);
    ui->g_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::g, Enumerations::val),'f',5));

    ui->mue_ab_sub->setText(QString::number(dst->pars->getParameterValue(Enumerations::mue_ab, Enumerations::dec),'f',5));
    ui->mue_ab_add->setText(QString::number(dst->pars->getParameterValue(Enumerations::mue_ab, Enumerations::inc),'f',5));
    ui->mue_ab_min->setText(QString::number(dst->pars->getParameterValue(Enumerations::mue_ab, Enumerations::min),'f',5));
    ui->mue_ab_max->setText(QString::number(dst->pars->getParameterValue(Enumerations::mue_ab, Enumerations::max),'f',5));
    prevState = ui->mue_ab->blockSignals(true);
    ui->mue_ab->setMaximum((int)(floor(dst->pars->getParameterValue(Enumerations::mue_ab, Enumerations::max)*1.E5)));
    ui->mue_ab->setMinimum((int)(floor(dst->pars->getParameterValue(Enumerations::mue_ab, Enumerations::min)*1.E5)));
    set_mue_ab_min_max();
    ui->mue_ab->setValue((int)(floor(dst->pars->getParameterValue(Enumerations::mue_ab, Enumerations::val) * 1.E5)));
    ui->mue_ab->blockSignals(prevState);
    ui->mue_ab_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::mue_ab, Enumerations::val),'f',5));

    ui->p_sub->setText(QString::number(dst->pars->getParameterValue(Enumerations::p, Enumerations::dec),'f',5));
    ui->p_add->setText(QString::number(dst->pars->getParameterValue(Enumerations::p, Enumerations::inc),'f',5));
    ui->p_min->setText(QString::number(dst->pars->getParameterValue(Enumerations::p, Enumerations::min),'f',5));
    ui->p_max->setText(QString::number(dst->pars->getParameterValue(Enumerations::p, Enumerations::max),'f',5));
    prevState = ui->p->blockSignals(true);
    ui->p->setMaximum((int)(floor(dst->pars->getParameterValue(Enumerations::p, Enumerations::max)*1.E5)));
    ui->p->setMinimum((int)(floor(dst->pars->getParameterValue(Enumerations::p, Enumerations::min)*1.E5)));
    set_p_min_max();
    ui->p->setValue((int)(floor(dst->pars->getParameterValue(Enumerations::p, Enumerations::val) * 1.E5)));
    ui->p->blockSignals(prevState);
    ui->p_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::p, Enumerations::val),'f',5));

    ui->R_sub->setText(QString::number(dst->pars->getParameterValue(Enumerations::R, Enumerations::dec),'f',5));
    ui->R_add->setText(QString::number(dst->pars->getParameterValue(Enumerations::R, Enumerations::inc),'f',5));
    ui->R_min->setText(QString::number(dst->pars->getParameterValue(Enumerations::R, Enumerations::min),'f',5));
    ui->R_max->setText(QString::number(dst->pars->getParameterValue(Enumerations::R, Enumerations::max),'f',5));
    prevState = ui->R->blockSignals(true);
    ui->R->setMaximum((int)(floor(dst->pars->getParameterValue(Enumerations::R, Enumerations::max)*1.E5)));
    ui->R->setMinimum((int)(floor(dst->pars->getParameterValue(Enumerations::R, Enumerations::min)*1.E5)));
    set_R_min_max();
    ui->R->setValue((int)(floor(dst->pars->getParameterValue(Enumerations::R, Enumerations::val) * 1.E5)));
    ui->R->blockSignals(prevState);
    ui->R_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::R, Enumerations::val),'f',5));

    ui->t_sub->setText(QString::number(dst->pars->getParameterValue(Enumerations::t, Enumerations::dec),'f',5));
    ui->t_add->setText(QString::number(dst->pars->getParameterValue(Enumerations::t, Enumerations::inc),'f',5));
    ui->t_min->setText(QString::number(dst->pars->getParameterValue(Enumerations::t, Enumerations::min),'f',5));
    ui->t_max->setText(QString::number(dst->pars->getParameterValue(Enumerations::t, Enumerations::max),'f',5));
    prevState = ui->t->blockSignals(true);
    ui->t->setMaximum((int)(floor(dst->pars->getParameterValue(Enumerations::t, Enumerations::max)*1.E5)));
    ui->t->setMinimum((int)(floor(dst->pars->getParameterValue(Enumerations::t, Enumerations::min)*1.E5)));
    set_t_min_max();
    ui->t->setValue((int)(floor(dst->pars->getParameterValue(Enumerations::t, Enumerations::val) * 1.E5)));
    ui->t->blockSignals(prevState);
    ui->t_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::t, Enumerations::val),'f',5));

    sst->checkParameters_MainWindow = true; //re-enable check on user input
}


void MainWindow::set_widget_values(bool startup){

    if (dst->dataPath == 0)
        ui->buttonLoad->setText(tr("Datensatz laden"));
    else{
        if (sst->prgset->displayFullPath)
            ui->buttonLoad->setText(tr("Datensatz laden  ---  aktuell geladen: ") + *(dst->dataPath));
        else{
            QStringList splitted = dst->dataPath->split("/");
            ui->buttonLoad->setText(tr("Datensatz laden  ---  aktuell geladen: ") + splitted.at(splitted.size() - 1));
        }
    }

    setWindowCaption();

    ui->buttonDrawData->setChecked(sst->drawData);
    ui->buttonDrawFit->setChecked(sst->drawFit);
    ui->buttonShowDeviation->setChecked(sst->drawDev);
    check_uncheck_buttons();

    if (sst->prgset->adjustAxisIntervals || startup){
        ui->plot_minX->setText(QString::number(dst->prgpar->plotMinX,'f',5));
        ui->plot_maxX->setText(QString::number(dst->prgpar->plotMaxX,'f',5));
        ui->plot->setAxisScale(QwtPlot::xBottom, dst->prgpar->plotMinX, dst->prgpar->plotMaxX);
        ui->plot->setAxisScale(QwtPlot::yLeft, dst->prgpar->plotMinY, dst->prgpar->plotMaxY);
        ui->checkBox_Autoscale->setChecked(dst->prgpar->autoScaleYAxis);
        ui->plot->setAxisAutoScale(QwtPlot::yLeft, dst->prgpar->autoScaleYAxis);
        ui->plot->replot();
        if (dst->prgpar->autoScaleYAxis){
            dst->prgpar->plotMinY = ui->plot->axisInterval(QwtPlot::yLeft).minValue();
            dst->prgpar->plotMaxY = ui->plot->axisInterval(QwtPlot::yLeft).maxValue();
        }
        ui->plot_minY->setText(QString::number(dst->prgpar->plotMinY,'f',3));
        ui->plot_maxY->setText(QString::number(dst->prgpar->plotMaxY,'f',3));
    }
    else
        if (ui->plot_maxX->text().toDouble() > (floor(( (2 * sin(70 * M_PI / 180)) / (dst->prgpar->currentWavelength->value) ) * 10.) / 10.)){
            ui->plot_maxX->setText(QString::number((floor(( (2 * sin(70 * M_PI / 180)) / (dst->prgpar->currentWavelength->value) ) * 10.) / 10.),'f',5));
            ui->plot->setAxisScale(QwtPlot::xBottom, ui->plot_minX->text().toDouble(), ui->plot_maxX->text().toDouble());
        }

    if (!(sst->prgset->simpleMode))
        ui->spinBoxThinningFactor->setValue(dst->prgpar->thinningFactor);

    ui->r->setText(QString::number(dst->prgpar->par_r,'f',4));
    ui->d->setText(QString::number(dst->prgpar->par_d,'f',4));
    ui->b->setText(QString::number(dst->prgpar->par_b,'f',5));
    ui->divergenceAngle->setText(QString::number(dst->prgpar->par_delta,'f',3));
    ui->irradiatedLength->setText(QString::number(dst->prgpar->par_l,'f',4));

    ui->checkBox_optQ->setChecked(dst->prgpar->useQ);
    ui->checkBox_optA->setChecked(dst->prgpar->useA);
    ui->checkBox_optGradient->setChecked(dst->prgpar->useGradient);
    ui->checkBox_optCorrAutoColl->setChecked(dst->prgpar->useCorrAutoColl);

    ui->checkBox_optPosition->setChecked(dst->prgpar->corrPosition);
    bool prevState = ui->comboBox_Position->blockSignals(true); //save previous state
    ui->comboBox_Position->setCurrentIndex(dst->prgpar->corrPositionMethod);
    ui->comboBox_Position->blockSignals(prevState); //restore previous state

    ui->checkBox_optIntensity->setChecked(dst->prgpar->corrIntensity);
    prevState = ui->comboBox_Intensity->blockSignals(true); //save previous state
    ui->comboBox_Intensity->setCurrentIndex(dst->prgpar->corrIntensityMethod);
    ui->comboBox_Intensity->blockSignals(prevState); //restore previous state

    setEnabledDisabled();

    prevState = sst->prgset->autoUpdateFit; //save previous state
    sst->prgset->autoUpdateFit = true; //temporary so that calling draw() updates the plot
    draw(false);
    sst->prgset->autoUpdateFit = prevState; //restore previous state
    if (sst->drawDev){
        if (dev->isHidden())
            dev->show();
        else
            dev->updatePlot();
    }
    else
        dev->hide();

    dev->setInterpolation(dst->prgpar->showInterpolation);
    dev->setFollowChanges(dst->prgpar->followChanges);
}


void MainWindow::check_uncheck_buttons(){
    if (sst->drawData && sst->drawFit && sst->drawDev)
        ui->buttonDrawAll->setChecked(true);
    else
        ui->buttonDrawAll->setChecked(false);
    if( !(sst->drawData && sst->drawFit) && sst->drawDev){
        ui->buttonShowDeviation->setChecked(false);
        sst->drawDev = false;
        dev->hide();
        sst->reCheck_buttonShowDeviation_MainWindow = true;
    }
    if (sst->drawData && sst->drawFit && sst->reCheck_buttonShowDeviation_MainWindow && ( !(sst->drawDev) ) ){
        ui->buttonShowDeviation->setChecked(true);
        sst->drawDev = true;
        dev->show();
        sst->reCheck_buttonShowDeviation_MainWindow = true;
    }
    ui->action_ChooseTemplate->setEnabled(sst->drawData);
}

void MainWindow:: setEnabledDisabled(){ //only enable relevant parameters for chosen fit-options and data-correction
    ui->b->setEnabled(ui->checkBox_optQ->isChecked());

    if (ui->checkBox_optA->isChecked() || (ui->checkBox_optPosition->isChecked() && ui->comboBox_Position->currentIndex() == 0) || (ui->checkBox_optIntensity->isChecked() && ui->comboBox_Intensity->currentIndex() == 0))
        ui->d->setEnabled(true);
    else
        ui->d->setDisabled(true);

    if ( (ui->checkBox_optPosition->isChecked() && ui->comboBox_Position->currentIndex() == 0) || ui->checkBox_optCorrAutoColl->isChecked() )
        ui->r->setEnabled(true);
    else
        ui->r->setDisabled(true);

    ui->divergenceAngle->setEnabled(ui->checkBox_optCorrAutoColl->isChecked());

    ui->irradiatedLength->setEnabled(ui->checkBox_optCorrAutoColl->isChecked());

    ui->widget_g->setEnabled(ui->checkBox_optGradient->isChecked());

    if (ui->checkBox_optA->isChecked() || (ui->checkBox_optPosition->isChecked() && (ui->comboBox_Position->currentIndex() == 0 || ui->comboBox_Position->currentIndex() == 1) ) || (ui->checkBox_optIntensity->isChecked() && ui->comboBox_Intensity->currentIndex() == 0) )
        ui->widget_mue_ab->setEnabled(true);
    else
        ui->widget_mue_ab->setDisabled(true);

    if (ui->checkBox_optPosition->isChecked() && ui->comboBox_Position->currentIndex() == 1){
        ui->widget_R->setEnabled(true);
        ui->widget_t->setEnabled(true);
    }
    else{
        ui->widget_R->setDisabled(true);
        ui->widget_t->setDisabled(true);
    }

    if (ui->checkBox_optIntensity->isChecked() && ui->comboBox_Intensity->currentIndex() == 1)
        ui->widget_p->setEnabled(true);
    else
        ui->widget_p->setDisabled(true);

    ui->comboBox_Position->setEnabled(dst->prgpar->corrPosition);
    ui->comboBox_Intensity->setEnabled(dst->prgpar->corrIntensity);
}


void MainWindow::draw(bool mustUpdateDeviation){

    if (sst->drawData){
        datacal->calcDataCurve(dst->data_cor, dst->data, dst->prgpar->corrPosition, dst->prgpar->corrPositionMethod, dst->prgpar->corrIntensity, dst->prgpar->corrIntensityMethod, dst->pars, dst->prgpar->par_r, dst->prgpar->par_d, dst->prgpar->currentWavelength->value, &(dst->data_for_ChooseTemplate_AutoFit), dst->prgpar->plotMinX, dst->prgpar->plotMaxX);
        curveData->setData(new QwtPointSeriesData(*(dst->data_cor)));
    }

    if (sst->drawFit){
        if (sst->prgset->autoUpdateFit){
            datacal->calcFitCurve(dst->fit, dst->plotPars_MainWindow, dst->prgpar->plotMinX, dst->prgpar->plotMaxX, ui->spinBoxThinningFactor->value(), dst->prgpar->currentWavelength->value, dst->prgpar->useA, dst->prgpar->par_d, dst->prgpar->useGradient, dst->prgpar->useCorrAutoColl, dst->prgpar->par_r, dst->prgpar->par_delta, dst->prgpar->par_l, dst->prgpar->useQ, dst->prgpar->par_b);
            curveFit->setData(new QwtPointSeriesData(*(dst->fit)));
        }

    }

    updateCurves();

    if (dst->prgpar->autoScaleYAxis){
        dst->prgpar->plotMinY = ui->plot->axisInterval(QwtPlot::yLeft).minValue();
        dst->prgpar->plotMaxY = ui->plot->axisInterval(QwtPlot::yLeft).maxValue();
        ui->plot_minY->setText(QString::number(dst->prgpar->plotMinY,'f',3));
        ui->plot_maxY->setText(QString::number(dst->prgpar->plotMaxY,'f',3));
    }

    if (sst->prgset->autoUpdateFit && dst->prgpar->followChanges && mustUpdateDeviation)
        dev->updatePlot();

}


void MainWindow::updateCurves(){
    delete sizeData;
    sizeData = new QSize(sst->prgset->thicknessOfCurveDots,sst->prgset->thicknessOfCurveDots);
    symbolData = new QwtSymbol(QwtSymbol::Ellipse, *brushData, *penData, *sizeData);
    curveData->setSymbol(symbolData);

    delete penFit;
    penFit = new QPen(*colorFit);
    penFit->setWidth(sst->prgset->thicknessOfCurveLines);
    curveFit->setPen(*penFit);

    curveData->detach();
    curveFit->detach();
    switch(sst->prgset->elementInFront){
        case Enumerations::dotsInFront :
            if (sst->drawFit)
                curveFit->attach(ui->plot);
            if (sst->drawData)
                curveData->attach(ui->plot);
            break;
        case Enumerations::lineInFront :
            if (sst->drawData)
                curveData->attach(ui->plot);
            if (sst->drawFit)
                curveFit->attach(ui->plot);
            break;
        default:
            if (sst->drawFit)
                curveFit->attach(ui->plot);
            if (sst->drawData)
                curveData->attach(ui->plot);
            break;
    }

    ui->plot->replot();
}


void MainWindow::setWindowCaption(){
    if (dst->parsPath == 0)
            setWindowTitle(tr("CarbX - Carbon X-Ray Calculator"));
    else{
        if (sst->prgset->displayFullPath)
            setWindowTitle(tr("CarbX - Carbon X-Ray Calculator") + " --- " + *(dst->parsPath));
        else{
            QStringList splitted = dst->parsPath->split("/");
            setWindowTitle(tr("CarbX - Carbon X-Ray Calculator") + " --- " + splitted.at(splitted.size() - 1));
        }
    }
}


void MainWindow::chooseTemplate(){
    Dialog_chooseTemplate * dlg = new Dialog_chooseTemplate(datacal, dst, sst, pfa, dst->data_for_ChooseTemplate_AutoFit);
    dlg->exec();
    delete dlg;
    if (dst->chosenTemplate > 0)
        loadTemplate(dst->chosenTemplate);
    else
        ui->action_ValuesBackToTemplate->setDisabled(true);
}

void MainWindow::loadTemplate(short templateNumber){
    bool useA;
    bool useGradient;
    QString *templateName = 0;
    if (pfa->readParsTPL(dst->pars, useA, useGradient, &templateName, new QString(*(dst->applicationDirPath) + "/data/templates/" + QString::number(templateNumber) + ".tpl"), dst, false)){
        set_slider_values();
        pfa->readParsTPL(dst->pars, useA, useGradient, &templateName, new QString(*(dst->applicationDirPath) + "/data/templates/" + QString::number(templateNumber) + ".tpl"), dst, false);
        set_slider_values();//set slider values twice due to an issue with QSlider
        if (useGradient != dst->prgpar->useGradient){
            ui->checkBox_optGradient->setChecked(useGradient);
            dst->prgpar->useGradient = useGradient;
        }
        if (useA != dst->prgpar->useA){
            if (useA && dst->prgpar->corrIntensity){
                QMessageBox *msg = new QMessageBox(this);
                msg->setIcon(QMessageBox::Warning);
                msg->setText("Fuer das gewaehlte Template ist der Absorptionsfaktor\nfuer die Anpassungskurve aktiviert. Bei Beibehaltung\nwird die Intensitaetskorrektur der Messwerte deaktiviert.\nSoll der Absorptionsfaktor beibehalten werden?");
                msg->setStandardButtons(QMessageBox::Yes | QMessageBox::No);
                msg->setDefaultButton(QMessageBox::No);
                int ret = msg->exec();
                switch (ret) {
                   case QMessageBox::Yes:
                       break;
                   case QMessageBox::No:
                    useA = false;
                    break;
                   default:
                       break;
                }
                delete msg;
            }
            if (useA != dst->prgpar->useA){ //useA has maybe changed
                ui->checkBox_optA->setChecked(useA);
                on_checkBox_optA_clicked(useA);
                if (useA)
                    sst->reCheck_optIntensity_MainWindow = false;
            }
        }
        setEnabledDisabled();
        sst->prgset->autoUpdateFit = true; //temporary so that calling draw() updates the plot
        draw(true);
        bool prevState = sst->prgset->autoUpdateFit; //save previous state
        sst->prgset->autoUpdateFit = prevState; //restore previous state
        sst->mustSave = true;
        ui->action_ValuesBackToTemplate->setEnabled(true);
    }
    delete templateName;
}


bool MainWindow::saveMaybe(){
    if (sst->mustSave){
        QMessageBox *msg = new QMessageBox(this);
        msg->setIcon(QMessageBox::Question);
        msg->setText(tr("Moechten Sie die aktuelle Anpassung speichern?"));
        msg->setStandardButtons(QMessageBox::Save | QMessageBox::Discard | QMessageBox::Cancel);
        msg->setDefaultButton(QMessageBox::Save);
        int ret = msg->exec();
        delete msg;
        switch (ret) {
           case QMessageBox::Save:
               ui->action_Save->trigger();
               return true;
               break;
           case QMessageBox::Discard:
               return true;
               break;
           case QMessageBox::Cancel:
               return false;
               break;
           default:
               return false;
               break;
         }
    }
    else
        return true;
}



//Events


void MainWindow::resizeEvent(QResizeEvent *event){
    //TODO

    //only temporary to avoid unused parameter warning
    if (event->oldSize().width() == event->size().width() && event->oldSize().height() == event->size().height())
        event->accept();
    else
        event->ignore();
}


void MainWindow::closeEvent(QCloseEvent *event){
    if ( (afs == 0) && saveMaybe()){
        dev->setCloseAllowed();
        dev->close();
        dst->writeWavelengths(ui->comboBox_Wavelengths->currentIndex());
        if (!(sst->writeProgramSettings()))
            AuxiliaryMethods::errorMessage(this, new QString(tr("Speichern der Programmeinstellungen fehlgeschlagen!")));
        event->accept();
    }
    else
        event->ignore();
}



//Slider-Functionality (slots and one private method for each parameter)


//cno

void MainWindow::on_cno_min_editingFinished(){
    bool prevState = ui->cno_min->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->cno_min->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if(temp <= dst->pars->getParameterValue(Enumerations::cno, Enumerations::max)){
            dst->pars->setParameterValue(Enumerations::cno, Enumerations::min, temp, sst->checkParameters_MainWindow);
            ui->cno->setMinimum((int)(floor(dst->pars->getParameterValue(Enumerations::cno, Enumerations::min)*1.E5)));
            set_cno_min_max();
        }
    }
    ui->cno_min->setText(QString::number(dst->pars->getParameterValue(Enumerations::cno, Enumerations::min),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->cno_min->blockSignals(prevState); //restore previous state
}

void MainWindow::on_cno_max_editingFinished(){
    bool prevState = ui->cno_max->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->cno_max->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if(temp >= dst->pars->getParameterValue(Enumerations::cno, Enumerations::min)){
            dst->pars->setParameterValue(Enumerations::cno, Enumerations::max, temp, sst->checkParameters_MainWindow);
            ui->cno->setMaximum((int)(floor(dst->pars->getParameterValue(Enumerations::cno, Enumerations::max)*1.E5)));
            set_cno_min_max();
        }
    }
    ui->cno_max->setText(QString::number(dst->pars->getParameterValue(Enumerations::cno, Enumerations::max),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->cno_max->blockSignals(prevState); //restore previous state
}

void MainWindow::on_cno_sub_editingFinished(){
    bool prevState = ui->cno_sub->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->cno_sub->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp >= 0)
            dst->pars->setParameterValue(Enumerations::cno, Enumerations::dec, temp, sst->checkParameters_MainWindow);
    }
    ui->cno_sub->setText(QString::number(dst->pars->getParameterValue(Enumerations::cno, Enumerations::dec),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->cno_sub->blockSignals(prevState); //restore previous state
}

void MainWindow::on_cno_add_editingFinished(){
    bool prevState = ui->cno_add->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->cno_add->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp >= 0)
            dst->pars->setParameterValue(Enumerations::cno, Enumerations::inc, temp, sst->checkParameters_MainWindow);
    }
    ui->cno_add->setText(QString::number(dst->pars->getParameterValue(Enumerations::cno, Enumerations::inc),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->cno_add->blockSignals(prevState); //restore previous state
}

void MainWindow::on_cno_value_editingFinished(){
    bool prevState = ui->cno_value->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->cno_value->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        ui->cno->setValue((int)(floor(temp * 1.E5)));
    }
    ui->cno_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::cno, Enumerations::val),'f',5));
    delete tempText;
    ui->cno_value->blockSignals(prevState); //restore previous state
}

void MainWindow::on_cno_doSub_clicked(){
    ui->cno->setValue((int)(floor((dst->pars->getParameterValue(Enumerations::cno, Enumerations::val) - dst->pars->getParameterValue(Enumerations::cno, Enumerations::dec)) * 1.E5)));
}

void MainWindow::on_cno_doAdd_clicked(){
    ui->cno->setValue((int)(floor((dst->pars->getParameterValue(Enumerations::cno, Enumerations::val) + dst->pars->getParameterValue(Enumerations::cno, Enumerations::inc)) * 1.E5)));
}

void MainWindow::on_cno_valueChanged(int value){
    dst->pars->setParameterValue(Enumerations::cno, Enumerations::val, (value / 1.E5), sst->checkParameters_MainWindow);
    bool prevState = ui->cno->blockSignals(true);
    ui->cno->setValue((int)(floor(dst->pars->getParameterValue(Enumerations::cno, Enumerations::val) * 1.E5)));
    ui->cno->blockSignals(prevState);
    ui->cno_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::cno, Enumerations::val),'f',5));
    draw(true);
    sst->mustSave = true;
}

void MainWindow::set_cno_min_max(){
    ui->cno->setTickInterval((ui->cno->maximum()-ui->cno->minimum()) / 10);
    ui->cno->setSingleStep(ui->cno->tickInterval());
    ui->cno->setPageStep(ui->cno->tickInterval()*2);
}


//mu

void MainWindow::on_mu_min_editingFinished(){
    bool prevState = ui->mu_min->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->mu_min->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if(temp <= dst->pars->getParameterValue(Enumerations::mu, Enumerations::max)){
            dst->pars->setParameterValue(Enumerations::mu, Enumerations::min, temp, sst->checkParameters_MainWindow);
            ui->mu->setMinimum((int)(floor(dst->pars->getParameterValue(Enumerations::mu, Enumerations::min)*1.E5)));
            set_mu_min_max();
        }
    }
    ui->mu_min->setText(QString::number(dst->pars->getParameterValue(Enumerations::mu, Enumerations::min),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->mu_min->blockSignals(prevState); //restore previous state
}

void MainWindow::on_mu_max_editingFinished(){
    bool prevState = ui->mu_max->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->mu_max->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if(temp >= dst->pars->getParameterValue(Enumerations::mu, Enumerations::min)){
            dst->pars->setParameterValue(Enumerations::mu, Enumerations::max, temp, sst->checkParameters_MainWindow);
            ui->mu->setMaximum((int)(floor(dst->pars->getParameterValue(Enumerations::mu, Enumerations::max)*1.E5)));
            set_mu_min_max();
        }
    }
    ui->mu_max->setText(QString::number(dst->pars->getParameterValue(Enumerations::mu, Enumerations::max),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->mu_max->blockSignals(prevState); //restore previous state
}

void MainWindow::on_mu_sub_editingFinished(){
    bool prevState = ui->mu_sub->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->mu_sub->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp >= 0)
            dst->pars->setParameterValue(Enumerations::mu, Enumerations::dec, temp, sst->checkParameters_MainWindow);
    }
    ui->mu_sub->setText(QString::number(dst->pars->getParameterValue(Enumerations::mu, Enumerations::dec),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->mu_sub->blockSignals(prevState); //restore previous state
}

void MainWindow::on_mu_add_editingFinished(){
    bool prevState = ui->mu_add->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->mu_add->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp >= 0)
            dst->pars->setParameterValue(Enumerations::mu, Enumerations::inc, temp, sst->checkParameters_MainWindow);
    }
    ui->mu_add->setText(QString::number(dst->pars->getParameterValue(Enumerations::mu, Enumerations::inc),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->mu_add->blockSignals(prevState); //restore previous state
}

void MainWindow::on_mu_value_editingFinished(){
    bool prevState = ui->mu_value->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->mu_value->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        ui->mu->setValue((int)(floor(temp * 1.E5)));
    }
    ui->mu_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::mu, Enumerations::val),'f',5));
    delete tempText;
    ui->mu_value->blockSignals(prevState); //restore previous state
}

void MainWindow::on_mu_doSub_clicked(){
    ui->mu->setValue((int)(floor((dst->pars->getParameterValue(Enumerations::mu, Enumerations::val) - dst->pars->getParameterValue(Enumerations::mu, Enumerations::dec)) * 1.E5)));
}

void MainWindow::on_mu_doAdd_clicked(){
    ui->mu->setValue((int)(floor((dst->pars->getParameterValue(Enumerations::mu, Enumerations::val) + dst->pars->getParameterValue(Enumerations::mu, Enumerations::inc)) * 1.E5)));
}

void MainWindow::on_mu_valueChanged(int value){
    dst->pars->setParameterValue(Enumerations::mu, Enumerations::val, (value / 1.E5), sst->checkParameters_MainWindow);
    bool prevState = ui->mu->blockSignals(true);
    ui->mu->setValue((int)(floor(dst->pars->getParameterValue(Enumerations::mu, Enumerations::val) * 1.E5)));
    ui->mu->blockSignals(prevState);
    ui->mu_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::mu, Enumerations::val),'f',5));
    draw(true);
    sst->mustSave = true;
}

void MainWindow::set_mu_min_max(){
    ui->mu->setTickInterval((ui->mu->maximum()-ui->mu->minimum()) / 10);
    ui->mu->setSingleStep(ui->mu->tickInterval());
    ui->mu->setPageStep(ui->mu->tickInterval()*2);
}


//beta

void MainWindow::on_beta_min_editingFinished(){
    bool prevState = ui->beta_min->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->beta_min->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if(temp <= dst->pars->getParameterValue(Enumerations::beta, Enumerations::max)){
            dst->pars->setParameterValue(Enumerations::beta, Enumerations::min, temp, sst->checkParameters_MainWindow);
            ui->beta->setMinimum((int)(floor(dst->pars->getParameterValue(Enumerations::beta, Enumerations::min)*1.E5)));
            set_beta_min_max();
        }
    }
    ui->beta_min->setText(QString::number(dst->pars->getParameterValue(Enumerations::beta, Enumerations::min),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->beta_min->blockSignals(prevState); //restore previous state
}

void MainWindow::on_beta_max_editingFinished(){
    bool prevState = ui->beta_max->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->beta_max->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if(temp >= dst->pars->getParameterValue(Enumerations::beta, Enumerations::min)){
            dst->pars->setParameterValue(Enumerations::beta, Enumerations::max, temp, sst->checkParameters_MainWindow);
            ui->beta->setMaximum((int)(floor(dst->pars->getParameterValue(Enumerations::beta, Enumerations::max)*1.E5)));
            set_beta_min_max();
        }
    }
    ui->beta_max->setText(QString::number(dst->pars->getParameterValue(Enumerations::beta, Enumerations::max),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->beta_max->blockSignals(prevState); //restore previous state
}

void MainWindow::on_beta_sub_editingFinished(){
    bool prevState = ui->beta_sub->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->beta_sub->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp >= 0)
            dst->pars->setParameterValue(Enumerations::beta, Enumerations::dec, temp, sst->checkParameters_MainWindow);
    }
    ui->beta_sub->setText(QString::number(dst->pars->getParameterValue(Enumerations::beta, Enumerations::dec),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->beta_sub->blockSignals(prevState); //restore previous state
}

void MainWindow::on_beta_add_editingFinished(){
    bool prevState = ui->beta_add->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->beta_add->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp >= 0)
            dst->pars->setParameterValue(Enumerations::beta, Enumerations::inc, temp, sst->checkParameters_MainWindow);
    }
    ui->beta_add->setText(QString::number(dst->pars->getParameterValue(Enumerations::beta, Enumerations::inc),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->beta_add->blockSignals(prevState); //restore previous state
}

void MainWindow::on_beta_value_editingFinished(){
    bool prevState = ui->beta_value->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->beta_value->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        ui->beta->setValue((int)(floor(temp * 1.E5)));
    }
    ui->beta_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::beta, Enumerations::val),'f',5));
    delete tempText;
    ui->beta_value->blockSignals(prevState); //restore previous state
}

void MainWindow::on_beta_doSub_clicked(){
    ui->beta->setValue((int)(floor((dst->pars->getParameterValue(Enumerations::beta, Enumerations::val) - dst->pars->getParameterValue(Enumerations::beta, Enumerations::dec)) * 1.E5)));
}

void MainWindow::on_beta_doAdd_clicked(){
    ui->beta->setValue((int)(floor((dst->pars->getParameterValue(Enumerations::beta, Enumerations::val) + dst->pars->getParameterValue(Enumerations::beta, Enumerations::inc)) * 1.E5)));
}

void MainWindow::on_beta_valueChanged(int value){
    dst->pars->setParameterValue(Enumerations::beta, Enumerations::val, (value / 1.E5), sst->checkParameters_MainWindow);
    bool prevState = ui->beta->blockSignals(true);
    ui->beta->setValue((int)(floor(dst->pars->getParameterValue(Enumerations::beta, Enumerations::val) * 1.E5)));
    ui->beta->blockSignals(prevState);
    ui->beta_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::beta, Enumerations::val),'f',5));
    draw(true);
    sst->mustSave = true;
}

void MainWindow::set_beta_min_max(){
    ui->beta->setTickInterval((ui->beta->maximum()-ui->beta->minimum()) / 10);
    ui->beta->setSingleStep(ui->beta->tickInterval());
    ui->beta->setPageStep(ui->beta->tickInterval()*2);
}


//a3

void MainWindow::on_a3_min_editingFinished(){
    bool prevState = ui->a3_min->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->a3_min->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if(temp <= dst->pars->getParameterValue(Enumerations::a3, Enumerations::max)){
            dst->pars->setParameterValue(Enumerations::a3, Enumerations::min, temp, sst->checkParameters_MainWindow);
            ui->a3->setMinimum((int)(floor(dst->pars->getParameterValue(Enumerations::a3, Enumerations::min)*1.E5)));
            set_a3_min_max();
        }
    }
    ui->a3_min->setText(QString::number(dst->pars->getParameterValue(Enumerations::a3, Enumerations::min),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->a3_min->blockSignals(prevState); //restore previous state
}

void MainWindow::on_a3_max_editingFinished(){
    bool prevState = ui->a3_max->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->a3_max->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if(temp >= dst->pars->getParameterValue(Enumerations::a3, Enumerations::min)){
            dst->pars->setParameterValue(Enumerations::a3, Enumerations::max, temp, sst->checkParameters_MainWindow);
            ui->a3->setMaximum((int)(floor(dst->pars->getParameterValue(Enumerations::a3, Enumerations::max)*1.E5)));
            set_a3_min_max();
        }
    }
    ui->a3_max->setText(QString::number(dst->pars->getParameterValue(Enumerations::a3, Enumerations::max),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->a3_max->blockSignals(prevState); //restore previous state
}

void MainWindow::on_a3_sub_editingFinished(){
    bool prevState = ui->a3_sub->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->a3_sub->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp >= 0)
            dst->pars->setParameterValue(Enumerations::a3, Enumerations::dec, temp, sst->checkParameters_MainWindow);
    }
    ui->a3_sub->setText(QString::number(dst->pars->getParameterValue(Enumerations::a3, Enumerations::dec),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->a3_sub->blockSignals(prevState); //restore previous state
}

void MainWindow::on_a3_add_editingFinished(){
    bool prevState = ui->a3_add->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->a3_add->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp >= 0)
            dst->pars->setParameterValue(Enumerations::a3, Enumerations::inc, temp, sst->checkParameters_MainWindow);
    }
    ui->a3_add->setText(QString::number(dst->pars->getParameterValue(Enumerations::a3, Enumerations::inc),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->a3_add->blockSignals(prevState); //restore previous state
}

void MainWindow::on_a3_value_editingFinished(){
    bool prevState = ui->a3_value->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->a3_value->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        ui->a3->setValue((int)(floor(temp * 1.E5)));
    }
    ui->a3_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::a3, Enumerations::val),'f',5));
    delete tempText;
    ui->a3_value->blockSignals(prevState); //restore previous state
}

void MainWindow::on_a3_doSub_clicked(){
    ui->a3->setValue((int)(floor((dst->pars->getParameterValue(Enumerations::a3, Enumerations::val) - dst->pars->getParameterValue(Enumerations::a3, Enumerations::dec)) * 1.E5)));
}

void MainWindow::on_a3_doAdd_clicked(){
    ui->a3->setValue((int)(floor((dst->pars->getParameterValue(Enumerations::a3, Enumerations::val) + dst->pars->getParameterValue(Enumerations::a3, Enumerations::inc)) * 1.E5)));
}


void MainWindow::on_a3_valueChanged(int value){
    dst->pars->setParameterValue(Enumerations::a3, Enumerations::val, (value / 1.E5), sst->checkParameters_MainWindow);
    bool prevState = ui->a3->blockSignals(true);
    ui->a3->setValue((int)(floor(dst->pars->getParameterValue(Enumerations::a3, Enumerations::val) * 1.E5)));
    ui->a3->blockSignals(prevState);
    ui->a3_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::a3, Enumerations::val),'f',5));
    draw(true);
    sst->mustSave = true;
}

void MainWindow::set_a3_min_max(){
    ui->a3->setTickInterval((ui->a3->maximum()-ui->a3->minimum()) / 10);
    ui->a3->setSingleStep(ui->a3->tickInterval());
    ui->a3->setPageStep(ui->a3->tickInterval()*2);
}


//da3

void MainWindow::on_da3_min_editingFinished(){
    bool prevState = ui->da3_min->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->da3_min->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if(temp <= dst->pars->getParameterValue(Enumerations::da3, Enumerations::max)){
            dst->pars->setParameterValue(Enumerations::da3, Enumerations::min, temp, sst->checkParameters_MainWindow);
            ui->da3->setMinimum((int)(floor(dst->pars->getParameterValue(Enumerations::da3, Enumerations::min)*1.E5)));
            set_da3_min_max();
        }
    }
    ui->da3_min->setText(QString::number(dst->pars->getParameterValue(Enumerations::da3, Enumerations::min),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->da3_min->blockSignals(prevState); //restore previous state
}

void MainWindow::on_da3_max_editingFinished(){
    bool prevState = ui->da3_max->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->da3_max->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if(temp >= dst->pars->getParameterValue(Enumerations::da3, Enumerations::min)){
            dst->pars->setParameterValue(Enumerations::da3, Enumerations::max, temp, sst->checkParameters_MainWindow);
            ui->da3->setMaximum((int)(floor(dst->pars->getParameterValue(Enumerations::da3, Enumerations::max)*1.E5)));
            set_da3_min_max();
        }
    }
    ui->da3_max->setText(QString::number(dst->pars->getParameterValue(Enumerations::da3, Enumerations::max),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->da3_max->blockSignals(prevState); //restore previous state
}

void MainWindow::on_da3_sub_editingFinished(){
    bool prevState = ui->da3_sub->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->da3_sub->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp >= 0)
            dst->pars->setParameterValue(Enumerations::da3, Enumerations::dec, temp, sst->checkParameters_MainWindow);
    }
    ui->da3_sub->setText(QString::number(dst->pars->getParameterValue(Enumerations::da3, Enumerations::dec),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->da3_sub->blockSignals(prevState); //restore previous state
}

void MainWindow::on_da3_add_editingFinished(){
    bool prevState = ui->da3_add->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->da3_add->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp >= 0)
            dst->pars->setParameterValue(Enumerations::da3, Enumerations::inc, temp, sst->checkParameters_MainWindow);
    }
    ui->da3_add->setText(QString::number(dst->pars->getParameterValue(Enumerations::da3, Enumerations::inc),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->da3_add->blockSignals(prevState); //restore previous state
}

void MainWindow::on_da3_value_editingFinished(){
    bool prevState = ui->da3_value->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->da3_value->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        ui->da3->setValue((int)(floor(temp * 1.E5)));
    }
    ui->da3_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::da3, Enumerations::val),'f',5));
    delete tempText;
    ui->da3_value->blockSignals(prevState); //restore previous state
}

void MainWindow::on_da3_doSub_clicked(){
    ui->da3->setValue((int)(floor((dst->pars->getParameterValue(Enumerations::da3, Enumerations::val) - dst->pars->getParameterValue(Enumerations::da3, Enumerations::dec)) * 1.E5)));
}

void MainWindow::on_da3_doAdd_clicked(){
    ui->da3->setValue((int)(floor((dst->pars->getParameterValue(Enumerations::da3, Enumerations::val) + dst->pars->getParameterValue(Enumerations::da3, Enumerations::inc)) * 1.E5)));
}

void MainWindow::on_da3_valueChanged(int value){
    dst->pars->setParameterValue(Enumerations::da3, Enumerations::val, (value / 1.E5), sst->checkParameters_MainWindow);
    bool prevState = ui->da3->blockSignals(true);
    ui->da3->setValue((int)(floor(dst->pars->getParameterValue(Enumerations::da3, Enumerations::val) * 1.E5)));
    ui->da3->blockSignals(prevState);
    ui->da3_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::da3, Enumerations::val),'f',5));
    draw(true);
    sst->mustSave = true;
}

void MainWindow::set_da3_min_max(){
    ui->da3->setTickInterval((ui->da3->maximum()-ui->da3->minimum()) / 10);
    ui->da3->setSingleStep(ui->da3->tickInterval());
    ui->da3->setPageStep(ui->da3->tickInterval()*2);
}


//sig3

void MainWindow::on_sig3_min_editingFinished(){
    bool prevState = ui->sig3_min->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->sig3_min->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if(temp <= dst->pars->getParameterValue(Enumerations::sig3, Enumerations::max)){
            dst->pars->setParameterValue(Enumerations::sig3, Enumerations::min, temp, sst->checkParameters_MainWindow);
            ui->sig3->setMinimum((int)(floor(dst->pars->getParameterValue(Enumerations::sig3, Enumerations::min)*1.E5)));
            set_sig3_min_max();
        }
    }
    ui->sig3_min->setText(QString::number(dst->pars->getParameterValue(Enumerations::sig3, Enumerations::min),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->sig3_min->blockSignals(prevState); //restore previous state
}

void MainWindow::on_sig3_max_editingFinished(){
    bool prevState = ui->sig3_max->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->sig3_max->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if(temp >= dst->pars->getParameterValue(Enumerations::sig3, Enumerations::min)){
            dst->pars->setParameterValue(Enumerations::sig3, Enumerations::max, temp, sst->checkParameters_MainWindow);
            ui->sig3->setMaximum((int)(floor(dst->pars->getParameterValue(Enumerations::sig3, Enumerations::max)*1.E5)));
            set_sig3_min_max();
        }
    }
    ui->sig3_max->setText(QString::number(dst->pars->getParameterValue(Enumerations::sig3, Enumerations::max),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->sig3_max->blockSignals(prevState); //restore previous state
}

void MainWindow::on_sig3_sub_editingFinished(){
    bool prevState = ui->sig3_sub->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->sig3_sub->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp >= 0)
            dst->pars->setParameterValue(Enumerations::sig3, Enumerations::dec, temp, sst->checkParameters_MainWindow);
    }
    ui->sig3_sub->setText(QString::number(dst->pars->getParameterValue(Enumerations::sig3, Enumerations::dec),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->sig3_sub->blockSignals(prevState); //restore previous state
}

void MainWindow::on_sig3_add_editingFinished(){
    bool prevState = ui->sig3_add->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->sig3_add->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp >= 0)
            dst->pars->setParameterValue(Enumerations::sig3, Enumerations::inc, temp, sst->checkParameters_MainWindow);
    }
    ui->sig3_add->setText(QString::number(dst->pars->getParameterValue(Enumerations::sig3, Enumerations::inc),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->sig3_add->blockSignals(prevState); //restore previous state
}

void MainWindow::on_sig3_value_editingFinished(){
    bool prevState = ui->sig3_value->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->sig3_value->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        ui->sig3->setValue((int)(floor(temp * 1.E5)));
    }
    ui->sig3_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::sig3, Enumerations::val),'f',5));
    delete tempText;
    ui->sig3_value->blockSignals(prevState); //restore previous state
}

void MainWindow::on_sig3_doSub_clicked(){
    ui->sig3->setValue((int)(floor((dst->pars->getParameterValue(Enumerations::sig3, Enumerations::val) - dst->pars->getParameterValue(Enumerations::sig3, Enumerations::dec)) * 1.E5)));
}

void MainWindow::on_sig3_doAdd_clicked(){
    ui->sig3->setValue((int)(floor((dst->pars->getParameterValue(Enumerations::sig3, Enumerations::val) + dst->pars->getParameterValue(Enumerations::sig3, Enumerations::inc)) * 1.E5)));
}

void MainWindow::on_sig3_valueChanged(int value){
    dst->pars->setParameterValue(Enumerations::sig3, Enumerations::val, (value / 1.E5), sst->checkParameters_MainWindow);
    bool prevState = ui->sig3->blockSignals(true);
    ui->sig3->setValue((int)(floor(dst->pars->getParameterValue(Enumerations::sig3, Enumerations::val) * 1.E5)));
    ui->sig3->blockSignals(prevState);
    ui->sig3_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::sig3, Enumerations::val),'f',5));
    draw(true);
    sst->mustSave = true;
}

void MainWindow::set_sig3_min_max(){
    ui->sig3->setTickInterval((ui->sig3->maximum()-ui->sig3->minimum()) / 10);
    ui->sig3->setSingleStep(ui->sig3->tickInterval());
    ui->sig3->setPageStep(ui->sig3->tickInterval()*2);
}


//u3

void MainWindow::on_u3_min_editingFinished(){
    bool prevState = ui->u3_min->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->u3_min->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if(temp <= dst->pars->getParameterValue(Enumerations::u3, Enumerations::max)){
            dst->pars->setParameterValue(Enumerations::u3, Enumerations::min, temp, sst->checkParameters_MainWindow);
            ui->u3->setMinimum((int)(floor(dst->pars->getParameterValue(Enumerations::u3, Enumerations::min)*1.E5)));
            set_u3_min_max();
        }
    }
    ui->u3_min->setText(QString::number(dst->pars->getParameterValue(Enumerations::u3, Enumerations::min),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->u3_min->blockSignals(prevState); //restore previous state
}

void MainWindow::on_u3_max_editingFinished(){
    bool prevState = ui->u3_max->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->u3_max->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if(temp >= dst->pars->getParameterValue(Enumerations::u3, Enumerations::min)){
            dst->pars->setParameterValue(Enumerations::u3, Enumerations::max, temp, sst->checkParameters_MainWindow);
            ui->u3->setMaximum((int)(floor(dst->pars->getParameterValue(Enumerations::u3, Enumerations::max)*1.E5)));
            set_u3_min_max();
        }
    }
    ui->u3_max->setText(QString::number(dst->pars->getParameterValue(Enumerations::u3, Enumerations::max),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->u3_max->blockSignals(prevState); //restore previous state
}

void MainWindow::on_u3_sub_editingFinished(){
    bool prevState = ui->u3_sub->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->u3_sub->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp >= 0)
            dst->pars->setParameterValue(Enumerations::u3, Enumerations::dec, temp, sst->checkParameters_MainWindow);
    }
    ui->u3_sub->setText(QString::number(dst->pars->getParameterValue(Enumerations::u3, Enumerations::dec),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->u3_sub->blockSignals(prevState); //restore previous state
}

void MainWindow::on_u3_add_editingFinished(){
    bool prevState = ui->u3_add->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->u3_add->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp >= 0)
            dst->pars->setParameterValue(Enumerations::u3, Enumerations::inc, temp, sst->checkParameters_MainWindow);
    }
    ui->u3_add->setText(QString::number(dst->pars->getParameterValue(Enumerations::u3, Enumerations::inc),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->u3_add->blockSignals(prevState); //restore previous state
}

void MainWindow::on_u3_value_editingFinished(){
    bool prevState = ui->u3_value->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->u3_value->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        ui->u3->setValue((int)(floor(temp * 1.E5)));
    }
    ui->u3_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::u3, Enumerations::val),'f',5));
    delete tempText;
    ui->u3_value->blockSignals(prevState); //restore previous state
}

void MainWindow::on_u3_doSub_clicked(){
    ui->u3->setValue((int)(floor((dst->pars->getParameterValue(Enumerations::u3, Enumerations::val) - dst->pars->getParameterValue(Enumerations::u3, Enumerations::dec)) * 1.E5)));
}

void MainWindow::on_u3_doAdd_clicked(){
    ui->u3->setValue((int)(floor((dst->pars->getParameterValue(Enumerations::u3, Enumerations::val) + dst->pars->getParameterValue(Enumerations::u3, Enumerations::inc)) * 1.E5)));
}

void MainWindow::on_u3_valueChanged(int value){
    dst->pars->setParameterValue(Enumerations::u3, Enumerations::val, (value / 1.E5), sst->checkParameters_MainWindow);
    bool prevState = ui->u3->blockSignals(true);
    ui->u3->setValue((int)(floor(dst->pars->getParameterValue(Enumerations::u3, Enumerations::val) * 1.E5)));
    ui->u3->blockSignals(prevState);
    ui->u3_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::u3, Enumerations::val),'f',5));
    draw(true);
    sst->mustSave = true;
}

void MainWindow::set_u3_min_max(){
    ui->u3->setTickInterval((ui->u3->maximum()-ui->u3->minimum()) / 10);
    ui->u3->setSingleStep(ui->u3->tickInterval());
    ui->u3->setPageStep(ui->u3->tickInterval()*2);
}


//eta

void MainWindow::on_eta_min_editingFinished(){
    bool prevState = ui->eta_min->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->eta_min->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if(temp <= dst->pars->getParameterValue(Enumerations::eta, Enumerations::max)){
            dst->pars->setParameterValue(Enumerations::eta, Enumerations::min, temp, sst->checkParameters_MainWindow);
            ui->eta->setMinimum((int)(floor(dst->pars->getParameterValue(Enumerations::eta, Enumerations::min)*1.E5)));
            set_eta_min_max();
        }
    }
    ui->eta_min->setText(QString::number(dst->pars->getParameterValue(Enumerations::eta, Enumerations::min),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->eta_min->blockSignals(prevState); //restore previous state
}

void MainWindow::on_eta_max_editingFinished(){
    bool prevState = ui->eta_max->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->eta_max->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if(temp >= dst->pars->getParameterValue(Enumerations::eta, Enumerations::min)){
            dst->pars->setParameterValue(Enumerations::eta, Enumerations::max, temp, sst->checkParameters_MainWindow);
            ui->eta->setMaximum((int)(floor(dst->pars->getParameterValue(Enumerations::eta, Enumerations::max)*1.E5)));
            set_eta_min_max();
        }
    }
    ui->eta_max->setText(QString::number(dst->pars->getParameterValue(Enumerations::eta, Enumerations::max),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->eta_max->blockSignals(prevState); //restore previous state
}

void MainWindow::on_eta_sub_editingFinished(){
    bool prevState = ui->eta_sub->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->eta_sub->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp >= 0)
            dst->pars->setParameterValue(Enumerations::eta, Enumerations::dec, temp, sst->checkParameters_MainWindow);
    }
    ui->eta_sub->setText(QString::number(dst->pars->getParameterValue(Enumerations::eta, Enumerations::dec),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->eta_sub->blockSignals(prevState); //restore previous state
}

void MainWindow::on_eta_add_editingFinished(){
    bool prevState = ui->eta_add->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->eta_add->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp >= 0)
            dst->pars->setParameterValue(Enumerations::eta, Enumerations::inc, temp, sst->checkParameters_MainWindow);
    }
    ui->eta_add->setText(QString::number(dst->pars->getParameterValue(Enumerations::eta, Enumerations::inc),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->eta_add->blockSignals(prevState); //restore previous state
}

void MainWindow::on_eta_value_editingFinished(){
    bool prevState = ui->eta_value->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->eta_value->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        ui->eta->setValue((int)(floor(temp * 1.E5)));
    }
    ui->eta_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::eta, Enumerations::val),'f',5));
    delete tempText;
    ui->eta_value->blockSignals(prevState); //restore previous state
}

void MainWindow::on_eta_doSub_clicked(){
    ui->eta->setValue((int)(floor((dst->pars->getParameterValue(Enumerations::eta, Enumerations::val) - dst->pars->getParameterValue(Enumerations::eta, Enumerations::dec)) * 1.E5)));
}

void MainWindow::on_eta_doAdd_clicked(){
    ui->eta->setValue((int)(floor((dst->pars->getParameterValue(Enumerations::eta, Enumerations::val) + dst->pars->getParameterValue(Enumerations::eta, Enumerations::inc)) * 1.E5)));
}

void MainWindow::on_eta_valueChanged(int value){
    dst->pars->setParameterValue(Enumerations::eta, Enumerations::val, (value / 1.E5), sst->checkParameters_MainWindow);
    bool prevState = ui->eta->blockSignals(true);
    ui->eta->setValue((int)(floor(dst->pars->getParameterValue(Enumerations::eta, Enumerations::val) * 1.E5)));
    ui->eta->blockSignals(prevState);
    ui->eta_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::eta, Enumerations::val),'f',5));
    draw(true);
    sst->mustSave = true;
}

void MainWindow::set_eta_min_max(){
    ui->eta->setTickInterval((ui->eta->maximum()-ui->eta->minimum()) / 10);
    ui->eta->setSingleStep(ui->eta->tickInterval());
    ui->eta->setPageStep(ui->eta->tickInterval()*2);
}


//nu

void MainWindow::on_nu_min_editingFinished(){
    bool prevState = ui->nu_min->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->nu_min->text());
    if (AuxiliaryMethods::isInt(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 0);
        if(temp <= dst->pars->getParameterValue(Enumerations::nu, Enumerations::max)){
            dst->pars->setParameterValue(Enumerations::nu, Enumerations::min, temp, sst->checkParameters_MainWindow);
            ui->nu->setMinimum((int)(floor(dst->pars->getParameterValue(Enumerations::nu, Enumerations::min))));
            set_nu_min_max();
        }
    }
    ui->nu_min->setText(QString::number(dst->pars->getParameterValue(Enumerations::nu, Enumerations::min),'f',0));
    delete tempText;
    sst->mustSave = true;
    ui->nu_min->blockSignals(prevState); //restore previous state
}

void MainWindow::on_nu_max_editingFinished(){
    bool prevState = ui->nu_max->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->nu_max->text());
    if (AuxiliaryMethods::isInt(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 0);
        if(temp >= dst->pars->getParameterValue(Enumerations::nu, Enumerations::min)){
            dst->pars->setParameterValue(Enumerations::nu, Enumerations::max, temp, sst->checkParameters_MainWindow);
            ui->nu->setMaximum((int)(floor(dst->pars->getParameterValue(Enumerations::nu, Enumerations::max))));
            set_nu_min_max();
        }
    }
    ui->nu_max->setText(QString::number(dst->pars->getParameterValue(Enumerations::nu, Enumerations::max),'f',0));
    delete tempText;
    sst->mustSave = true;
    ui->nu_max->blockSignals(prevState); //restore previous state
}

void MainWindow::on_nu_sub_editingFinished(){
    bool prevState = ui->nu_sub->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->nu_sub->text());
    if (AuxiliaryMethods::isInt(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 0);
        if (temp >= 0)
            dst->pars->setParameterValue(Enumerations::nu, Enumerations::dec, temp, sst->checkParameters_MainWindow);
    }
    ui->nu_sub->setText(QString::number(dst->pars->getParameterValue(Enumerations::nu, Enumerations::dec),'f',0));
    delete tempText;
    sst->mustSave = true;
    ui->nu_sub->blockSignals(prevState); //restore previous state
}

void MainWindow::on_nu_add_editingFinished(){
    bool prevState = ui->nu_add->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->nu_add->text());
    if (AuxiliaryMethods::isInt(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 0);
        if (temp >= 0)
            dst->pars->setParameterValue(Enumerations::nu, Enumerations::inc, temp, sst->checkParameters_MainWindow);
    }
    ui->nu_add->setText(QString::number(dst->pars->getParameterValue(Enumerations::nu, Enumerations::inc),'f',0));
    delete tempText;
    sst->mustSave = true;
    ui->nu_add->blockSignals(prevState); //restore previous state
}

void MainWindow::on_nu_value_editingFinished(){
    bool prevState = ui->nu_value->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->nu_value->text());
    if (AuxiliaryMethods::isInt(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 0);
        ui->nu->setValue((int)(floor(temp)));
    }
    ui->nu_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::nu, Enumerations::val),'f',0));
    delete tempText;
    ui->nu_value->blockSignals(prevState); //restore previous state
}

void MainWindow::on_nu_doSub_clicked(){
    ui->nu->setValue((int)(floor((dst->pars->getParameterValue(Enumerations::nu, Enumerations::val) - dst->pars->getParameterValue(Enumerations::nu, Enumerations::dec)))));
}

void MainWindow::on_nu_doAdd_clicked(){
    ui->nu->setValue((int)(floor((dst->pars->getParameterValue(Enumerations::nu, Enumerations::val) + dst->pars->getParameterValue(Enumerations::nu, Enumerations::inc)))));
}

void MainWindow::on_nu_valueChanged(int value){
    dst->pars->setParameterValue(Enumerations::nu, Enumerations::val, value, sst->checkParameters_MainWindow);
    bool prevState = ui->nu->blockSignals(true);
    ui->nu->setValue((int)(floor(dst->pars->getParameterValue(Enumerations::nu, Enumerations::val))));
    ui->nu->blockSignals(prevState);
    ui->nu_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::nu, Enumerations::val),'f',0));
    draw(true);
    sst->mustSave = true;
}

void MainWindow::set_nu_min_max(){
    ui->nu->setTickInterval((ui->nu->maximum()-ui->nu->minimum()) / 10);
    ui->nu->setSingleStep(ui->nu->tickInterval());
    ui->nu->setPageStep(ui->nu->tickInterval()*2);
}


//alpha

void MainWindow::on_alpha_min_editingFinished(){
    bool prevState = ui->alpha_min->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->alpha_min->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if(temp <= dst->pars->getParameterValue(Enumerations::alpha, Enumerations::max)){
            dst->pars->setParameterValue(Enumerations::alpha, Enumerations::min, temp, sst->checkParameters_MainWindow);
            ui->alpha->setMinimum((int)(floor(dst->pars->getParameterValue(Enumerations::alpha, Enumerations::min)*1.E5)));
            set_alpha_min_max();
        }
    }
    ui->alpha_min->setText(QString::number(dst->pars->getParameterValue(Enumerations::alpha, Enumerations::min),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->alpha_min->blockSignals(prevState); //restore previous state
}

void MainWindow::on_alpha_max_editingFinished(){
    bool prevState = ui->alpha_max->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->alpha_max->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if(temp >= dst->pars->getParameterValue(Enumerations::alpha, Enumerations::min)){
            dst->pars->setParameterValue(Enumerations::alpha, Enumerations::max, temp, sst->checkParameters_MainWindow);
            ui->alpha->setMaximum((int)(floor(dst->pars->getParameterValue(Enumerations::alpha, Enumerations::max)*1.E5)));
            set_alpha_min_max();
        }
    }
    ui->alpha_max->setText(QString::number(dst->pars->getParameterValue(Enumerations::alpha, Enumerations::max),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->alpha_max->blockSignals(prevState); //restore previous state
}

void MainWindow::on_alpha_sub_editingFinished(){
    bool prevState = ui->alpha_sub->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->alpha_sub->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp >= 0)
            dst->pars->setParameterValue(Enumerations::alpha, Enumerations::dec, temp, sst->checkParameters_MainWindow);
    }
    ui->alpha_sub->setText(QString::number(dst->pars->getParameterValue(Enumerations::alpha, Enumerations::dec),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->alpha_sub->blockSignals(prevState); //restore previous state
}

void MainWindow::on_alpha_add_editingFinished(){
    bool prevState = ui->alpha_add->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->alpha_add->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp >= 0)
            dst->pars->setParameterValue(Enumerations::alpha, Enumerations::inc, temp, sst->checkParameters_MainWindow);
    }
    ui->alpha_add->setText(QString::number(dst->pars->getParameterValue(Enumerations::alpha, Enumerations::inc),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->alpha_add->blockSignals(prevState); //restore previous state
}

void MainWindow::on_alpha_value_editingFinished(){
    bool prevState = ui->alpha_value->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->alpha_value->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        ui->alpha->setValue((int)(floor(temp * 1.E5)));
    }
    ui->alpha_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::alpha, Enumerations::val),'f',5));
    delete tempText;
    ui->alpha_value->blockSignals(prevState); //restore previous state
}

void MainWindow::on_alpha_doSub_clicked(){
    ui->alpha->setValue((int)(floor((dst->pars->getParameterValue(Enumerations::alpha, Enumerations::val) - dst->pars->getParameterValue(Enumerations::alpha, Enumerations::dec)) * 1.E5)));
}

void MainWindow::on_alpha_doAdd_clicked(){
    ui->alpha->setValue((int)(floor((dst->pars->getParameterValue(Enumerations::alpha, Enumerations::val) + dst->pars->getParameterValue(Enumerations::alpha, Enumerations::inc)) * 1.E5)));
}

void MainWindow::on_alpha_valueChanged(int value){
    dst->pars->setParameterValue(Enumerations::alpha, Enumerations::val, (value / 1.E5), sst->checkParameters_MainWindow);
    bool prevState = ui->alpha->blockSignals(true);
    ui->alpha->setValue((int)(floor(dst->pars->getParameterValue(Enumerations::alpha, Enumerations::val) * 1.E5)));
    ui->alpha->blockSignals(prevState);
    ui->alpha_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::alpha, Enumerations::val),'f',5));
    draw(true);
    sst->mustSave = true;
}

void MainWindow::set_alpha_min_max(){
    ui->alpha->setTickInterval((ui->alpha->maximum()-ui->alpha->minimum()) / 10);
    ui->alpha->setSingleStep(ui->alpha->tickInterval());
    ui->alpha->setPageStep(ui->alpha->tickInterval()*2);
}

//sig1

void MainWindow::on_sig1_min_editingFinished(){
    bool prevState = ui->sig1_min->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->sig1_min->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if(temp <= dst->pars->getParameterValue(Enumerations::sig1, Enumerations::max)){
            dst->pars->setParameterValue(Enumerations::sig1, Enumerations::min, temp, sst->checkParameters_MainWindow);
            ui->sig1->setMinimum((int)(floor(dst->pars->getParameterValue(Enumerations::sig1, Enumerations::min)*1.E5)));
            set_sig1_min_max();
        }
    }
    ui->sig1_min->setText(QString::number(dst->pars->getParameterValue(Enumerations::sig1, Enumerations::min),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->sig1_min->blockSignals(prevState); //restore previous state
}

void MainWindow::on_sig1_max_editingFinished(){
    bool prevState = ui->sig1_max->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->sig1_max->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if(temp >= dst->pars->getParameterValue(Enumerations::sig1, Enumerations::min)){
            dst->pars->setParameterValue(Enumerations::sig1, Enumerations::max, temp, sst->checkParameters_MainWindow);
            ui->sig1->setMaximum((int)(floor(dst->pars->getParameterValue(Enumerations::sig1, Enumerations::max)*1.E5)));
            set_sig1_min_max();
        }
    }
    ui->sig1_max->setText(QString::number(dst->pars->getParameterValue(Enumerations::sig1, Enumerations::max),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->sig1_max->blockSignals(prevState); //restore previous state
}

void MainWindow::on_sig1_sub_editingFinished(){
    bool prevState = ui->sig1_sub->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->sig1_sub->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp >= 0)
            dst->pars->setParameterValue(Enumerations::sig1, Enumerations::dec, temp, sst->checkParameters_MainWindow);
    }
    ui->sig1_sub->setText(QString::number(dst->pars->getParameterValue(Enumerations::sig1, Enumerations::dec),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->sig1_sub->blockSignals(prevState); //restore previous state
}

void MainWindow::on_sig1_add_editingFinished(){
    bool prevState = ui->sig1_add->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->sig1_add->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp >= 0)
            dst->pars->setParameterValue(Enumerations::sig1, Enumerations::inc, temp, sst->checkParameters_MainWindow);
    }
    ui->sig1_add->setText(QString::number(dst->pars->getParameterValue(Enumerations::sig1, Enumerations::inc),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->sig1_add->blockSignals(prevState); //restore previous state
}

void MainWindow::on_sig1_value_editingFinished(){
    bool prevState = ui->sig1_value->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->sig1_value->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        ui->sig1->setValue((int)(floor(temp * 1.E5)));
    }
    ui->sig1_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::sig1, Enumerations::val),'f',5));
    delete tempText;
    ui->sig1_value->blockSignals(prevState); //restore previous state
}

void MainWindow::on_sig1_doSub_clicked(){
    ui->sig1->setValue((int)(floor((dst->pars->getParameterValue(Enumerations::sig1, Enumerations::val) - dst->pars->getParameterValue(Enumerations::sig1, Enumerations::dec)) * 1.E5)));
}

void MainWindow::on_sig1_doAdd_clicked(){
    ui->sig1->setValue((int)(floor((dst->pars->getParameterValue(Enumerations::sig1, Enumerations::val) + dst->pars->getParameterValue(Enumerations::sig1, Enumerations::inc)) * 1.E5)));
}

void MainWindow::on_sig1_valueChanged(int value){
    dst->pars->setParameterValue(Enumerations::sig1, Enumerations::val, (value / 1.E5), sst->checkParameters_MainWindow);
    bool prevState = ui->sig1->blockSignals(true);
    ui->sig1->setValue((int)(floor(dst->pars->getParameterValue(Enumerations::sig1, Enumerations::val) * 1.E5)));
    ui->sig1->blockSignals(prevState);
    ui->sig1_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::sig1, Enumerations::val),'f',5));
    draw(true);
    sst->mustSave = true;
}

void MainWindow::set_sig1_min_max(){
    ui->sig1->setTickInterval((ui->sig1->maximum()-ui->sig1->minimum()) / 10);
    ui->sig1->setSingleStep(ui->sig1->tickInterval());
    ui->sig1->setPageStep(ui->sig1->tickInterval()*2);
}


//lcc

void MainWindow::on_lcc_min_editingFinished(){
    bool prevState = ui->lcc_min->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->lcc_min->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if(temp <= dst->pars->getParameterValue(Enumerations::lcc, Enumerations::max)){
            dst->pars->setParameterValue(Enumerations::lcc, Enumerations::min, temp, sst->checkParameters_MainWindow);
            ui->lcc->setMinimum((int)(floor(dst->pars->getParameterValue(Enumerations::lcc, Enumerations::min)*1.E5)));
            set_lcc_min_max();
        }
    }
    ui->lcc_min->setText(QString::number(dst->pars->getParameterValue(Enumerations::lcc, Enumerations::min),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->lcc_min->blockSignals(prevState); //restore previous state
}

void MainWindow::on_lcc_max_editingFinished(){
    bool prevState = ui->lcc_max->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->lcc_max->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if(temp >= dst->pars->getParameterValue(Enumerations::lcc, Enumerations::min)){
            dst->pars->setParameterValue(Enumerations::lcc, Enumerations::max, temp, sst->checkParameters_MainWindow);
            ui->lcc->setMaximum((int)(floor(dst->pars->getParameterValue(Enumerations::lcc, Enumerations::max)*1.E5)));
            set_lcc_min_max();
        }
    }
    ui->lcc_max->setText(QString::number(dst->pars->getParameterValue(Enumerations::lcc, Enumerations::max),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->lcc_max->blockSignals(prevState); //restore previous state
}

void MainWindow::on_lcc_sub_editingFinished(){
    bool prevState = ui->lcc_sub->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->lcc_sub->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp >= 0)
            dst->pars->setParameterValue(Enumerations::lcc, Enumerations::dec, temp, sst->checkParameters_MainWindow);
    }
    ui->lcc_sub->setText(QString::number(dst->pars->getParameterValue(Enumerations::lcc, Enumerations::dec),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->lcc_sub->blockSignals(prevState); //restore previous state
}

void MainWindow::on_lcc_add_editingFinished(){
    bool prevState = ui->lcc_add->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->lcc_add->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp >= 0)
            dst->pars->setParameterValue(Enumerations::lcc, Enumerations::inc, temp, sst->checkParameters_MainWindow);
    }
    ui->lcc_add->setText(QString::number(dst->pars->getParameterValue(Enumerations::lcc, Enumerations::inc),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->lcc_add->blockSignals(prevState); //restore previous state
}

void MainWindow::on_lcc_value_editingFinished(){
    bool prevState = ui->lcc_value->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->lcc_value->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        ui->lcc->setValue((int)(floor(temp * 1.E5)));
    }
    ui->lcc_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::lcc, Enumerations::val),'f',5));
    delete tempText;
    ui->lcc_value->blockSignals(prevState); //restore previous state
}

void MainWindow::on_lcc_doSub_clicked(){
    ui->lcc->setValue((int)(floor((dst->pars->getParameterValue(Enumerations::lcc, Enumerations::val) - dst->pars->getParameterValue(Enumerations::lcc, Enumerations::dec)) * 1.E5)));
}

void MainWindow::on_lcc_doAdd_clicked(){
    ui->lcc->setValue((int)(floor((dst->pars->getParameterValue(Enumerations::lcc, Enumerations::val) + dst->pars->getParameterValue(Enumerations::lcc, Enumerations::inc)) * 1.E5)));
}

void MainWindow::on_lcc_valueChanged(int value){
    dst->pars->setParameterValue(Enumerations::lcc, Enumerations::val, (value / 1.E5), sst->checkParameters_MainWindow);
    bool prevState = ui->lcc->blockSignals(true);
    ui->lcc->setValue((int)(floor(dst->pars->getParameterValue(Enumerations::lcc, Enumerations::val) * 1.E5)));
    ui->lcc->blockSignals(prevState);
    ui->lcc_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::lcc, Enumerations::val),'f',5));
    draw(true);
    sst->mustSave = true;
}

void MainWindow::set_lcc_min_max(){
    ui->lcc->setTickInterval((ui->lcc->maximum()-ui->lcc->minimum()) / 10);
    ui->lcc->setSingleStep(ui->lcc->tickInterval());
    ui->lcc->setPageStep(ui->lcc->tickInterval()*2);
}


//q

void MainWindow::on_q_min_editingFinished(){
    bool prevState = ui->q_min->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->q_min->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if(temp <= dst->pars->getParameterValue(Enumerations::q, Enumerations::max)){
            dst->pars->setParameterValue(Enumerations::q, Enumerations::min, temp, sst->checkParameters_MainWindow);
            ui->q->setMinimum((int)(floor(dst->pars->getParameterValue(Enumerations::q, Enumerations::min)*1.E5)));
            set_q_min_max();
        }
    }
    ui->q_min->setText(QString::number(dst->pars->getParameterValue(Enumerations::q, Enumerations::min),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->q_min->blockSignals(prevState); //restore previous state
}

void MainWindow::on_q_max_editingFinished(){
    bool prevState = ui->q_max->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->q_max->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if(temp >= dst->pars->getParameterValue(Enumerations::q, Enumerations::min)){
            dst->pars->setParameterValue(Enumerations::q, Enumerations::max, temp, sst->checkParameters_MainWindow);
            ui->q->setMaximum((int)(floor(dst->pars->getParameterValue(Enumerations::q, Enumerations::max)*1.E5)));
            set_q_min_max();
        }
    }
    ui->q_max->setText(QString::number(dst->pars->getParameterValue(Enumerations::q, Enumerations::max),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->q_max->blockSignals(prevState); //restore previous state
}

void MainWindow::on_q_sub_editingFinished(){
    bool prevState = ui->q_sub->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->q_sub->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp >= 0)
            dst->pars->setParameterValue(Enumerations::q, Enumerations::dec, temp, sst->checkParameters_MainWindow);
    }
    ui->q_sub->setText(QString::number(dst->pars->getParameterValue(Enumerations::q, Enumerations::dec),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->q_sub->blockSignals(prevState); //restore previous state
}

void MainWindow::on_q_add_editingFinished(){
    bool prevState = ui->q_add->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->q_add->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp >= 0)
            dst->pars->setParameterValue(Enumerations::q, Enumerations::inc, temp, sst->checkParameters_MainWindow);
    }
    ui->q_add->setText(QString::number(dst->pars->getParameterValue(Enumerations::q, Enumerations::inc),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->q_add->blockSignals(prevState); //restore previous state
}

void MainWindow::on_q_value_editingFinished(){
    bool prevState = ui->q_value->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->q_value->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        ui->q->setValue((int)(floor(temp * 1.E5)));
    }
    ui->q_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::q, Enumerations::val),'f',5));
    delete tempText;
    ui->q_value->blockSignals(prevState); //restore previous state
}

void MainWindow::on_q_doSub_clicked(){
    ui->q->setValue((int)(floor((dst->pars->getParameterValue(Enumerations::q, Enumerations::val) - dst->pars->getParameterValue(Enumerations::q, Enumerations::dec)) * 1.E5)));
}

void MainWindow::on_q_doAdd_clicked(){
    ui->q->setValue((int)(floor((dst->pars->getParameterValue(Enumerations::q, Enumerations::val) + dst->pars->getParameterValue(Enumerations::q, Enumerations::inc)) * 1.E5)));
}

void MainWindow::on_q_valueChanged(int value){
    dst->pars->setParameterValue(Enumerations::q, Enumerations::val, (value / 1.E5), sst->checkParameters_MainWindow);
    bool prevState = ui->q->blockSignals(true);
    ui->q->setValue((int)(floor(dst->pars->getParameterValue(Enumerations::q, Enumerations::val) * 1.E5)));
    ui->q->blockSignals(prevState);
    ui->q_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::q, Enumerations::val),'f',5));
    draw(true);
    sst->mustSave = true;
}

void MainWindow::set_q_min_max(){
    ui->q->setTickInterval((ui->q->maximum()-ui->q->minimum()) / 10);
    ui->q->setSingleStep(ui->q->tickInterval());
    ui->q->setPageStep(ui->q->tickInterval()*2);
}


//cN

void MainWindow::on_cN_min_editingFinished(){
    bool prevState = ui->cN_min->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->cN_min->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if(temp <= dst->pars->getParameterValue(Enumerations::cN, Enumerations::max)){
            dst->pars->setParameterValue(Enumerations::cN, Enumerations::min, temp, sst->checkParameters_MainWindow);
            ui->cN->setMinimum((int)(floor(dst->pars->getParameterValue(Enumerations::cN, Enumerations::min)*1.E5)));
            set_cN_min_max();
        }
    }
    ui->cN_min->setText(QString::number(dst->pars->getParameterValue(Enumerations::cN, Enumerations::min),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->cN_min->blockSignals(prevState); //restore previous state
}

void MainWindow::on_cN_max_editingFinished(){
    bool prevState = ui->cN_max->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->cN_max->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if(temp >= dst->pars->getParameterValue(Enumerations::cN, Enumerations::min)){
            dst->pars->setParameterValue(Enumerations::cN, Enumerations::max, temp, sst->checkParameters_MainWindow);
            ui->cN->setMaximum((int)(floor(dst->pars->getParameterValue(Enumerations::cN, Enumerations::max)*1.E5)));
            set_cN_min_max();
        }
    }
    ui->cN_max->setText(QString::number(dst->pars->getParameterValue(Enumerations::cN, Enumerations::max),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->cN_max->blockSignals(prevState); //restore previous state
}

void MainWindow::on_cN_sub_editingFinished(){
    bool prevState = ui->cN_sub->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->cN_sub->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp >= 0)
            dst->pars->setParameterValue(Enumerations::cN, Enumerations::dec, temp, sst->checkParameters_MainWindow);
    }
    ui->cN_sub->setText(QString::number(dst->pars->getParameterValue(Enumerations::cN, Enumerations::dec),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->cN_sub->blockSignals(prevState); //restore previous state
}

void MainWindow::on_cN_add_editingFinished(){
    bool prevState = ui->cN_add->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->cN_add->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp >= 0)
            dst->pars->setParameterValue(Enumerations::cN, Enumerations::inc, temp, sst->checkParameters_MainWindow);
    }
    ui->cN_add->setText(QString::number(dst->pars->getParameterValue(Enumerations::cN, Enumerations::inc),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->cN_add->blockSignals(prevState); //restore previous state
}

void MainWindow::on_cN_value_editingFinished(){
    bool prevState = ui->cN_value->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->cN_value->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        ui->cN->setValue((int)(floor(temp * 1.E5)));
    }
    ui->cN_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::cN, Enumerations::val),'f',5));
    delete tempText;
    ui->cN_value->blockSignals(prevState); //restore previous state
}

void MainWindow::on_cN_doSub_clicked(){
    ui->cN->setValue((int)(floor((dst->pars->getParameterValue(Enumerations::cN, Enumerations::val) - dst->pars->getParameterValue(Enumerations::cN, Enumerations::dec)) * 1.E5)));
}

void MainWindow::on_cN_doAdd_clicked(){
    ui->cN->setValue((int)(floor((dst->pars->getParameterValue(Enumerations::cN, Enumerations::val) + dst->pars->getParameterValue(Enumerations::cN, Enumerations::inc)) * 1.E5)));
}

void MainWindow::on_cN_valueChanged(int value){
    dst->pars->setParameterValue(Enumerations::cN, Enumerations::val, (value / 1.E5), sst->checkParameters_MainWindow);
    bool prevState = ui->cN->blockSignals(true);
    ui->cN->setValue((int)(floor(dst->pars->getParameterValue(Enumerations::cN, Enumerations::val) * 1.E5)));
    ui->cN->blockSignals(prevState);
    ui->cN_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::cN, Enumerations::val),'f',5));
    draw(true);
    sst->mustSave = true;
}

void MainWindow::set_cN_min_max(){
    ui->cN->setTickInterval((ui->cN->maximum()-ui->cN->minimum()) / 10);
    ui->cN->setSingleStep(ui->cN->tickInterval());
    ui->cN->setPageStep(ui->cN->tickInterval()*2);
}


//cO

void MainWindow::on_cO_min_editingFinished(){
    bool prevState = ui->cO_min->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->cO_min->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if(temp <= dst->pars->getParameterValue(Enumerations::cO, Enumerations::max)){
            dst->pars->setParameterValue(Enumerations::cO, Enumerations::min, temp, sst->checkParameters_MainWindow);
            ui->cO->setMinimum((int)(floor(dst->pars->getParameterValue(Enumerations::cO, Enumerations::min)*1.E5)));
            set_cO_min_max();
        }
    }
    ui->cO_min->setText(QString::number(dst->pars->getParameterValue(Enumerations::cO, Enumerations::min),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->cO_min->blockSignals(prevState); //restore previous state
}

void MainWindow::on_cO_max_editingFinished(){
    bool prevState = ui->cO_max->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->cO_max->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if(temp >= dst->pars->getParameterValue(Enumerations::cO, Enumerations::min)){
            dst->pars->setParameterValue(Enumerations::cO, Enumerations::max, temp, sst->checkParameters_MainWindow);
            ui->cO->setMaximum((int)(floor(dst->pars->getParameterValue(Enumerations::cO, Enumerations::max)*1.E5)));
            set_cO_min_max();
        }
    }
    ui->cO_max->setText(QString::number(dst->pars->getParameterValue(Enumerations::cO, Enumerations::max),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->cO_max->blockSignals(prevState); //restore previous state
}

void MainWindow::on_cO_sub_editingFinished(){
    bool prevState = ui->cO_sub->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->cO_sub->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp >= 0)
            dst->pars->setParameterValue(Enumerations::cO, Enumerations::dec, temp, sst->checkParameters_MainWindow);
    }
    ui->cO_sub->setText(QString::number(dst->pars->getParameterValue(Enumerations::cO, Enumerations::dec),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->cO_sub->blockSignals(prevState); //restore previous state
}

void MainWindow::on_cO_add_editingFinished(){
    bool prevState = ui->cO_add->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->cO_add->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp >= 0)
            dst->pars->setParameterValue(Enumerations::cO, Enumerations::inc, temp, sst->checkParameters_MainWindow);
    }
    ui->cO_add->setText(QString::number(dst->pars->getParameterValue(Enumerations::cO, Enumerations::inc),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->cO_add->blockSignals(prevState); //restore previous state
}

void MainWindow::on_cO_value_editingFinished(){
    bool prevState = ui->cO_value->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->cO_value->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        ui->cO->setValue((int)(floor(temp * 1.E5)));
    }
    ui->cO_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::cO, Enumerations::val),'f',5));
    delete tempText;
    ui->cO_value->blockSignals(prevState); //restore previous state
}

void MainWindow::on_cO_doSub_clicked(){
    ui->cO->setValue((int)(floor((dst->pars->getParameterValue(Enumerations::cO, Enumerations::val) - dst->pars->getParameterValue(Enumerations::cO, Enumerations::dec)) * 1.E5)));
}

void MainWindow::on_cO_doAdd_clicked(){
    ui->cO->setValue((int)(floor((dst->pars->getParameterValue(Enumerations::cO, Enumerations::val) + dst->pars->getParameterValue(Enumerations::cO, Enumerations::inc)) * 1.E5)));
}

void MainWindow::on_cO_valueChanged(int value){
    dst->pars->setParameterValue(Enumerations::cO, Enumerations::val, (value / 1.E5), sst->checkParameters_MainWindow);
    bool prevState = ui->cO->blockSignals(true);
    ui->cO->setValue((int)(floor(dst->pars->getParameterValue(Enumerations::cO, Enumerations::val) * 1.E5)));
    ui->cO->blockSignals(prevState);
    ui->cO_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::cO, Enumerations::val),'f',5));
    draw(true);
    sst->mustSave = true;
}

void MainWindow::set_cO_min_max(){
    ui->cO->setTickInterval((ui->cO->maximum()-ui->cO->minimum()) / 10);
    ui->cO->setSingleStep(ui->cO->tickInterval());
    ui->cO->setPageStep(ui->cO->tickInterval()*2);
}


//dan

void MainWindow::on_dan_min_editingFinished(){
    bool prevState = ui->dan_min->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->dan_min->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if(temp <= dst->pars->getParameterValue(Enumerations::dan, Enumerations::max)){
            dst->pars->setParameterValue(Enumerations::dan, Enumerations::min, temp, sst->checkParameters_MainWindow);
            ui->dan->setMinimum((int)(floor(dst->pars->getParameterValue(Enumerations::dan, Enumerations::min)*1.E5)));
            set_dan_min_max();
        }
    }
    ui->dan_min->setText(QString::number(dst->pars->getParameterValue(Enumerations::dan, Enumerations::min),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->dan_min->blockSignals(prevState); //restore previous state
}

void MainWindow::on_dan_max_editingFinished(){
    bool prevState = ui->dan_max->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->dan_max->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if(temp >= dst->pars->getParameterValue(Enumerations::dan, Enumerations::min)){
            dst->pars->setParameterValue(Enumerations::dan, Enumerations::max, temp, sst->checkParameters_MainWindow);
            ui->dan->setMaximum((int)(floor(dst->pars->getParameterValue(Enumerations::dan, Enumerations::max)*1.E5)));
            set_dan_min_max();
        }
    }
    ui->dan_max->setText(QString::number(dst->pars->getParameterValue(Enumerations::dan, Enumerations::max),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->dan_max->blockSignals(prevState); //restore previous state
}

void MainWindow::on_dan_sub_editingFinished(){
    bool prevState = ui->dan_sub->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->dan_sub->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp >= 0)
            dst->pars->setParameterValue(Enumerations::dan, Enumerations::dec, temp, sst->checkParameters_MainWindow);
    }
    ui->dan_sub->setText(QString::number(dst->pars->getParameterValue(Enumerations::dan, Enumerations::dec),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->dan_sub->blockSignals(prevState); //restore previous state
}

void MainWindow::on_dan_add_editingFinished(){
    bool prevState = ui->dan_add->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->dan_add->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp >= 0)
            dst->pars->setParameterValue(Enumerations::dan, Enumerations::inc, temp, sst->checkParameters_MainWindow);
    }
    ui->dan_add->setText(QString::number(dst->pars->getParameterValue(Enumerations::dan, Enumerations::inc),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->dan_add->blockSignals(prevState); //restore previous state
}

void MainWindow::on_dan_value_editingFinished(){
    bool prevState = ui->dan_value->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->dan_value->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        ui->dan->setValue((int)(floor(temp * 1.E5)));
    }
    ui->dan_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::dan, Enumerations::val),'f',5));
    delete tempText;
    ui->dan_value->blockSignals(prevState); //restore previous state
}

void MainWindow::on_dan_doSub_clicked(){
    ui->dan->setValue((int)(floor((dst->pars->getParameterValue(Enumerations::dan, Enumerations::val) - dst->pars->getParameterValue(Enumerations::dan, Enumerations::dec)) * 1.E5)));
}

void MainWindow::on_dan_doAdd_clicked(){
    ui->dan->setValue((int)(floor((dst->pars->getParameterValue(Enumerations::dan, Enumerations::val) + dst->pars->getParameterValue(Enumerations::dan, Enumerations::inc)) * 1.E5)));
}

void MainWindow::on_dan_valueChanged(int value){
    dst->pars->setParameterValue(Enumerations::dan, Enumerations::val, (value / 1.E5), sst->checkParameters_MainWindow);
    bool prevState = ui->dan->blockSignals(true);
    ui->dan->setValue((int)(floor(dst->pars->getParameterValue(Enumerations::dan, Enumerations::val) * 1.E5)));
    ui->dan->blockSignals(prevState);
    ui->dan_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::dan, Enumerations::val),'f',5));
    draw(true);
    sst->mustSave = true;
}

void MainWindow::set_dan_min_max(){
    ui->dan->setTickInterval((ui->dan->maximum()-ui->dan->minimum()) / 10);
    ui->dan->setSingleStep(ui->dan->tickInterval());
    ui->dan->setPageStep(ui->dan->tickInterval()*2);
}


//k

void MainWindow::on_k_min_editingFinished(){
    bool prevState = ui->k_min->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->k_min->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 3);
        if(temp <= dst->pars->getParameterValue(Enumerations::k, Enumerations::max)){
            dst->pars->setParameterValue(Enumerations::k, Enumerations::min, temp, sst->checkParameters_MainWindow);
            ui->k->setMinimum((int)(floor(dst->pars->getParameterValue(Enumerations::k, Enumerations::min)*1.E3)));
            set_k_min_max();
        }
    }
    ui->k_min->setText(QString::number(dst->pars->getParameterValue(Enumerations::k, Enumerations::min),'f',3));
    delete tempText;
    sst->mustSave = true;
    ui->k_min->blockSignals(prevState); //restore previous state
}

void MainWindow::on_k_max_editingFinished(){
    bool prevState = ui->k_max->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->k_max->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 3);
        if(temp >= dst->pars->getParameterValue(Enumerations::k, Enumerations::min)){
            dst->pars->setParameterValue(Enumerations::k, Enumerations::max, temp, sst->checkParameters_MainWindow);
            ui->k->setMaximum((int)(floor(dst->pars->getParameterValue(Enumerations::k, Enumerations::max)*1.E3)));
            set_k_min_max();
        }
    }
    ui->k_max->setText(QString::number(dst->pars->getParameterValue(Enumerations::k, Enumerations::max),'f',3));
    delete tempText;
    sst->mustSave = true;
    ui->k_max->blockSignals(prevState); //restore previous state
}

void MainWindow::on_k_sub_editingFinished(){
    bool prevState = ui->k_sub->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->k_sub->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 3);
        if (temp >= 0)
            dst->pars->setParameterValue(Enumerations::k, Enumerations::dec, temp, sst->checkParameters_MainWindow);
    }
    ui->k_sub->setText(QString::number(dst->pars->getParameterValue(Enumerations::k, Enumerations::dec),'f',3));
    delete tempText;
    sst->mustSave = true;
    ui->k_sub->blockSignals(prevState); //restore previous state
}

void MainWindow::on_k_add_editingFinished(){
    bool prevState = ui->k_add->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->k_add->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 3);
        if (temp >= 0)
            dst->pars->setParameterValue(Enumerations::k, Enumerations::inc, temp, sst->checkParameters_MainWindow);
    }
    ui->k_add->setText(QString::number(dst->pars->getParameterValue(Enumerations::k, Enumerations::inc),'f',3));
    delete tempText;
    sst->mustSave = true;
    ui->k_add->blockSignals(prevState); //restore previous state
}

void MainWindow::on_k_value_editingFinished(){
    bool prevState = ui->k_value->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->k_value->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 3);
        ui->k->setValue((int)(floor(temp * 1.E3)));
    }
    ui->k_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::k, Enumerations::val),'f',3));
    delete tempText;
    ui->k_value->blockSignals(prevState); //restore previous state
}

void MainWindow::on_k_doSub_clicked(){
    ui->k->setValue((int)(floor((dst->pars->getParameterValue(Enumerations::k, Enumerations::val) - dst->pars->getParameterValue(Enumerations::k, Enumerations::dec)) * 1.E3)));
}

void MainWindow::on_k_doAdd_clicked(){
    ui->k->setValue((int)(floor((dst->pars->getParameterValue(Enumerations::k, Enumerations::val) + dst->pars->getParameterValue(Enumerations::k, Enumerations::inc)) * 1.E3)));
}

void MainWindow::on_k_valueChanged(int value){
    dst->pars->setParameterValue(Enumerations::k, Enumerations::val, (value / 1.E3), sst->checkParameters_MainWindow);
    bool prevState = ui->k->blockSignals(true);
    ui->k->setValue((int)(floor(dst->pars->getParameterValue(Enumerations::k, Enumerations::val) * 1.E3)));
    ui->k->blockSignals(prevState);
    ui->k_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::k, Enumerations::val),'f',3));
    draw(true);
    sst->mustSave = true;
}

void MainWindow::set_k_min_max(){
    ui->k->setTickInterval((ui->k->maximum()-ui->k->minimum()) / 10);
    ui->k->setSingleStep(ui->k->tickInterval());
    ui->k->setPageStep(ui->k->tickInterval()*2);
}


//const1

void MainWindow::on_const1_min_editingFinished(){
    bool prevState = ui->const1_min->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->const1_min->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 3);
        if(temp <= dst->pars->getParameterValue(Enumerations::const1, Enumerations::max)){
            dst->pars->setParameterValue(Enumerations::const1, Enumerations::min, temp, sst->checkParameters_MainWindow);
            ui->const1->setMinimum((int)(floor(dst->pars->getParameterValue(Enumerations::const1, Enumerations::min)*1.E3)));
            set_const1_min_max();
        }
    }
    ui->const1_min->setText(QString::number(dst->pars->getParameterValue(Enumerations::const1, Enumerations::min),'f',3));
    delete tempText;
    sst->mustSave = true;
    ui->const1_min->blockSignals(prevState); //restore previous state
}

void MainWindow::on_const1_max_editingFinished(){
    bool prevState = ui->const1_max->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->const1_max->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 3);
        if(temp >= dst->pars->getParameterValue(Enumerations::const1, Enumerations::min)){
            dst->pars->setParameterValue(Enumerations::const1, Enumerations::max, temp, sst->checkParameters_MainWindow);
            ui->const1->setMaximum((int)(floor(dst->pars->getParameterValue(Enumerations::const1, Enumerations::max)*1.E3)));
            set_const1_min_max();
        }
    }
    ui->const1_max->setText(QString::number(dst->pars->getParameterValue(Enumerations::const1, Enumerations::max),'f',3));
    delete tempText;
    sst->mustSave = true;
    ui->const1_max->blockSignals(prevState); //restore previous state
}

void MainWindow::on_const1_sub_editingFinished(){
    bool prevState = ui->const1_sub->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->const1_sub->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 3);
        if (temp >= 0)
            dst->pars->setParameterValue(Enumerations::const1, Enumerations::dec, temp, sst->checkParameters_MainWindow);
    }
    ui->const1_sub->setText(QString::number(dst->pars->getParameterValue(Enumerations::const1, Enumerations::dec),'f',3));
    delete tempText;
    sst->mustSave = true;
    ui->const1_sub->blockSignals(prevState); //restore previous state
}

void MainWindow::on_const1_add_editingFinished(){
    bool prevState = ui->const1_add->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->const1_add->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 3);
        if (temp >= 0)
            dst->pars->setParameterValue(Enumerations::const1, Enumerations::inc, temp, sst->checkParameters_MainWindow);
    }
    ui->const1_add->setText(QString::number(dst->pars->getParameterValue(Enumerations::const1, Enumerations::inc),'f',3));
    delete tempText;
    sst->mustSave = true;
    ui->const1_add->blockSignals(prevState); //restore previous state
}

void MainWindow::on_const1_value_editingFinished(){
    bool prevState = ui->const1_value->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->const1_value->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 3);
        ui->const1->setValue((int)(floor(temp * 1.E3)));
    }
    ui->const1_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::const1, Enumerations::val),'f',3));
    delete tempText;
    ui->const1_value->blockSignals(prevState); //restore previous state
}

void MainWindow::on_const1_doSub_clicked(){
    ui->const1->setValue((int)(floor((dst->pars->getParameterValue(Enumerations::const1, Enumerations::val) - dst->pars->getParameterValue(Enumerations::const1, Enumerations::dec)) * 1.E3)));
}

void MainWindow::on_const1_doAdd_clicked(){
    ui->const1->setValue((int)(floor((dst->pars->getParameterValue(Enumerations::const1, Enumerations::val) + dst->pars->getParameterValue(Enumerations::const1, Enumerations::inc)) * 1.E3)));
}

void MainWindow::on_const1_valueChanged(int value){
    dst->pars->setParameterValue(Enumerations::const1, Enumerations::val, (value / 1.E3), sst->checkParameters_MainWindow);
    bool prevState = ui->const1->blockSignals(true);
    ui->const1->setValue((int)(floor(dst->pars->getParameterValue(Enumerations::const1, Enumerations::val) * 1.E3)));
    ui->const1->blockSignals(prevState);
    ui->const1_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::const1, Enumerations::val),'f',3));
    draw(true);
    sst->mustSave = true;
}

void MainWindow::set_const1_min_max(){
    ui->const1->setTickInterval((ui->const1->maximum()-ui->const1->minimum()) / 10);
    ui->const1->setSingleStep(ui->const1->tickInterval());
    ui->const1->setPageStep(ui->const1->tickInterval()*2);
}


//const2

void MainWindow::on_const2_min_editingFinished(){
    bool prevState = ui->const2_min->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->const2_min->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if(temp <= dst->pars->getParameterValue(Enumerations::const2, Enumerations::max)){
            dst->pars->setParameterValue(Enumerations::const2, Enumerations::min, temp, sst->checkParameters_MainWindow);
            ui->const2->setMinimum((int)(floor(dst->pars->getParameterValue(Enumerations::const2, Enumerations::min)*1.E5)));
            set_const2_min_max();
        }
    }
    ui->const2_min->setText(QString::number(dst->pars->getParameterValue(Enumerations::const2, Enumerations::min),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->const2_min->blockSignals(prevState); //restore previous state
}

void MainWindow::on_const2_max_editingFinished(){
    bool prevState = ui->const2_max->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->const2_max->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if(temp >= dst->pars->getParameterValue(Enumerations::const2, Enumerations::min)){
            dst->pars->setParameterValue(Enumerations::const2, Enumerations::max, temp, sst->checkParameters_MainWindow);
            ui->const2->setMaximum((int)(floor(dst->pars->getParameterValue(Enumerations::const2, Enumerations::max)*1.E5)));
            set_const2_min_max();
        }
    }
    ui->const2_max->setText(QString::number(dst->pars->getParameterValue(Enumerations::const2, Enumerations::max),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->const2_max->blockSignals(prevState); //restore previous state
}

void MainWindow::on_const2_sub_editingFinished(){
    bool prevState = ui->const2_sub->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->const2_sub->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp >= 0)
            dst->pars->setParameterValue(Enumerations::const2, Enumerations::dec, temp, sst->checkParameters_MainWindow);
    }
    ui->const2_sub->setText(QString::number(dst->pars->getParameterValue(Enumerations::const2, Enumerations::dec),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->const2_sub->blockSignals(prevState); //restore previous state
}

void MainWindow::on_const2_add_editingFinished(){
    bool prevState = ui->const2_add->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->const2_add->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp >= 0)
            dst->pars->setParameterValue(Enumerations::const2, Enumerations::inc, temp, sst->checkParameters_MainWindow);
    }
    ui->const2_add->setText(QString::number(dst->pars->getParameterValue(Enumerations::const2, Enumerations::inc),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->const2_add->blockSignals(prevState); //restore previous state
}

void MainWindow::on_const2_value_editingFinished(){
    bool prevState = ui->const2_value->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->const2_value->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        ui->const2->setValue((int)(floor(temp * 1.E5)));
    }
    ui->const2_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::const2, Enumerations::val),'f',5));
    delete tempText;
    ui->const2_value->blockSignals(prevState); //restore previous state
}

void MainWindow::on_const2_doSub_clicked(){
    ui->const2->setValue((int)(floor((dst->pars->getParameterValue(Enumerations::const2, Enumerations::val) - dst->pars->getParameterValue(Enumerations::const2, Enumerations::dec)) * 1.E5)));
}

void MainWindow::on_const2_doAdd_clicked(){
    ui->const2->setValue((int)(floor((dst->pars->getParameterValue(Enumerations::const2, Enumerations::val) + dst->pars->getParameterValue(Enumerations::const2, Enumerations::inc)) * 1.E5)));
}

void MainWindow::on_const2_valueChanged(int value){
    dst->pars->setParameterValue(Enumerations::const2, Enumerations::val, (value / 1.E5), sst->checkParameters_MainWindow);
    bool prevState = ui->const2->blockSignals(true);
    ui->const2->setValue((int)(floor(dst->pars->getParameterValue(Enumerations::const2, Enumerations::val) * 1.E5)));
    ui->const2->blockSignals(prevState);
    ui->const2_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::const2, Enumerations::val),'f',5));
    draw(true);
    sst->mustSave = true;
}

void MainWindow::set_const2_min_max(){
    ui->const2->setTickInterval((ui->const2->maximum()-ui->const2->minimum()) / 10);
    ui->const2->setSingleStep(ui->const2->tickInterval());
    ui->const2->setPageStep(ui->const2->tickInterval()*2);
}


//g

void MainWindow::on_g_min_editingFinished(){
    bool prevState = ui->g_min->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->g_min->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if(temp <= dst->pars->getParameterValue(Enumerations::g, Enumerations::max)){
            dst->pars->setParameterValue(Enumerations::g, Enumerations::min, temp, sst->checkParameters_MainWindow);
            ui->g->setMinimum((int)(floor(dst->pars->getParameterValue(Enumerations::g, Enumerations::min)*1.E5)));
            set_g_min_max();
        }
    }
    ui->g_min->setText(QString::number(dst->pars->getParameterValue(Enumerations::g, Enumerations::min),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->g_min->blockSignals(prevState); //restore previous state
}

void MainWindow::on_g_max_editingFinished(){
    bool prevState = ui->g_max->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->g_max->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if(temp >= dst->pars->getParameterValue(Enumerations::g, Enumerations::min)){
            dst->pars->setParameterValue(Enumerations::g, Enumerations::max, temp, sst->checkParameters_MainWindow);
            ui->g->setMaximum((int)(floor(dst->pars->getParameterValue(Enumerations::g, Enumerations::max)*1.E5)));
            set_g_min_max();
        }
    }
    ui->g_max->setText(QString::number(dst->pars->getParameterValue(Enumerations::g, Enumerations::max),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->g_max->blockSignals(prevState); //restore previous state
}

void MainWindow::on_g_sub_editingFinished(){
    bool prevState = ui->g_sub->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->g_sub->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp >= 0)
            dst->pars->setParameterValue(Enumerations::g, Enumerations::dec, temp, sst->checkParameters_MainWindow);
    }
    ui->g_sub->setText(QString::number(dst->pars->getParameterValue(Enumerations::g, Enumerations::dec),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->g_sub->blockSignals(prevState); //restore previous state
}

void MainWindow::on_g_add_editingFinished(){
    bool prevState = ui->g_add->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->g_add->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp >= 0)
            dst->pars->setParameterValue(Enumerations::g, Enumerations::inc, temp, sst->checkParameters_MainWindow);
    }
    ui->g_add->setText(QString::number(dst->pars->getParameterValue(Enumerations::g, Enumerations::inc),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->g_add->blockSignals(prevState); //restore previous state
}

void MainWindow::on_g_value_editingFinished(){
    bool prevState = ui->g_value->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->g_value->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        ui->g->setValue((int)(floor(temp * 1.E5)));
    }
    ui->g_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::g, Enumerations::val),'f',5));
    delete tempText;
    ui->g_value->blockSignals(prevState); //restore previous state
}

void MainWindow::on_g_doSub_clicked(){
    ui->g->setValue((int)(floor((dst->pars->getParameterValue(Enumerations::g, Enumerations::val) - dst->pars->getParameterValue(Enumerations::g, Enumerations::dec)) * 1.E5)));
}

void MainWindow::on_g_doAdd_clicked(){
    ui->g->setValue((int)(floor((dst->pars->getParameterValue(Enumerations::g, Enumerations::val) + dst->pars->getParameterValue(Enumerations::g, Enumerations::inc)) * 1.E5)));
}

void MainWindow::on_g_valueChanged(int value){
    dst->pars->setParameterValue(Enumerations::g, Enumerations::val, (value / 1.E5), sst->checkParameters_MainWindow);
    bool prevState = ui->g->blockSignals(true);
    ui->g->setValue((int)(floor(dst->pars->getParameterValue(Enumerations::g, Enumerations::val) * 1.E5)));
    ui->g->blockSignals(prevState);
    ui->g_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::g, Enumerations::val),'f',5));
    draw(true);
    sst->mustSave = true;
}

void MainWindow::set_g_min_max(){
    ui->g->setTickInterval((ui->g->maximum()-ui->g->minimum()) / 10);
    ui->g->setSingleStep(ui->g->tickInterval());
    ui->g->setPageStep(ui->g->tickInterval()*2);
}


//mue_ab

void MainWindow::on_mue_ab_min_editingFinished(){
    bool prevState = ui->mue_ab_min->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->mue_ab_min->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if(temp <= dst->pars->getParameterValue(Enumerations::mue_ab, Enumerations::max)){
            dst->pars->setParameterValue(Enumerations::mue_ab, Enumerations::min, temp, sst->checkParameters_MainWindow);
            ui->mue_ab->setMinimum((int)(floor(dst->pars->getParameterValue(Enumerations::mue_ab, Enumerations::min)*1.E5)));
            set_mue_ab_min_max();
        }
    }
    ui->mue_ab_min->setText(QString::number(dst->pars->getParameterValue(Enumerations::mue_ab, Enumerations::min),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->mue_ab_min->blockSignals(prevState); //restore previous state
}

void MainWindow::on_mue_ab_max_editingFinished(){
    bool prevState = ui->mue_ab_max->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->mue_ab_max->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if(temp >= dst->pars->getParameterValue(Enumerations::mue_ab, Enumerations::min)){
            dst->pars->setParameterValue(Enumerations::mue_ab, Enumerations::max, temp, sst->checkParameters_MainWindow);
            ui->mue_ab->setMaximum((int)(floor(dst->pars->getParameterValue(Enumerations::mue_ab, Enumerations::max)*1.E5)));
            set_mue_ab_min_max();
        }
    }
    ui->mue_ab_max->setText(QString::number(dst->pars->getParameterValue(Enumerations::mue_ab, Enumerations::max),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->mue_ab_max->blockSignals(prevState); //restore previous state
}

void MainWindow::on_mue_ab_sub_editingFinished(){
    bool prevState = ui->mue_ab_sub->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->mue_ab_sub->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp >= 0)
            dst->pars->setParameterValue(Enumerations::mue_ab, Enumerations::dec, temp, sst->checkParameters_MainWindow);
    }
    ui->mue_ab_sub->setText(QString::number(dst->pars->getParameterValue(Enumerations::mue_ab, Enumerations::dec),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->mue_ab_sub->blockSignals(prevState); //restore previous state
}

void MainWindow::on_mue_ab_add_editingFinished(){
    bool prevState = ui->mue_ab_add->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->mue_ab_add->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp >= 0)
            dst->pars->setParameterValue(Enumerations::mue_ab, Enumerations::inc, temp, sst->checkParameters_MainWindow);
    }
    ui->mue_ab_add->setText(QString::number(dst->pars->getParameterValue(Enumerations::mue_ab, Enumerations::inc),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->mue_ab_add->blockSignals(prevState); //restore previous state
}

void MainWindow::on_mue_ab_value_editingFinished(){
    bool prevState = ui->mue_ab_value->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->mue_ab_value->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        ui->mue_ab->setValue((int)(floor(temp * 1.E5)));
    }
    ui->mue_ab_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::mue_ab, Enumerations::val),'f',5));
    delete tempText;
    ui->mue_ab_value->blockSignals(prevState); //restore previous state
}

void MainWindow::on_mue_ab_doSub_clicked(){
    ui->mue_ab->setValue((int)(floor((dst->pars->getParameterValue(Enumerations::mue_ab, Enumerations::val) - dst->pars->getParameterValue(Enumerations::mue_ab, Enumerations::dec)) * 1.E5)));
}

void MainWindow::on_mue_ab_doAdd_clicked(){
    ui->mue_ab->setValue((int)(floor((dst->pars->getParameterValue(Enumerations::mue_ab, Enumerations::val) + dst->pars->getParameterValue(Enumerations::mue_ab, Enumerations::inc)) * 1.E5)));
}

void MainWindow::on_mue_ab_valueChanged(int value){
    dst->pars->setParameterValue(Enumerations::mue_ab, Enumerations::val, (value / 1.E5), sst->checkParameters_MainWindow);
    bool prevState = ui->mue_ab->blockSignals(true);
    ui->mue_ab->setValue((int)(floor(dst->pars->getParameterValue(Enumerations::mue_ab, Enumerations::val) * 1.E5)));
    ui->mue_ab->blockSignals(prevState);
    ui->mue_ab_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::mue_ab, Enumerations::val),'f',5));
    draw(true);
    sst->mustSave = true;
}

void MainWindow::set_mue_ab_min_max(){
    ui->mue_ab->setTickInterval((ui->mue_ab->maximum()-ui->mue_ab->minimum()) / 10);
    ui->mue_ab->setSingleStep(ui->mue_ab->tickInterval());
    ui->mue_ab->setPageStep(ui->mue_ab->tickInterval()*2);
}

//p

void MainWindow::on_p_min_editingFinished(){
    bool prevState = ui->p_min->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->p_min->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if(temp <= dst->pars->getParameterValue(Enumerations::p, Enumerations::max)){
            dst->pars->setParameterValue(Enumerations::p, Enumerations::min, temp, sst->checkParameters_MainWindow);
            ui->p->setMinimum((int)(floor(dst->pars->getParameterValue(Enumerations::p, Enumerations::min)*1.E5)));
            set_p_min_max();
        }
    }
    ui->p_min->setText(QString::number(dst->pars->getParameterValue(Enumerations::p, Enumerations::min),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->p_min->blockSignals(prevState); //restore previous state
}

void MainWindow::on_p_max_editingFinished(){
    bool prevState = ui->p_max->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->p_max->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if(temp >= dst->pars->getParameterValue(Enumerations::p, Enumerations::min)){
            dst->pars->setParameterValue(Enumerations::p, Enumerations::max, temp, sst->checkParameters_MainWindow);
            ui->p->setMaximum((int)(floor(dst->pars->getParameterValue(Enumerations::p, Enumerations::max)*1.E5)));
            set_p_min_max();
        }
    }
    ui->p_max->setText(QString::number(dst->pars->getParameterValue(Enumerations::p, Enumerations::max),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->p_max->blockSignals(prevState); //restore previous state
}

void MainWindow::on_p_sub_editingFinished(){
    bool prevState = ui->p_sub->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->p_sub->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp >= 0)
            dst->pars->setParameterValue(Enumerations::p, Enumerations::dec, temp, sst->checkParameters_MainWindow);
    }
    ui->p_sub->setText(QString::number(dst->pars->getParameterValue(Enumerations::p, Enumerations::dec),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->p_sub->blockSignals(prevState); //restore previous state
}

void MainWindow::on_p_add_editingFinished(){
    bool prevState = ui->p_add->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->p_add->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp >= 0)
            dst->pars->setParameterValue(Enumerations::p, Enumerations::inc, temp, sst->checkParameters_MainWindow);
    }
    ui->p_add->setText(QString::number(dst->pars->getParameterValue(Enumerations::p, Enumerations::inc),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->p_add->blockSignals(prevState); //restore previous state
}

void MainWindow::on_p_value_editingFinished(){
    bool prevState = ui->p_value->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->p_value->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        ui->p->setValue((int)(floor(temp * 1.E5)));
    }
    ui->p_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::p, Enumerations::val),'f',5));
    delete tempText;
    ui->p_value->blockSignals(prevState); //restore previous state
}

void MainWindow::on_p_doSub_clicked(){
    ui->p->setValue((int)(floor((dst->pars->getParameterValue(Enumerations::p, Enumerations::val) - dst->pars->getParameterValue(Enumerations::p, Enumerations::dec)) * 1.E5)));
}

void MainWindow::on_p_doAdd_clicked(){
    ui->p->setValue((int)(floor((dst->pars->getParameterValue(Enumerations::p, Enumerations::val) + dst->pars->getParameterValue(Enumerations::p, Enumerations::inc)) * 1.E5)));
}

void MainWindow::on_p_valueChanged(int value){
    dst->pars->setParameterValue(Enumerations::p, Enumerations::val, (value / 1.E5), sst->checkParameters_MainWindow);
    bool prevState = ui->p->blockSignals(true);
    ui->p->setValue((int)(floor(dst->pars->getParameterValue(Enumerations::p, Enumerations::val) * 1.E5)));
    ui->p->blockSignals(prevState);
    ui->p_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::p, Enumerations::val),'f',5));
    draw(true);
    sst->mustSave = true;
}

void MainWindow::set_p_min_max(){
    ui->p->setTickInterval((ui->p->maximum()-ui->p->minimum()) / 10);
    ui->p->setSingleStep(ui->p->tickInterval());
    ui->p->setPageStep(ui->p->tickInterval()*2);
}


//R

void MainWindow::on_R_min_editingFinished(){
    bool prevState = ui->R_min->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->R_min->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if(temp <= dst->pars->getParameterValue(Enumerations::R, Enumerations::max)){
            dst->pars->setParameterValue(Enumerations::R, Enumerations::min, temp, sst->checkParameters_MainWindow);
            ui->R->setMinimum((int)(floor(dst->pars->getParameterValue(Enumerations::R, Enumerations::min)*1.E5)));
            set_R_min_max();
        }
    }
    ui->R_min->setText(QString::number(dst->pars->getParameterValue(Enumerations::R, Enumerations::min),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->R_min->blockSignals(prevState); //restore previous state
}

void MainWindow::on_R_max_editingFinished(){
    bool prevState = ui->R_max->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->R_max->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if(temp >= dst->pars->getParameterValue(Enumerations::R, Enumerations::min)){
            dst->pars->setParameterValue(Enumerations::R, Enumerations::max, temp, sst->checkParameters_MainWindow);
            ui->R->setMaximum((int)(floor(dst->pars->getParameterValue(Enumerations::R, Enumerations::max)*1.E5)));
            set_R_min_max();
        }
    }
    ui->R_max->setText(QString::number(dst->pars->getParameterValue(Enumerations::R, Enumerations::max),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->R_max->blockSignals(prevState); //restore previous state
}

void MainWindow::on_R_sub_editingFinished(){
    bool prevState = ui->R_sub->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->R_sub->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp >= 0)
            dst->pars->setParameterValue(Enumerations::R, Enumerations::dec, temp, sst->checkParameters_MainWindow);
    }
    ui->R_sub->setText(QString::number(dst->pars->getParameterValue(Enumerations::R, Enumerations::dec),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->R_sub->blockSignals(prevState); //restore previous state
}

void MainWindow::on_R_add_editingFinished(){
    bool prevState = ui->R_add->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->R_add->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp >= 0)
            dst->pars->setParameterValue(Enumerations::R, Enumerations::inc, temp, sst->checkParameters_MainWindow);
    }
    ui->R_add->setText(QString::number(dst->pars->getParameterValue(Enumerations::R, Enumerations::inc),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->R_add->blockSignals(prevState); //restore previous state
}

void MainWindow::on_R_value_editingFinished(){
    bool prevState = ui->R_value->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->R_value->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        ui->R->setValue((int)(floor(temp * 1.E5)));
    }
    ui->R_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::R, Enumerations::val),'f',5));
    delete tempText;
    ui->R_value->blockSignals(prevState); //restore previous state
}

void MainWindow::on_R_doSub_clicked(){
    ui->R->setValue((int)(floor((dst->pars->getParameterValue(Enumerations::R, Enumerations::val) - dst->pars->getParameterValue(Enumerations::R, Enumerations::dec)) * 1.E5)));
}

void MainWindow::on_R_doAdd_clicked(){
    ui->R->setValue((int)(floor((dst->pars->getParameterValue(Enumerations::R, Enumerations::val) + dst->pars->getParameterValue(Enumerations::R, Enumerations::inc)) * 1.E5)));
}

void MainWindow::on_R_valueChanged(int value){
    dst->pars->setParameterValue(Enumerations::R, Enumerations::val, (value / 1.E5), sst->checkParameters_MainWindow);
    bool prevState = ui->R->blockSignals(true);
    ui->R->setValue((int)(floor(dst->pars->getParameterValue(Enumerations::R, Enumerations::val) * 1.E5)));
    ui->R->blockSignals(prevState);
    ui->R_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::R, Enumerations::val),'f',5));
    draw(true);
    sst->mustSave = true;
}

void MainWindow::set_R_min_max(){
    ui->R->setTickInterval((ui->R->maximum()-ui->R->minimum()) / 10);
    ui->R->setSingleStep(ui->R->tickInterval());
    ui->R->setPageStep(ui->R->tickInterval()*2);
}


//t

void MainWindow::on_t_min_editingFinished(){
    bool prevState = ui->t_min->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->t_min->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if(temp <= dst->pars->getParameterValue(Enumerations::t, Enumerations::max)){
            dst->pars->setParameterValue(Enumerations::t, Enumerations::min, temp, sst->checkParameters_MainWindow);
            ui->t->setMinimum((int)(floor(dst->pars->getParameterValue(Enumerations::t, Enumerations::min)*1.E5)));
            set_t_min_max();
        }
    }
    ui->t_min->setText(QString::number(dst->pars->getParameterValue(Enumerations::t, Enumerations::min),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->t_min->blockSignals(prevState); //restore previous state
}

void MainWindow::on_t_max_editingFinished(){
    bool prevState = ui->t_max->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->t_max->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if(temp >= dst->pars->getParameterValue(Enumerations::t, Enumerations::min)){
            dst->pars->setParameterValue(Enumerations::t, Enumerations::max, temp, sst->checkParameters_MainWindow);
            ui->t->setMaximum((int)(floor(dst->pars->getParameterValue(Enumerations::t, Enumerations::max)*1.E5)));
            set_t_min_max();
        }
    }
    ui->t_max->setText(QString::number(dst->pars->getParameterValue(Enumerations::t, Enumerations::max),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->t_max->blockSignals(prevState); //restore previous state
}

void MainWindow::on_t_sub_editingFinished(){
    bool prevState = ui->t_sub->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->t_sub->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp >= 0)
            dst->pars->setParameterValue(Enumerations::t, Enumerations::dec, temp, sst->checkParameters_MainWindow);
    }
    ui->t_sub->setText(QString::number(dst->pars->getParameterValue(Enumerations::t, Enumerations::dec),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->t_sub->blockSignals(prevState); //restore previous state
}

void MainWindow::on_t_add_editingFinished(){
    bool prevState = ui->t_add->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->t_add->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp >= 0)
            dst->pars->setParameterValue(Enumerations::t, Enumerations::inc, temp, sst->checkParameters_MainWindow);
    }
    ui->t_add->setText(QString::number(dst->pars->getParameterValue(Enumerations::t, Enumerations::inc),'f',5));
    delete tempText;
    sst->mustSave = true;
    ui->t_add->blockSignals(prevState); //restore previous state
}

void MainWindow::on_t_value_editingFinished(){
    bool prevState = ui->t_value->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->t_value->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        ui->t->setValue((int)(floor(temp * 1.E5)));
    }
    ui->t_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::t, Enumerations::val),'f',5));
    delete tempText;
    ui->t_value->blockSignals(prevState); //restore previous state
}

void MainWindow::on_t_doSub_clicked(){
    ui->t->setValue((int)(floor((dst->pars->getParameterValue(Enumerations::t, Enumerations::val) - dst->pars->getParameterValue(Enumerations::t, Enumerations::dec)) * 1.E5)));
}

void MainWindow::on_t_doAdd_clicked(){
    ui->t->setValue((int)(floor((dst->pars->getParameterValue(Enumerations::t, Enumerations::val) + dst->pars->getParameterValue(Enumerations::t, Enumerations::inc)) * 1.E5)));
}

void MainWindow::on_t_valueChanged(int value){
    dst->pars->setParameterValue(Enumerations::t, Enumerations::val, (value / 1.E5), sst->checkParameters_MainWindow);
    bool prevState = ui->t->blockSignals(true);
    ui->t->setValue((int)(floor(dst->pars->getParameterValue(Enumerations::t, Enumerations::val) * 1.E5)));
    ui->t->blockSignals(prevState);
    ui->t_value->setText(QString::number(dst->pars->getParameterValue(Enumerations::t, Enumerations::val),'f',5));
    draw(true);
    sst->mustSave = true;
}

void MainWindow::set_t_min_max(){
    ui->t->setTickInterval((ui->t->maximum()-ui->t->minimum()) / 10);
    ui->t->setSingleStep(ui->t->tickInterval());
    ui->t->setPageStep(ui->t->tickInterval()*2);
}



//Slots


void MainWindow::on_action_New_triggered(){
    if(saveMaybe()){

        sst->drawData = false;
        sst->drawFit  = false;
        sst->drawDev  = false;
        sst->reCheck_buttonShowDeviation_MainWindow = false;

        sst->reCheck_optA_MainWindow = false;
        sst->reCheck_optIntensity_MainWindow = false;

        dst->prgpar->setDefaultValues(sst->prgset->adjustAxisIntervals);

        delete dst->parsPath;
        dst->parsPath = 0;
        if (dst->readParsDefaultValues(dst->pars)){ // read default values
            set_slider_values();           // also initialises sliders
            dst->readParsDefaultValues(dst->pars); //  read default values again as initalisation of sliders may changed some values
            set_slider_values();          //  set slider values again
        }
        else
            AuxiliaryMethods::errorMessage(this, new QString(tr("Die Liste der Parameter-Standardwerte \"pars-defaultValues_de.txt\"\nkonnte nicht geladen werden.\nDie zuvor eingestellten Werte werden daher beibehalten sofern\ndiese nicht durch Laden einer NGC-Datei ueberschrieben werden.")));

        delete dst->dataPath;
        dst->dataPath = 0;
        dst->data->clear();
        dst->data_cor->clear();
        dst->data_for_ChooseTemplate_AutoFit->clear();
        dst->fit->clear();

        curveData->setData(new QwtPointSeriesData(*(dst->data_cor))); //needed for correct reset of Y-axis scale
        curveFit->setData(new QwtPointSeriesData(*(dst->fit))); //needed for correct reset of Y-axis scale

        set_widget_values(false);

        dev->resetPlot();

        dst->chosenTemplate = 0;
        ui->action_ValuesBackToTemplate->setDisabled(true);

        ui->scrollBarControls->setValue(0);
        sst->mustSave = false;
    }

}

void MainWindow::on_action_Open_triggered(){
    if (saveMaybe()){
        sst->mustSave = false;
        bool mustSave_dueToDataReload = false; // if data file was not found and loaded from another location
        ui->action_New->trigger();
        Enumerations::errorType result = (pfa->loadPars(dst, sst));
        if (result == Enumerations::NoError){

            //set window title
            setWindowCaption();

            //set value of scrollBarControls
            ui->scrollBarControls->setValue(sst->scrollBarControls_Value_temp);

            //set wavelength
            bool prevState = ui->comboBox_Wavelengths->blockSignals(true); //save previous state
            int pos = AuxiliaryMethods::findWavelength(dst->prgpar->currentWavelength, dst->wavelengths);
            if (pos >= 0)
                ui->comboBox_Wavelengths->setCurrentIndex(pos);
            else{
                dst->wavelengths->append(*(dst->prgpar->currentWavelength));
                ui->comboBox_Wavelengths->addItem( ( (dst->prgpar->currentWavelength->name.length() > 0) ? (dst->prgpar->currentWavelength->name + " - ") : "" ) + QString::number(dst->prgpar->currentWavelength->value,'g',10));
                ui->comboBox_Wavelengths->setCurrentIndex(dst->wavelengths->size() - 1);
                ui->buttonDeleteWavelength->setEnabled(true);
            }
            ui->comboBox_Wavelengths->blockSignals(prevState); //restore previous state

            //set slider values twice due to an issue with QSlider
            set_slider_values();
            pfa->readSliderPars(new QString(*(dst->parsPath)), dst);
            set_slider_values();

            sst->mustSave = false; //set again to false for correct storing of result of "click" an buttonLoad
            //read measurement data if needed
            if (dst->dataPath != 0)
                if (! dst->readDataFile()){
                    sst->drawData = false;
                    delete dst->dataPath;
                    dst->dataPath = 0;
                    QMessageBox *msg = new QMessageBox(this);
                    msg->setIcon(QMessageBox::Question);
                    msg->setText(tr("Die zur Anpassung gehoerigen Messwerte wurden nicht gefunden. Moechten Sie diese neu laden?"));
                    msg->setStandardButtons(QMessageBox::Yes | QMessageBox::No);
                    msg->setDefaultButton(QMessageBox::Yes);
                    if (msg->exec() == QMessageBox::Yes){
                        prevState = sst->prgset->enableChooseTemplate; //save previous state
                        sst->prgset->enableChooseTemplate = false;
                        ui->buttonLoad->click();
                        mustSave_dueToDataReload = sst->mustSave;
                        sst->prgset->enableChooseTemplate = prevState; //restore previous state
                    }
                }

            set_widget_values(false);

        }
        else
            if (result == Enumerations::OtherError)
                AuxiliaryMethods::errorMessage(this, new QString(tr("Laden der Parameter fehlgeschlagen")));
        sst->mustSave = mustSave_dueToDataReload;
    }

}


void MainWindow::on_action_Save_triggered(){
    if (dst->parsPath != 0){
        if ( !( pfa->writePars( new QString(*(dst->parsPath)), dst, sst, ui->scrollBarControls->value() ) ) )
             AuxiliaryMethods::errorMessage(this, new QString(tr("Speichern der Parameter fehlgeschlagen")));
        else
            setWindowCaption();
    }
    else
        ui->action_SaveAs->trigger();
}


void MainWindow::on_action_SaveAs_triggered(){
    if ( !( pfa->savePars(dst, sst, ui->scrollBarControls->value() ) ) )
        AuxiliaryMethods::errorMessage(this, new QString(tr("Speichern der Parameter fehlgeschlagen")));
    else
        setWindowCaption();
}

void MainWindow::on_action_SaveAsTemplate_triggered(){
    if ( !(pfa->saveParsTPL(dst, sst->prgset->defaultDir)) )
        AuxiliaryMethods::errorMessage(this, new QString(tr("Speichern der Vorlage fehlgeschlagen!\nEs wurde eventuell eine fehlerhafte Datei erzeugt,\ndie keinesfalls verwendet werden darf!")));
}


void MainWindow::on_action_Close_triggered(){
    close();
}


void MainWindow::on_action_simpleMode_triggered(bool checked){
    sst->prgset->simpleMode = checked;
    applyProgramSettings(false);
}


void MainWindow::on_action_autoUpdateFit_triggered(bool checked){
    sst->prgset->autoUpdateFit = checked;
    applyProgramSettings(false);
}


void MainWindow::on_action_Settings_triggered(){
    Dialog_Preferences *dlg_pref = new Dialog_Preferences(sst->prgset);
    if ( (dlg_pref->exec()) == (QDialog::Accepted)){
        if (dst->dataPath != 0){
            if (sst->prgset->displayFullPath)
                ui->buttonLoad->setText(tr("Datensatz laden  ---  aktuell geladen: ") + *(dst->dataPath));
            else{
                QStringList splitted = dst->dataPath->split("/");
                ui->buttonLoad->setText(tr("Datensatz laden  ---  aktuell geladen: ") + splitted.at(splitted.size() - 1));
            }
        }
        setWindowCaption();
        updateCurves();
        dev->updateCurves();
    }
    delete dlg_pref;
}


void MainWindow::on_action_ChooseTemplate_triggered(){
    chooseTemplate();
}


void MainWindow::on_action_ValuesBackToTemplate_triggered(){
    sst->mustSave = true;
    loadTemplate(dst->chosenTemplate);
    sst->prgset->autoUpdateFit = true; //temporary so that calling draw() updates the plot
    draw(true);
    bool prevState = sst->prgset->autoUpdateFit; //save previous state
    sst->prgset->autoUpdateFit = prevState; //restore previous state
}


void MainWindow::on_scrollBarH_valueChanged(int value){
    ui->groupBoxFileActions->move(-value + 10, ui->groupBoxFileActions->y());
    ui->groupBoxControls->move(-value + 10, ui->groupBoxControls->y());
    ui->widgetPlot->move(-value + 810, ui->widgetPlot->y());
    ui->widget_FitPoints_UpdateFit->move(-value + 810, ui->widget_FitPoints_UpdateFit->y());
    ui->groupBoxParSettings->move(-value + 810, ui->groupBoxParSettings->y());
}


void MainWindow::on_scrollBarV_valueChanged(int value){
    ui->groupBoxFileActions->move(ui->groupBoxFileActions->x(), -value + 10);
    ui->groupBoxControls->move(ui->groupBoxControls->x(), -value + 230);
    ui->widgetPlot->move(ui->widgetPlot->x(), -value + 10);
    ui->widget_FitPoints_UpdateFit->move(ui->widget_FitPoints_UpdateFit->x(), -value + 655);
    ui->groupBoxParSettings->move(ui->groupBoxParSettings->x(), -value + 705);
}


void MainWindow::on_scrollBarControls_valueChanged(int value){

    int movement = -90 * value;

    //move by x = 0, y = movement + y(Widget)
    ui->widget_cno->move(0, movement + 10);
    ui->widget_mu->move(0, movement + 100);
    ui->widget_beta->move(0, movement + 190);
    ui->widget_a3->move(0, movement + 280);
    ui->widget_da3->move(0, movement + 370);
    ui->widget_sig3->move(0, movement + 460);
    ui->widget_u3->move(0, movement + 550);
    ui->widget_eta->move(0, movement + 640);
    ui->widget_k->move(0, movement + 730);
    ui->widget_const1->move(0, movement + 820);
    ui->widget_mue_ab->move(0, movement + 910);
    ui->widget_R->move(0, movement + 1000);

    //move by x = 380, y = movement + y(Widget)
    ui->widget_nu->move(380, movement + 10);
    ui->widget_alpha->move(380, movement + 100);
    ui->widget_sig1->move(380, movement + 190);
    ui->widget_lcc->move(380, movement + 280);
    ui->widget_q->move(380, movement + 370);
    ui->widget_cN->move(380, movement + 460);
    ui->widget_cO->move(380, movement + 550);
    ui->widget_dan->move(380, movement + 640);
    ui->widget_g->move(380, movement + 730);
    ui->widget_const2->move(380, movement + 820);
    ui->widget_p->move(380, movement + 910);
    ui->widget_t->move(380, movement + 1000);

    sst->mustSave = true;

}


void MainWindow::on_buttonLoad_clicked(){
    Enumerations::errorType result = dst->loadDataFile(sst->prgset->defaultDir);
    if (result == Enumerations::NoError){
        sst->mustSave = true;
        Dialog_FileConversion *dlg_FC = new Dialog_FileConversion();
        dlg_FC->exec();
        dst->prgpar->XFormat = dlg_FC->get_XFormat();
        dst->prgpar->YFormat = dlg_FC->get_YFormat();
        delete dlg_FC;
        datacal->dataConversion(dst->data, dst->prgpar->XFormat, dst->prgpar->YFormat, dst->prgpar->currentWavelength->value);
        sst->drawData = true;
        ui->buttonDrawData->setChecked(sst->drawData);
        bool prevState = sst->prgset->autoUpdateFit; //save previous state
        sst->prgset->autoUpdateFit = true; //temporary so that calling draw() updates the plot
        draw(true);
        sst->prgset->autoUpdateFit = prevState; //restore previous state
        if (sst->prgset->enableChooseTemplate)
            chooseTemplate();
        check_uncheck_buttons();
        if (sst->prgset->displayFullPath)
            ui->buttonLoad->setText(tr("Datensatz laden  ---  aktuell geladen: ") + *(dst->dataPath));
        else{
            QStringList splitted = dst->dataPath->split("/");
            ui->buttonLoad->setText(tr("Datensatz laden  ---  aktuell geladen: ") + splitted.at(splitted.size() - 1));
        }
    }
    else{
        if (result == Enumerations::OtherError){
            delete dst->dataPath;
            dst->dataPath = 0;
            AuxiliaryMethods::errorMessage(this, new QString(tr("Laden der Messwerte fehlgeschlagen")));
        }
    }
}


void MainWindow::on_buttonDrawData_clicked(bool checked){
    sst->drawData = checked;
    if (sst->drawData){
        if (dst->dataPath == 0){
            QMessageBox *msg = new QMessageBox(this);
            msg->setIcon(QMessageBox::Question);
            msg->setText(tr("Es sind keine Messwerte geladen.\nMoechten Sie diese jetzt laden?"));
            msg->setStandardButtons(QMessageBox::Yes | QMessageBox::No);
            msg->setDefaultButton(QMessageBox::Yes);
            if (msg->exec() == QMessageBox::Yes){
                ui->buttonLoad->click();
                if (dst->dataPath == 0)
                    sst->drawData = false;
            }
            else
                sst->drawData = false;
            ui->buttonDrawData->setChecked(sst->drawData);
        }
    }
    draw(true);
    check_uncheck_buttons();
    sst->mustSave = true;
}


void MainWindow::on_buttonDrawFit_clicked(bool checked){
    sst->drawFit = checked;
    if (sst->prgset->autoUpdateFit)
        draw(true);
    else
        ui->buttonUpdateFit->click();
    check_uncheck_buttons();
    sst->mustSave = true;
}

void MainWindow::on_buttonShowDeviation_clicked(bool checked){
    if (checked)
        if (sst->drawData && sst->drawFit){
            sst->drawDev = true;
            dev->show();
            sst->mustSave = true;
        }
        else{
            AuxiliaryMethods::errorMessage(this, new QString(tr("Die Darstellung von Abweichungen ist nur moeglich,\nwenn sowohl die Messwerte als auch die Anpassung\ndargestellt werden!")));
            ui->buttonShowDeviation->setChecked(false);
        }
    else{
        sst->drawDev = false;
        dev->hide();
        sst->reCheck_buttonShowDeviation_MainWindow = false;
        sst->mustSave = true;
    }
    check_uncheck_buttons();
}


void MainWindow::on_buttonDrawAll_clicked(bool checked){
    ui->buttonDrawData->setChecked(checked);
    on_buttonDrawData_clicked(checked);
    ui->buttonDrawFit->setChecked(checked);
    on_buttonDrawFit_clicked(checked);
    ui->buttonShowDeviation->setChecked(checked);
    ui->buttonShowDeviation->clicked(checked);
}

void MainWindow::on_buttonAutoFit_clicked(){

    if (sst->drawData && sst->drawFit){
        //disable almost all widgets in MainWindow
        ui->menuBar->setDisabled(true);
        ui->buttonLoad->setDisabled(true);
        ui->buttonDrawData->setDisabled(true);
        ui->buttonDrawFit->setDisabled(true);
        ui->buttonDrawAll->setDisabled(true);
        ui->buttonAutoFit->setDisabled(true);
        ui->buttonAutoFit->setChecked(true);
        ui->groupBoxParSettings->setDisabled(true);
        ui->widget_cN->setDisabled(true);
        ui->widget_cO->setDisabled(true);
        ui->widget_mue_ab->setDisabled(true);
        ui->widget_p->setDisabled(true);
        ui->widget_R->setDisabled(true);
        ui->widget_t->setDisabled(true);
        afs = new AutoFitSettings(dst, sst, pfa, this);
        afs->show();
    }
    else{
        ui->buttonAutoFit->setChecked(false);
       AuxiliaryMethods::errorMessage(this, new QString(tr("Die automatische Anpassung ist nur moeglich,\nwenn sowohl die Messwerte als auch die\nAnpassung dargestellt werden!")));
    }
}


void MainWindow::on_buttonOutputPars_clicked(){
    QFileDialog *dlg = new QFileDialog(this,tr("Liste der Parameter ausgeben"),*(sst->prgset->defaultDir),tr("Textdatei (*.txt)"));
    dlg->setAcceptMode(QFileDialog::AcceptSave);
    dlg->setFileMode(QFileDialog::AnyFile);
    dlg->setHistory(QStringList());
    dlg->setViewMode(QFileDialog::List);
    dlg->setDefaultSuffix("txt");
    if (dlg->exec()){
        QFile *file = new QFile(dlg->selectedFiles().at(0));
        if (file->open(QIODevice::WriteOnly | QIODevice::Truncate | QIODevice::Text)){
            QTextStream *out = new QTextStream(file);
            out->setRealNumberPrecision(12);
            QString *sep = new QString ("\x09\x09"); //separator for columns lines
            for (int i = 0; i < Enumerations::countOf_parName; ++i)
                (*out) << dst->parsMeaning->at(i).symbol << *sep << dst->pars->getParameterValue(Enumerations::parName(i), Enumerations::val) << *sep << dst->parsMeaning->at(i).meaning << "\n";

            /* La    */ (*out) << dst->parsMeaning->at(24).symbol << *sep << ((dst->pars->getParameterValue(Enumerations::nu, Enumerations::val) + 1.) / dst->pars->getParameterValue(Enumerations::alpha, Enumerations::val)) << *sep << dst->parsMeaning->at(24).meaning << "\n";
            /* ‹l›   */ (*out) << dst->parsMeaning->at(25).symbol << *sep << ((dst->pars->getParameterValue(Enumerations::nu, Enumerations::val))  / (dst->pars->getParameterValue(Enumerations::alpha, Enumerations::val))) << *sep << dst->parsMeaning->at(25).meaning << "\n";
            /* kapa  */ (*out) << dst->parsMeaning->at(26).symbol << *sep << (1./(dst->pars->getParameterValue(Enumerations::nu, Enumerations::val))) << *sep << dst->parsMeaning->at(26).meaning << "\n";
            /* ‹N›   */ (*out) << dst->parsMeaning->at(27).symbol << *sep << ((dst->pars->getParameterValue(Enumerations::mu, Enumerations::val))  / (dst->pars->getParameterValue(Enumerations::beta, Enumerations::val))) << *sep << dst->parsMeaning->at(27).meaning << "\n";
            /* N*    */ (*out) << dst->parsMeaning->at(28).symbol << *sep << ( ( (dst->pars->getParameterValue(Enumerations::mu, Enumerations::val)) + 1 )  / (dst->pars->getParameterValue(Enumerations::beta, Enumerations::val))) << *sep << dst->parsMeaning->at(28).meaning << "\n";
            /* Lc    */ (*out) << dst->parsMeaning->at(29).symbol << *sep << ((dst->pars->getParameterValue(Enumerations::a3, Enumerations::val) * (dst->pars->getParameterValue(Enumerations::mu, Enumerations::val) + 1.)) / dst->pars->getParameterValue(Enumerations::beta, Enumerations::val)) << *sep << dst->parsMeaning->at(29).meaning << "\n";
            /* kapc  */ (*out) << dst->parsMeaning->at(30).symbol << *sep << (1./(dst->pars->getParameterValue(Enumerations::mu, Enumerations::val))) << *sep << dst->parsMeaning->at(30).meaning << "\n";
            /* a3min */ (*out) << dst->parsMeaning->at(31).symbol << *sep << ((dst->pars->getParameterValue(Enumerations::a3, Enumerations::val))  - (dst->pars->getParameterValue(Enumerations::da3, Enumerations::val))) << *sep << dst->parsMeaning->at(31).meaning << "\n";
            /* eps3  */ (*out) << dst->parsMeaning->at(32).symbol << *sep << ((dst->pars->getParameterValue(Enumerations::da3, Enumerations::val))  / ( (dst->pars->getParameterValue(Enumerations::a3, Enumerations::val)) - (dst->pars->getParameterValue(Enumerations::da3, Enumerations::val)))) << *sep << dst->parsMeaning->at(32).meaning << "\n";
            /* kapr  */ (*out) << dst->parsMeaning->at(33).symbol << *sep << ( (3 * pow(M_PI, 2) / 32) * ( (1./(dst->pars->getParameterValue(Enumerations::nu, Enumerations::val))) + 1) - 1) << *sep << dst->parsMeaning->at(33).meaning << "\n";
            out->flush();
            file->flush();
            file->close();
            delete sep;
            delete out;
            delete file;
        }
        else{
            delete file;
            AuxiliaryMethods::errorMessage(this, new QString(tr("Speichern der aktuellen Anpassungskurve als X-Y-Datei fehlgeschlagen")));
        }
    }
    delete dlg;
}


//screenshot of plot (plotWidget)
void MainWindow::on_buttonSavePlotImage_clicked(){
    QPixmap *plotScreenshot = new QPixmap(ui->plot->size());
    ui->plot->render(plotScreenshot);
    if ( !(dst->saveImageFile(plotScreenshot, new QString(tr("aktuellen Plot als Bild speichern")), sst->prgset->defaultDir, this)) )
         AuxiliaryMethods::errorMessage(this, new QString(tr("Speichern des aktuellen Plots als Bild fehlgeschlagen")));
    delete plotScreenshot;
}


void MainWindow::on_buttonSavePlotDataXY_clicked(){
    if (sst->drawData){
        if ( (dst->saveXYFile(Enumerations::DataCurve, sst->prgset->defaultDir, this)) == Enumerations::OtherError )
            AuxiliaryMethods::errorMessage(this, new QString(tr("Speichern der aktuellen Messkurve als X-Y-Datei fehlgeschlagen")));
    }
    else
        AuxiliaryMethods::errorMessage(this, new QString(tr("Es wird keine Messkurve angezeigt.\nDiese Funktion kann nur bei sichtbarer\nMesskurve genutzt werden.")));
}


void MainWindow::on_buttonSavePlotFitXY_clicked(){
    if (sst->drawFit){
        if ( (dst->saveXYFile(Enumerations::FitCurve, sst->prgset->defaultDir, this)) == Enumerations::OtherError )
            AuxiliaryMethods::errorMessage(this, new QString(tr("Speichern der aktuellen Anpassungskurve als X-Y-Datei fehlgeschlagen")));
    }
    else
        AuxiliaryMethods::errorMessage(this, new QString(tr("Es wird keine Anpassungskurve angezeigt.\nDiese Funktion kann nur bei sichtbarer\nAnpassungskurve genutzt werden.")));
}


void MainWindow::on_plot_minX_editingFinished(){
    sst->mustSave = true;
    QString *tempText = new QString(ui->plot_minX->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if ( (temp < dst->prgpar->plotMaxX) && (temp != dst->prgpar->plotMinX) ){
            if (temp < 0.01)
                dst->prgpar->plotMinX = 0.01;
            else
               dst->prgpar-> plotMinX = temp;
            ui->plot->setAxisScale(QwtPlot::xBottom, dst->prgpar->plotMinX, dst->prgpar->plotMaxX);
            draw(true);
            dev->updateBorders();
        }
    }
    ui->plot_minX->setText(QString::number(dst->prgpar->plotMinX,'f',5));
    delete tempText;
}


void MainWindow::on_plot_maxX_editingFinished(){
    sst->mustSave = true;
    QString *tempText = new QString(ui->plot_maxX->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if ( (temp > dst->prgpar->plotMinX) && (temp != dst->prgpar->plotMaxX) ){
            if (temp > (floor(( (2 * sin(70 * M_PI / 180)) / (dst->prgpar->currentWavelength->value) ) * 10.) / 10.))
                dst->prgpar->plotMaxX = (floor(( (2 * sin(70 * M_PI / 180)) / (dst->prgpar->currentWavelength->value) ) * 10.) / 10.);
            else
                dst->prgpar->plotMaxX = temp;
            ui->plot->setAxisScale(QwtPlot::xBottom, dst->prgpar->plotMinX, dst->prgpar->plotMaxX);
            dev->updateBorders();
            draw(true);
        }
    }
    ui->plot_maxX->setText(QString::number(dst->prgpar->plotMaxX,'f',5));
    delete tempText;
}


void MainWindow::on_plot_minY_editingFinished(){
    sst->mustSave = true;
    QString *tempText = new QString(ui->plot_minY->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 3);
        if ( (temp < dst->prgpar->plotMaxY) && (temp != dst->prgpar->plotMinY) ){
            if (temp < 0)
                dst->prgpar->plotMinY = 0;
            else
                dst->prgpar->plotMinY = temp;
            ui->checkBox_Autoscale->setChecked(false);
            dst->prgpar->autoScaleYAxis = false;
            ui->plot->setAxisAutoScale(QwtPlot::yLeft,false);
            ui->plot->setAxisScale(QwtPlot::yLeft, dst->prgpar->plotMinY, dst->prgpar->plotMaxY);
            draw(true);
        }
    }
    ui->plot_minY->setText(QString::number(dst->prgpar->plotMinY,'f',3));
    delete tempText;
}


void MainWindow::on_plot_maxY_editingFinished(){
    sst->mustSave = true;
    QString *tempText = new QString(ui->plot_maxY->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 3);
        if ( (temp > dst->prgpar->plotMinY) && (temp != dst->prgpar->plotMaxY) ){
            dst->prgpar->plotMaxY = temp;
            ui->checkBox_Autoscale->setChecked(false);
            dst->prgpar->autoScaleYAxis = false;
            ui->plot->setAxisAutoScale(QwtPlot::yLeft,false);
            ui->plot->setAxisScale(QwtPlot::yLeft, dst->prgpar->plotMinY, dst->prgpar->plotMaxY);
            draw(true);
        }
    }
    ui->plot_maxY->setText(QString::number(dst->prgpar->plotMaxY,'f',3));
    delete tempText;
}


void MainWindow::on_checkBox_Autoscale_clicked(bool checked){
    sst->mustSave = true;
    dst->prgpar->autoScaleYAxis = checked;
    ui->plot->setAxisAutoScale(QwtPlot::yLeft, checked);
    if (checked){
        ui->plot->replot();
        dst->prgpar->plotMinY = ui->plot->axisInterval(QwtPlot::yLeft).minValue();
        dst->prgpar->plotMaxY = ui->plot->axisInterval(QwtPlot::yLeft).maxValue();
        ui->plot_minY->setText(QString::number(dst->prgpar->plotMinY,'f',3));
        ui->plot_maxY->setText(QString::number(dst->prgpar->plotMaxY,'f',3));
    }
}


void MainWindow::on_spinBoxThinningFactor_valueChanged(int value){
    sst->mustSave = true;
    dst->prgpar->thinningFactor = value;
    draw(false);
}


void MainWindow::on_buttonUpdateFit_clicked(){
    sst->prgset->autoUpdateFit = true; //temporary so that calling draw() updates the plot
    draw(true);
    sst->prgset->autoUpdateFit = false; //no auto update afterwards
}


void MainWindow::on_r_editingFinished(){
    bool prevState = ui->r->blockSignals(true); //save previous state
    sst->mustSave = true;
    QString *tempText = new QString(ui->r->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 4);
        if (!(temp < 0)){
            dst->prgpar->par_r = temp;
        }
    }
    ui->r->setText(QString::number(dst->prgpar->par_r,'f',4));
    draw(true);
    delete tempText;
    ui->r->blockSignals(prevState); //restore previous state
}


void MainWindow::on_d_editingFinished(){
    bool prevState = ui->d->blockSignals(true); //save previous state
    sst->mustSave = true;
    QString *tempText = new QString(ui->d->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 4);
        if (!(temp < 0)){
            dst->prgpar->par_d = temp;
        }
    }
    ui->d->setText(QString::number(dst->prgpar->par_d,'f',4));
    draw(true);
    delete tempText;
    ui->d->blockSignals(prevState); //restore previous state
}


void MainWindow::on_b_editingFinished(){
    bool prevState = ui->b->blockSignals(true); //save previous state
    sst->mustSave = true;
    QString *tempText = new QString(ui->b->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (!(temp < 0)){
            dst->prgpar->par_b = temp;
        }
    }
    ui->b->setText(QString::number(dst->prgpar->par_b,'f',5));
    draw(true);
    delete tempText;
    ui->b->blockSignals(prevState); //restore previous state
}


void MainWindow::on_divergenceAngle_editingFinished(){
    bool prevState = ui->divergenceAngle->blockSignals(true); //save previous state
    sst->mustSave = true;
    QString *tempText = new QString(ui->divergenceAngle->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 3);
        if (!(temp <= 0 || temp >= 90)){
            dst->prgpar->par_delta = temp;
        }
    }
    ui->divergenceAngle->setText(QString::number(dst->prgpar->par_delta,'f',3));
    draw(true);
    delete tempText;
    ui->divergenceAngle->blockSignals(prevState); //restore previous state
}


void MainWindow::on_irradiatedLength_editingFinished(){
    bool prevState = ui->irradiatedLength->blockSignals(true); //save previous state
    sst->mustSave = true;
    QString *tempText = new QString(ui->irradiatedLength->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 4);
        if (!(temp < 0)){
            dst->prgpar->par_l = temp;
        }
    }
    ui->irradiatedLength->setText(QString::number(dst->prgpar->par_l,'f',4));
    draw(true);
    delete tempText;
    ui->irradiatedLength->blockSignals(prevState); //restore previous state
}


void MainWindow::on_checkBox_optQ_clicked(bool checked){
    dst->prgpar->useQ = checked;
    setEnabledDisabled();
    sst->mustSave = true;
    draw(true);
}


void MainWindow::on_checkBox_optA_clicked(bool checked){
    dst->prgpar->useA = checked;
    if (dst->prgpar->useA){
        if (ui->checkBox_optIntensity->isChecked()){
            sst->reCheck_optIntensity_MainWindow = true;
            dst->prgpar->corrIntensity = false;
            ui->checkBox_optIntensity->setChecked(dst->prgpar->corrIntensity);

        }
        else
            sst->reCheck_optIntensity_MainWindow = false;
    }
    else{
        if (sst->reCheck_optIntensity_MainWindow){
            dst->prgpar->corrIntensity = true;
            ui->checkBox_optIntensity->setChecked(dst->prgpar->corrIntensity);
            sst->reCheck_optA_MainWindow = false;
        }
    }
    setEnabledDisabled();
    sst->mustSave = true;
    draw(true);
}


void MainWindow::on_checkBox_optGradient_clicked(bool checked){
    dst->prgpar->useGradient = checked;
    setEnabledDisabled();
    sst->mustSave = true;
    draw(true);
}


void MainWindow::on_checkBox_optCorrAutoColl_clicked(bool checked){
    dst->prgpar->useCorrAutoColl = checked;
    setEnabledDisabled();
    sst->mustSave = true;
    draw(true);
}


void MainWindow::on_checkBox_optPosition_clicked(bool checked){
    dst->prgpar->corrPosition = checked;
    setEnabledDisabled();
    sst->mustSave = true;
    draw(true);
}


void MainWindow::on_comboBox_Position_currentIndexChanged(int index){
    dst->prgpar->corrPositionMethod = Enumerations::corrPositionMethod(index);
    setEnabledDisabled();
    sst->mustSave = true;
    draw(true);
}


void MainWindow::on_checkBox_optIntensity_clicked(bool checked){
    dst->prgpar->corrIntensity = checked;
    if (dst->prgpar->corrIntensity){
        if (ui->checkBox_optA->isChecked()){
            sst->reCheck_optA_MainWindow = true;
            dst->prgpar->useA = false;
            ui->checkBox_optA->setChecked(false);
        }
        else
            sst->reCheck_optA_MainWindow = false;
    }
    else{
        if (sst->reCheck_optA_MainWindow){
            dst->prgpar->useA = true;
            ui->checkBox_optA->setChecked(dst->prgpar->useA);
            sst->reCheck_optIntensity_MainWindow = false;
        }
    }
    setEnabledDisabled();
    sst->mustSave = true;
    draw(true);
}


void MainWindow::on_comboBox_Intensity_currentIndexChanged(int index){
    dst->prgpar->corrIntensityMethod = Enumerations::corrIntensityMethod(index);
    setEnabledDisabled();
    sst->mustSave = true;
    draw(true);
}


void MainWindow::on_comboBox_Wavelengths_editTextChanged(const QString &arg1){
        wavelength *tempWavelength = AuxiliaryMethods::validateAndParse_Wavelength(new QString(arg1));
        if (tempWavelength != 0){
            delete dst->prgpar->currentWavelength;
            dst->prgpar->currentWavelength = tempWavelength;
            dst->prgpar->plotMinX = 0.01;
            dst->prgpar->plotMaxX = (floor(( (2 * sin(70 * M_PI / 180)) / (dst->prgpar->currentWavelength->value) ) * 10.) / 10.);
            ui->plot_minX->setText(QString::number(dst->prgpar->plotMinX,'f',5));
            ui->plot_maxX->setText(QString::number(dst->prgpar->plotMaxX,'f',5));
            ui->plot->setAxisScale(QwtPlot::xBottom, dst->prgpar->plotMinX, dst->prgpar->plotMaxX);
            draw(true);
        }
}


void MainWindow::on_buttonInsertWavelength_clicked(){
    wavelength *toAdd = AuxiliaryMethods::validateAndParse_Wavelength(new QString(ui->comboBox_Wavelengths->currentText()));
    if (toAdd != 0){
        int pos = AuxiliaryMethods::findWavelength(toAdd, dst->wavelengths);
        if (pos == -1){
            dst->wavelengths->append(*toAdd);
            int index = dst->wavelengths->size() - 1;
            ui->comboBox_Wavelengths->addItem( ( (dst->wavelengths->at(index).name.length() > 0) ? (dst->wavelengths->at(index).name + " - ") : "" ) + QString::number(dst->wavelengths->at(index).value,'g',10));
            ui->comboBox_Wavelengths->setCurrentIndex(index);
            ui->buttonDeleteWavelength->setEnabled(true);
        }
        else
            AuxiliaryMethods::errorMessage(this, new QString(tr("Die hinzuzufuegende Wellenlaenge\nist bereits vorhanden!")));
    }
    else
        AuxiliaryMethods::errorMessage(this, new QString(tr("Ungueltiges Eingabeformat!\nDie Wellenlaenge wurde\nnicht hinzugefuegt.")));
}


void MainWindow::on_buttonSaveWavelength_clicked(){
    wavelength *toSave = AuxiliaryMethods::validateAndParse_Wavelength(new QString(ui->comboBox_Wavelengths->currentText()));
    if (toSave != 0){
        int pos =  AuxiliaryMethods::findWavelength(toSave, dst->wavelengths);
        if (pos == -1){
            int index = ui->comboBox_Wavelengths->currentIndex();
            dst->wavelengths->replace(index, *toSave);
            update__comboBox_Wavelengths(index);
        }
        else
            AuxiliaryMethods::errorMessage(this, new QString(tr("Die zu speichernde Wellenlaenge\nist bereits vorhanden!")));
    }
    else
        AuxiliaryMethods::errorMessage(this, new QString(tr("Ungueltiges Eingabeformat!\nDie Wellenlaenge wurde\nnicht gespeichert.")));
}


void MainWindow::on_buttonDeleteWavelength_clicked(){
    int index = ui->comboBox_Wavelengths->currentIndex();
    dst->wavelengths->remove(index);
    ui->comboBox_Wavelengths->removeItem(index);
    if (dst->wavelengths->size() <= 1)
        ui->buttonDeleteWavelength->setDisabled(true);
}
